/**
 ##############################################################################
 # Copyright (C) 2006-2010 INRIA
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 2 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ##############################################################################
 * 
 */

package fr.inria.icocoa.bpelgenerator;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.lang.reflect.Field;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.jar.JarEntry;
import java.util.jar.JarException;
import java.util.jar.JarOutputStream;

import javax.xml.namespace.QName;

import org.xml.sax.SAXException;

import fr.inria.icocoa.common.ConfigManager;
import fr.inria.icocoa.common.DebugPrinter;
import fr.inria.icocoa.common.MeasurementTool;
import fr.inria.icocoa.common.PropertyManager;
import fr.inria.icocoa.common.Tools;
import fr.inria.icocoa.exception.ParsingException;
import fr.inria.icocoa.semrep.SemanticRepositoryFactory;
import fr.inria.icocoa.semrep.SemanticRepositoryForCompositionEngine;
import fr.inria.icocoa.servicediscovery.SDResult;
import fr.inria.icocoa.servicediscovery.ServiceDiscovery;
import fr.inria.icocoa.servicemodel.Capability;
import fr.inria.icocoa.servicemodel.ControlConstruct;
import fr.inria.icocoa.servicemodel.IO;
import fr.inria.icocoa.servicemodel.Input;
import fr.inria.icocoa.servicemodel.Output;
import fr.inria.icocoa.servicemodel.ProcessInterface;
import fr.inria.icocoa.servicemodel.Service;
import fr.inria.icocoa.servicemodel.UserTask;
import fr.inria.icocoa.servicemodel.parsers.jenaparser.WSDLParser;
import fr.inria.icocoa.servicemodel.toolsobjects.ServiceMap;

/**
 * BPEL generator
 * 
 * Reads the process SDL and includes the real services' conversations
 * 
 * @author <a href="mailto:Galip-Ferda.Tartanoglu@inria.fr">Ferda Tartanoglu</a> -
 *         <a href="mailto:sebastien.bianco@inria.fr">Sebastien Bianco</a> - <a
 *         href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
 * 
 */
public class BpelGenerator {

	/** serviceURI=namespace */
	private Map<String, String> service_namespace = new HashMap<String, String>();

	/** serviceURI=serviceName */
	private Map<String, String> service_name = new HashMap<String, String>();

	/** namespace=messages */
	private Map<String, List<String>> service_message = new HashMap<String, List<String>>();

	/** servicePortType=namespace */
	private Map<String, String> name_namespace = new HashMap<String, String>();

	/** serviceURI=PortType */
	private Map<String, String> service_port = new HashMap<String, String>();

	/** serviceURI=real_WSDL_uri */
	private Map<String, String> service_uri = new HashMap<String, String>();

	/** serviceURI=real_URL odf service */
	private Map<String, String> service_soapAddress = new HashMap<String, String>();

	/** serviceURI=portType */
	private Map<String, String> service_portType = new HashMap<String, String>();

	/** list of all the I/O messages */
	private List<String> messages = new ArrayList<String>();

	/** URI of the wsdlFile */
	private String wsdlURI = null;

	/**
	 * The property manager used by this class
	 */
	private ConfigManager configManager = ConfigManager.getInstance();

	/**
	 * The result of the service discovery algorithm
	 */
	private SDResult sdResult = null;

	/**
	 * 
	 * BpelGenerator.java
	 * 
	 * This class generate and manage the global file structure needed for BPEL
	 * script
	 * 
	 * @author <a href="mailto:sebastien.bianco@inria.fr">sebastien bianco</a> -
	 *         <a href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
	 * 
	 * Creation Date : Jan 2, 2007 9:21:42 AM
	 */
	private class BpelFilesStructure {

		/**
		 * The path of the file containing the script
		 */
		private String bpelFile;

		/**
		 * the path of the file containing the WSDL part of the script
		 */
		private String fileProcessWSDL;

		/**
		 * the path of the repository containing all wsdl files
		 */
		private String bpelWSDLRepository;

		/**
		 * the path of the pdd file of the bpel script
		 */
		private String pddFile;

		/**
		 * the file containing all reference for WSDL part
		 */
		private String wsdlCatalog;

		/**
		 * The path were all Files will be geneareted
		 */
		private String basePath;

		/**
		 * the file Separator of the system
		 */
		private String fileSeparator;

		/**
		 * The name of the repository that will be generated
		 */
		private String repositoryName;

		/**
		 * the base folder that contains all the files that has to be "jared"
		 */
		private String toJarDirectory;

		/**
		 * the name of the file containing the data of the bpel script
		 */
		private String jarFileName;

		/**
		 * Constrcutor of the class. Create all files and repository needed
		 * 
		 * @param basePath :
		 *            the path were all subdirectory will be created
		 * @param repositoryName :
		 *            the name of the first level subdirectory (corresponding to
		 *            the base respositoryname of the script)
		 * @param fileSeparator :
		 *            the File separator of the system
		 */
		public BpelFilesStructure(String basePath, String repositoryName,
				String fileSeparator) {

			/*
			 * create directories
			 */
			String mainDir = basePath + fileSeparator + repositoryName;
			File mainDirFile = new File(mainDir);
			mainDirFile.mkdir();

			File dir_bpel = new File(mainDir + fileSeparator + "bpel"
					+ fileSeparator);

			dir_bpel.mkdir();

			File dir_wsdl = new File(mainDir + fileSeparator + "wsdl"
					+ fileSeparator);
			dir_wsdl.mkdir();

			File dir_meta = new File(mainDir + fileSeparator + "META-INF"
					+ fileSeparator);
			dir_meta.mkdir();

			/*
			 * create file path
			 */
			this.bpelFile = dir_bpel.getAbsolutePath() + fileSeparator
					+ repositoryName + ".bpel";
			this.fileProcessWSDL = dir_wsdl.getAbsolutePath() + fileSeparator
					+ repositoryName + ".wsdl";
			this.bpelWSDLRepository = dir_wsdl.getAbsolutePath();
			this.pddFile = mainDir + fileSeparator + repositoryName + ".pdd";
			this.wsdlCatalog = dir_meta.getAbsolutePath() + fileSeparator
					+ "wsdlCatalog.xml";
			this.basePath = basePath;
			this.fileSeparator = fileSeparator;
			this.repositoryName = repositoryName;
			this.toJarDirectory = mainDir;
			this.jarFileName = repositoryName + ".bpr";

		}

		/**
		 * this method generate the Jar File containig all information for the
		 * BPEL Engine
		 * 
		 * @return the AbsolutPath of the Jar File
		 * @throws GenerationException
		 *             this exception is thrown if an error occurs during the
		 *             generation process (here in case of JarException)
		 */
		public String generateJarFile() throws GenerationException {
			String fileName = this.basePath + this.fileSeparator
					+ this.jarFileName;
			try {
				FolderJarer.jarFolder(this.toJarDirectory, fileName);
			} catch (JarException e) {
				DebugPrinter.print(e, DebugPrinter.ERROR);
				throw new GenerationException();
			}
			return fileName;
		}

		/**
		 * @return the bpelFile
		 */
		public String getBpelFile() {
			return bpelFile;
		}

		/**
		 * @return the fileProcessWSDL
		 */
		public String getFileProcessWSDL() {
			return fileProcessWSDL;
		}

		/**
		 * @return the bpelWSDLRepository
		 */
		public String getBpelWSDLRepository() {
			return bpelWSDLRepository;
		}

		/**
		 * @return the pddFile
		 */
		public String getPddFile() {
			return pddFile;
		}

		/**
		 * @return the wsdlCatalog
		 */
		public String getWsdlCatalog() {
			return wsdlCatalog;
		}

		/**
		 * @return the repositoryName
		 */
		public String getRepositoryName() {
			return repositoryName;
		}

		/**
		 * @return the jarFileName
		 */
		public String getJarFileName() {
			return jarFileName;
		}

		public String toString() {
			Class c = this.getClass();
			Field[] f = c.getDeclaredFields();
			String result = "BpelFilesStructure : (size = " + f.length + ")";
			for (int i = 0; i < f.length; i++) {
				try {
					result += "\n\t> " + f[i].getName() + " = "
							+ f[i].get(this);
				} catch (IllegalArgumentException e) {
					DebugPrinter.print(e, DebugPrinter.WARN);
				} catch (IllegalAccessException e) {
					DebugPrinter.print(e, DebugPrinter.WARN);
				}
			}
			return result;
		}

	}

	/* ********************* */
	/* * Getter and Setter * */
	/* ********************* */

	public BpelGenerator(SDResult sdResult) {
		this.sdResult = sdResult;
	}

	public BpelGenerator() {
		// TODO Auto-generated constructor stub
	}

	/**
	 * @return the name_namespace
	 */
	public Map<String, String> getName_namespace() {
		return name_namespace;
	}

	/**
	 * @return the service_message
	 */
	public Map<String, List<String>> getService_message() {
		return service_message;
	}

	/**
	 * @return the service_name
	 */
	public Map<String, String> getService_name() {
		return service_name;
	}

	/**
	 * @return the service_namespace
	 */
	public Map<String, String> getService_namespace() {
		return service_namespace;
	}

	/**
	 * @return the service_port
	 */
	public Map<String, String> getService_port() {
		return service_port;
	}

	/**
	 * @return the service_uri
	 */
	public Map<String, String> getService_uri() {
		return service_uri;
	}

	/**
	 * @return the service_portType
	 */
	public Map<String, String> getService_portType() {
		return service_portType;
	}

	/**
	 * @return the service_soapAddress
	 */
	public Map<String, String> getService_soapAddress() {
		return service_soapAddress;
	}

	/* ********************** */
	/* * Generation methods * */
	/* ********************** */

	/**
	 * Generate the complete BpelProcess (creation of fileStructure, generation
	 * of file, compression in a jar file) and deploye it on the server
	 * 
	 * @param processURI :
	 *            the URI of the generated process
	 * @param matches :
	 *            the list of the selected operation that have to be composed
	 * @param sd.getServicesMap() :
	 *            the list of services containing the operations
	 * @return the list of URL of the generated Service
	 * @throws SAXException
	 *             if a problem occurs during the parsing of a WSDLFile
	 * @throws IOException
	 *             if an error occurs during an IO access
	 * @throws GenerationException
	 *             if an error occurs during the Generation
	 * @throws ParsingException
	 *             if an error occurs during the parsing of a file
	 */
	public List<String> generateBpelProcess(UserTask userTask,
			Map<String, String> matches, ServiceDiscovery sd)
			throws IOException, SAXException, GenerationException,
			ParsingException {

		long initGenerationTime = System.currentTimeMillis();
		
		List<String> result = new ArrayList<String>();

		// fill the different map required to generate code
		this.fillTheMapsFromServices(sd.getServicesMap());

		// list of requiredTask
		List<Capability> listOfRequiredCapability = userTask.getRequiredCapabilities();

		// Parse process for getting services
		Map<String, fr.inria.icocoa.servicemodel.Service> servicesParsed = new HashMap<String, fr.inria.icocoa.servicemodel.Service>();
		SemanticRepositoryForCompositionEngine rep = SemanticRepositoryFactory.getInstanceCE();
		for (Iterator<Map.Entry<String, ServiceMap>> it = sd.getServicesMap().entrySet().iterator(); it.hasNext();) {
			Map.Entry<String, ServiceMap> entry = it.next();

			// parsing of OWL-S File !!!!
			servicesParsed.put(entry.getKey(),
				rep.getServiceObjectFromBaseURL(entry.getKey()));
		}

		initGenerationTime = System.currentTimeMillis() - initGenerationTime;
		MeasurementTool.displayTime("Initialisation of BPEL Generation : ", initGenerationTime);
		
		
		long fileGenerationTime = System.currentTimeMillis();
		
		// for each required, we need to :
		// - create file structure
		// - create bpel script and other files
		// - deploy processes
		for (Iterator<Capability> requiredIt = listOfRequiredCapability.iterator(); requiredIt.hasNext();) {
			Capability required = requiredIt.next();
			try {

				/*
				 * change URIfragment to permit to be coherent in case of
				 * multiple required task
				 */
				String requiredURI = required.getCapabilityURI();

				/*
				 * DebugPrinter.print("requiredURI=" + requiredURI,
				 * DebugPrinter.INFO);
				 */

				// get the fragment and modify it
				String[] splited = requiredURI.split("#");

				// update the URI
				String processName = userTask.getLocalName() + "_"
						+ required.getTypeName();
				URI nameURI = new URI(splited[0] + "#" + processName);

				/*
				 * create directories
				 */
				BpelFilesStructure bfs = new BpelFilesStructure(
						this.configManager.getBpelFolder(),
						nameURI.getFragment(), ConfigManager.FILE_SEPARATOR);

				/*
				 * Generating bpel script (need to be done first to initialise
				 * variables
				 */
				this.generateBPELFile(bfs.getBpelFile(), nameURI, sd, matches,
					(ProcessInterface) userTask, required, servicesParsed);

				/*
				 * Generate .pdd
				 */
				this.generatePDDFile(bfs.getPddFile(), nameURI,
					sd.getServicesMap());

				/*
				 * generate wsdlCatalog
				 */
				this.generateCatalogFile(bfs.getWsdlCatalog(), nameURI,
					sd.getServicesMap());

				/*
				 * generate WSDL Files
				 */
				String wsdl = this.generateWSDLFiles(bfs, nameURI,
					(ProcessInterface) userTask, sd.getServicesMap(), required);

				DebugPrinter.print("All files generated in "
						+ this.configManager.getBpelFolder(), 1);

				fileGenerationTime = System.currentTimeMillis() - fileGenerationTime;
				MeasurementTool.displayTime("BEPL File Generation : ", fileGenerationTime);
				
				// Generate the jar File
				long jarGenerationTime = System.currentTimeMillis();
				String out = bfs.generateJarFile();
				jarGenerationTime = System.currentTimeMillis() - jarGenerationTime;
				MeasurementTool.displayTime("Jar File Generation : ", jarGenerationTime);

				DebugPrinter.print("Jar file generated in " + out, 1);

				// Deploy the generate File
				long sendToServerTime = System.currentTimeMillis();
				this.sendFileToBpelServer(out, bfs.getJarFileName());
				sendToServerTime = System.currentTimeMillis() - sendToServerTime;
				MeasurementTool.displayTime("Send to server : ", sendToServerTime);

				DebugPrinter.print(
					"Jar file deployed on "
							+ this.configManager.getBpelDeploymentServiceAddress(),
					1);

				// Get the `real' wsdl from the deployed BPEL service
				String bpelWsdl = configManager.getGenerated_bpel_services_base_address()
						+ "/" + processName + "?wsdl";
				result.add(bpelWsdl);
			} catch (URISyntaxException e) {
				DebugPrinter.print(e, DebugPrinter.ERROR);
				throw new GenerationException("the String "
						+ required.getCapabilityURI() + " is not a valid URI.");
			}

		}

		this.clearAll();

		return result;
	}

	/**
	 * Generate all WSDL Files
	 * 
	 * @param bfs
	 *            the file structure used to stored all generared file
	 * @param myProcess :
	 *            the current process to convert into a script
	 * @param nameURI :
	 *            the URI of the generated BPEL script
	 * @param services :
	 *            the list of services containing the operations
	 * @param currentCapability :
	 *            the name of the requiredCapability currently treated
	 * @return the WSDL file content of the Generated Service
	 * @throws URISyntaxException
	 *             if an error occurs during parsing of an URI
	 * @throws IOException
	 *             if an error occurs during an IO access
	 */
	private String generateWSDLFiles(BpelFilesStructure bfs, URI nameURI,
			ProcessInterface myProcess, Map<String, ServiceMap> services,
			Capability currentCapability) throws URISyntaxException,
			IOException {
		// Generate WSDL associated to the bpel script
		String wsdl = this.generateBpelProcessWSDLFile(
			bfs.getFileProcessWSDL(), nameURI, myProcess, currentCapability);

		// Generated WSDL file from all seelected services
		this.generateWSDLFromOriginal(bfs.getBpelWSDLRepository(), services);

		return wsdl;
	}

	/**
	 * This method copy the WSDL of initial service, but add a new Partner Link
	 * Type to each of them and the corresponding namespace
	 * 
	 * @param services :
	 *            the list of services containing the operations
	 * @param folder :
	 *            the folder where will be stored the genenatyed files
	 * @throws IOException
	 *             if an error occurs during an IO access
	 */
	private void generateWSDLFromOriginal(String folder,
			Map<String, ServiceMap> services) throws IOException {
		// hashmap where will be stored generated file before storing them
		HashMap<String, String> str_service_wsdl = new HashMap<String, String>();

		/*
		 * We do it for each selected Services
		 */
		for (Iterator<Map.Entry<String, ServiceMap>> i = services.entrySet().iterator(); i.hasNext();) {

			Map.Entry<String, ServiceMap> entry = i.next();

			// get the initial WSDL
			String wsdl = entry.getValue().getWsdl();

			String new_wsdl = "";

			// get each line one after the other
			String[] lines = wsdl.split("\n");

			// if all the wsdl file in on a single line (correspond to axis
			// reply)
			if (lines.length == 1) {
				wsdl = wsdl.replaceAll(">", ">\n");
				lines = wsdl.split("\n");
			}

			for (int k = 0; k < lines.length; k++) {

				// DebugPrinter.print("wsdline = "+lines[k], DebugPrinter.INFO);

				// add new namespace declaration
				if (lines[k].contains("<wsdl:definitions")) {

					new_wsdl += lines[k].replace(
						"<wsdl:definitions",
						"<wsdl:definitions \n\txmlns:plnk=\"http://schemas.xmlsoap.org/ws/2003/05/partner-link/\"\n\txmlns:mlkjhg=\""
								+ service_uri.get(entry.getKey()) + "\"\n")
							+ "\n";

				}
				// add the declaration of the parnter link type
				else if (lines[k].contains("</wsdl:definitions>")) {
					String serviceName = service_name.get(entry.getKey());

					new_wsdl += "<plnk:partnerLinkType name=\"" + serviceName
							+ "LinkType\">\n";
					new_wsdl += "  <plnk:role name=\"" + serviceName
							+ "Role\">\n";
					new_wsdl += "    <plnk:portType name=\"mlkjhg:"
							+ service_portType.get(entry.getKey()) + "\"/>\n";
					new_wsdl += "  </plnk:role>\n";
					new_wsdl += "</plnk:partnerLinkType>\n\n";

					new_wsdl += "</wsdl:definitions>\n";
				}
				// correct used namespace
				else if (lines[k].contains("soapenc:")) {
					new_wsdl += lines[k].replace("soapenc:", "xsd:") + "\n";
				}
				// if nothing to change in the line, just copy it
				else {
					new_wsdl += lines[k] + "\n";
				}
			}

			// store the generated WSDL File in the map
			str_service_wsdl.put(service_name.get(entry.getKey()), new_wsdl);
		}

		/*
		 * Real storage
		 */
		for (Iterator<Map.Entry<String, String>> it = str_service_wsdl.entrySet().iterator(); it.hasNext();) {
			Map.Entry<String, String> mapEntry = it.next();
			String port = mapEntry.getKey();
			String file_service_wsdl = folder + ConfigManager.FILE_SEPARATOR
					+ port + ".wsdl";
			writeFile(mapEntry.getValue(), file_service_wsdl);
		}
	}

	/**
	 * Generate the code of the WSDL file associated to the BPEL Script (which
	 * is a WS itself)
	 * 
	 * @param file :
	 *            the path of the file where will be stored the generated
	 *            content
	 * @param myProcess :
	 *            the current process to convert into a script
	 * @param nameURI :
	 *            the URI of the generated BPEL script
	 * @param currentCapability :
	 *            the name of the requiredCapability currently treated
	 * @return the WSDL file content of the Generated Service
	 * @throws URISyntaxException
	 *             if an error occurs during parsing of an URI
	 * @throws IOException
	 *             if an error occurs during an IO access
	 */
	private String generateBpelProcessWSDLFile(String file, URI nameURI,
			ProcessInterface myProcess, Capability currentCapability)
			throws URISyntaxException, IOException {

		// initialisation of local variables
		Map<String, Input> inputMessages = currentCapability.getInputs();
		Map<String, Output> outputMessages = currentCapability.getOutputs();

		/*
		 * Generating bpel process wsdl
		 */
		String str_process_wsdl = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n";
		str_process_wsdl += "<wsdl:definitions\n";
		str_process_wsdl += "\ttargetNamespace=\"" + this.wsdlURI + "\"\n";
		str_process_wsdl += "\txmlns:tns=\"" + this.wsdlURI + "\"\n";
		/*
		 * str_process_wsdl += "\ttargetNamespace=\"" + this.wsdlURI + "/" +
		 * nameURI.getFragment() + ".wsdl\"\n"; str_process_wsdl +=
		 * "\txmlns:tns=\"" + this.wsdlURI + "/" + nameURI.getFragment() +
		 * ".wsdl\"\n";
		 */
		str_process_wsdl += "\txmlns:soapenc=\"http://schemas.xmlsoap.org/soap/encoding/\"\n";
		str_process_wsdl += "\txmlns:plnk=\"http://schemas.xmlsoap.org/ws/2003/05/partner-link/\"\n";
		str_process_wsdl += "\txmlns:xsd=\"http://www.w3.org/2001/XMLSchema\"\n";
		str_process_wsdl += "\txmlns:soap=\"http://schemas.xmlsoap.org/wsdl/soap/\"\n";
		str_process_wsdl += "\txmlns:wsdl=\"http://schemas.xmlsoap.org/wsdl/\" >\n\n";

		// a list to store all messages URI (Input/Output)
		// List<URI> messagesURI = new ArrayList<URI>();

		/* Comment */
		str_process_wsdl += "\t\t<!-- ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ -->\n";
		str_process_wsdl += "\t\t<!-- PORT TYPE DEFINITION - A port type groups a set of operations into    -->\n";
		str_process_wsdl += "\t\t<!-- a logical service unit.                                               -->\n";
		str_process_wsdl += "\t\t<!-- ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ -->\n\n";
		str_process_wsdl += "\t<!-- portType implemented by the BPEL process -->\n";
		/* Generate open-tag for portType */
		str_process_wsdl += "\t<wsdl:portType name=\"" + nameURI.getFragment()
				+ "PortType\">\n";
		str_process_wsdl += "\t\t<wsdl:operation name=\""
				+ currentCapability.getTypeName() + "\">\n";

		/* Generate the Input */
		String message = currentCapability.getTypeName() + "Input";
		str_process_wsdl += "\t\t\t<wsdl:input message=\"tns:" + message
				+ "\"/>\n";

		/* Generate the Output */
		message = currentCapability.getTypeName() + "Output";
		str_process_wsdl += "\t\t\t<wsdl:output message=\"tns:" + message
				+ "\"/>\n";

		/*
		 * URI inputMsg = null; List<Input> il = new ArrayList<Input>(inputMessages.values());
		 * for (int i = 0; i < il.size(); i++) { inputMsg = new
		 * URI(il.get(i).getURI()); str_process_wsdl += "\t\t\t<wsdl:input
		 * message=\"tns:" + inputMsg.getFragment() + "\"/>\n";
		 * messagesURI.add(inputMsg); }
		 */

		/* Generate the list of Outputs */
		/*
		 * str_process_wsdl += "\n\t\t<!-- List of Outputs -->\n"; URI
		 * outputMsg = null; List<Output> ol = new ArrayList<Output>(outputMessages.values());
		 * for (int i = 0; i < ol.size(); i++) { outputMsg = new
		 * URI(ol.get(i).getURI()); str_process_wsdl += "\t\t\t<wsdl:output
		 * message=\"tns:" + outputMsg.getFragment() + "\"/>\n";
		 * messagesURI.add(outputMsg); }
		 */

		/* Close tag for portType */
		str_process_wsdl += "\t\t</wsdl:operation>\n";
		str_process_wsdl += "\t</wsdl:portType>\n\n";

		/* Comment */
		str_process_wsdl += "\t\t<!-- ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ -->\n";
		str_process_wsdl += "\t\t<!-- MESSAGE TYPE DEFINITION - Definition of the message types used as     -->\n";
		str_process_wsdl += "\t\t<!-- part of the port type defintions                                      -->\n";
		str_process_wsdl += "\t\t<!-- ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ -->\n\n";

		/*
		 * Generate the Input message
		 */
		str_process_wsdl += "\t<wsdl:message name=\""
				+ currentCapability.getTypeName() + "Input\">\n";

		// Get all Input
		List<Input> il = new ArrayList<Input>(inputMessages.values());
		for (int i = 0; i < il.size(); i++) {
			str_process_wsdl += "\t\t<wsdl:part name=\""
					+ il.get(i).getTypeName() + "\" type=\"xsd:string\"/>\n";
		}
		str_process_wsdl += "\t</wsdl:message>\n\n";

		/*
		 * Generate the Output message
		 */
		str_process_wsdl += "\t<wsdl:message name=\""
				+ currentCapability.getTypeName() + "Output\">\n";

		// Get all Output
		List<Output> ol = new ArrayList<Output>(outputMessages.values());
		for (int i = 0; i < ol.size(); i++) {
			str_process_wsdl += "\t\t<wsdl:part name=\""
					+ ol.get(i).getTypeName() + "\" type=\"xsd:string\"/>\n";
		}
		str_process_wsdl += "\t</wsdl:message>\n\n";

		/* Comment */
		str_process_wsdl += "\t\t<!-- ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ -->\n";
		str_process_wsdl += "\t\t<!-- PARTNER LINK TYPE DEFINITION                                          -->\n";
		str_process_wsdl += "\t\t<!-- the toto partnerLinkType binds the provider and                       -->\n";
		str_process_wsdl += "\t\t<!-- requester portType into an asynchronous conversation.                 -->\n";
		str_process_wsdl += "\t\t<!-- ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ -->\n\n";

		/* Generate ParnterLink */
		str_process_wsdl += "\t<plnk:partnerLinkType name=\""
				+ nameURI.getFragment() + "LinkType\">\n";
		str_process_wsdl += "\t\t<plnk:role name=\"" + nameURI.getFragment()
				+ "Role\">\n";
		str_process_wsdl += "\t\t\t<plnk:portType name=\"tns:"
				+ nameURI.getFragment() + "PortType\"/>\n";
		str_process_wsdl += "\t\t</plnk:role>\n";
		str_process_wsdl += "\t</plnk:partnerLinkType>\n\n";

		str_process_wsdl += "</wsdl:definitions>\n";

		/*
		 * Write the corresponding File
		 */
		writeFile(str_process_wsdl, file);

		return str_process_wsdl;
	}

	/**
	 * Generate the content of the BPEL script and store it in the given file.
	 * 
	 * @param file :
	 *            the path of the file where will be stored the generated
	 *            content
	 * @param processURI :
	 *            the URI of the generated process
	 * @param matches :
	 *            the list of the selected operation that have to be composed
	 * @param sd.getServicesMap() :
	 *            the list of services containing the operations
	 * @param myProcess :
	 *            the current process to convert into a script
	 * @param currentCapability :
	 *            the name of the requiredCapability currently treated
	 * @param servicesParsed :
	 *            the list of services (parsed) containing the operations
	 * @throws SAXException
	 *             if a problem occurs during the parsing of a WSDLFile
	 * @throws IOException
	 *             if an IO error occurs
	 * @throws URISyntaxException
	 *             if an error occurs during parsing of an URI
	 * @throws ParsingException
	 *             if an error occurs during the parsing of services
	 * @throws GenerationException
	 *             if no grounding is assiciated with one of capa
	 */
	private void generateBPELFile(String file, URI nameURI,
			ServiceDiscovery sd, Map<String, String> matches,
			ProcessInterface myProcess, Capability currentCapability,
			Map<String, Service> servicesParsed) throws IOException,
			SAXException, URISyntaxException, ParsingException,
			GenerationException {

		String str_bpel = "";

		// get the header of the BPEL Script
		str_bpel += this.generateBPELScriptHeader(nameURI, sd.getServicesMap());

		// get the list of partner links
		str_bpel += "\t\t<!-- ================================================================= -->\n";
		str_bpel += "\t\t<!-- PARTNERLINKS                                                      -->\n";
		str_bpel += "\t\t<!-- List of services participating in this BPEL process               -->\n";
		str_bpel += "\t\t<!-- ================================================================= -->\n\n";
		str_bpel += this.generateBPELPartnerLink(nameURI, sd.getServicesMap());

		// get the variables
		str_bpel += "\t\t<!-- ================================================================= -->\n";
		str_bpel += "\t\t<!-- VARIABLES                                                         -->\n";
		str_bpel += "\t\t<!-- List of messages and XML documents used within this BPEL process  -->\n";
		str_bpel += "\t\t<!-- ================================================================= -->\n\n";

		str_bpel += this.generateBPELVariable(currentCapability,
			servicesParsed, matches);

		// get the main process
		str_bpel += "\t\t<!-- ================================================================= -->\n";
		str_bpel += "\t\t<!-- ORCHESTRATION LOGIC                                               -->\n";
		str_bpel += "\t\t<!-- Set of activities coordinating the flow of messages across the    -->\n";
		str_bpel += "\t\t<!-- services integrated within this business process                  -->\n";
		str_bpel += "\t\t<!-- ================================================================= -->\n\n";
		str_bpel += this.generateMainProcess(myProcess, nameURI, sd, matches,
			currentCapability, servicesParsed);

		// get the footer of the file (close opened-tag)
		str_bpel += this.generateBPELFooter();

		// write the content in a file
		writeFile(str_bpel, file);

	}

	/**
	 * Generate the ".bpel" footer
	 * 
	 * @return the footer of the .bpel file
	 */
	private String generateBPELFooter() {
		return "</process>\n";
	}

	/**
	 * This method associate to each services a different namespace
	 * (xmls:service_'i':"value of namespace"), a port type, a list of messages,
	 * an uri, and store the different field
	 * 
	 * @param services :
	 *            the list of services containing the operations
	 * @throws SAXException
	 *             if a problem occurs during the parsing of a WSDLFile
	 * @throws IOException
	 *             if an IO error occurs
	 */
	private void fillTheMapsFromServices(Map<String, ServiceMap> services)
			throws IOException, SAXException {

		// the service number
		int service_num = 1;

		// DebugPrinter.print(new ArrayList(services.keySet()),
		// "new ArrayList(services.keySet())", DebugPrinter.INFO);

		for (Iterator<Map.Entry<String, ServiceMap>> i = services.entrySet().iterator(); i.hasNext();) {
			Map.Entry<String, ServiceMap> entry = i.next();

			// parse WSDL document associated with the service
			long time = System.currentTimeMillis();
			WSDLParser wsdlParser = new WSDLParser(entry.getValue().getWsdl());
			time = System.currentTimeMillis() - time;
			DebugPrinter.print("Parsing of " + entry.getKey()
					+ " (WSDL File) -> " + time + " ms", DebugPrinter.INFO);

			// store the namespace associated with each service
			String namespace = "service_" + service_num++;
			service_namespace.put(entry.getKey(), namespace);

			// store the name of the service
			service_name.put(entry.getKey(), wsdlParser.getMyServiceName());

			// store the list of messages for each service
			service_message.put(namespace, wsdlParser.getMessages());

			// store the list of message for all
			messages.addAll(wsdlParser.getMessages());

			// store the namespace associated with a port type
			name_namespace.put(wsdlParser.getMyPortType(), namespace);

			// store the porttype associated with each service
			service_portType.put(entry.getKey(), wsdlParser.getMyPortType());

			// store the port associated with each service
			service_port.put(entry.getKey(), wsdlParser.getMyPort());

			// store the uri associated with each service
			service_uri.put(entry.getKey(), wsdlParser.getMyuri());

			// store the soapAddress associated with each service
			service_soapAddress.put(entry.getKey(), wsdlParser.getSoapAddress());

		}

		/*
		 * DebugPrinter.print(service_soapAddress, "service_soapAddress",
		 * DebugPrinter.INFO); DebugPrinter.print(service_uri, "service_uri",
		 * DebugPrinter.INFO); DebugPrinter.print(service_port, "service_port",
		 * DebugPrinter.INFO); DebugPrinter.print(service_portType,
		 * "service_portType", DebugPrinter.INFO);
		 * DebugPrinter.print(name_namespace, "name_namespace",
		 * DebugPrinter.INFO); DebugPrinter.print(messages, "messages",
		 * DebugPrinter.INFO); DebugPrinter.print(service_message,
		 * "service_message", DebugPrinter.INFO);
		 * DebugPrinter.print(service_name, "service_name", DebugPrinter.INFO);
		 * DebugPrinter.print(service_namespace, "service_namespace",
		 * DebugPrinter.INFO);
		 */

	}

	/**
	 * Generate the header of the BPEL Script (xml header + namespaces)
	 * 
	 * @param nameURI :
	 *            the URI of the generated BPEL script
	 * @param services :
	 *            the list of services containing the operations
	 * @return the header of the BPELScript
	 * @throws IOException
	 *             if a problem occurs with the given file
	 * @throws SAXException
	 *             if a problem occurs during the parsing of the given file
	 */
	private String generateBPELScriptHeader(URI nameURI,
			Map<String, ServiceMap> services) throws IOException, SAXException {

		// DebugPrinter.print("nameURI = " + nameURI, 1);
		// this.wsdlURI = nameURI.getScheme() + "://" + nameURI.getHost();
		this.wsdlURI = nameURI.getScheme()
				+ "://"
				+ nameURI.getAuthority()
				+ nameURI.getPath().substring(0,
					nameURI.getPath().lastIndexOf('/'));

		String str_bpel = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n";
		str_bpel += "<process name=\"" + nameURI.getFragment() + "\"\n";
		str_bpel += "\tsuppressJoinFailure=\"yes\"\n";
		str_bpel += "\ttargetNamespace=\"" + this.wsdlURI + "/"
				+ nameURI.getFragment() + "\"\n";
		str_bpel += "\txmlns=\"http://schemas.xmlsoap.org/ws/2003/03/business-process/\"\n";

		/*
		 * Import the different namespace
		 */

		List<String> alreadyThreated = new ArrayList<String>();

		// get the list of all namespaces associated to each services
		for (Iterator<Map.Entry<String, String>> i = service_namespace.entrySet().iterator(); i.hasNext();) {
			Map.Entry<String, String> entry = i.next();

			String servicURI = service_uri.get(entry.getKey());

			if (!alreadyThreated.contains(servicURI)) {

				str_bpel += "\txmlns:" + entry.getValue() + "=\"" + servicURI
						+ "\"\n";

				alreadyThreated.add(servicURI);
			}
		}

		// get the namespace of the current process
		// TODO Graham - Test bug fix - writing out of URI misses out port
		// number and full directoy.
		// str_bpel += " xmlns:process=\"" + nameURI.getScheme() + "://" +
		// nameURI.getHost() + "/" + nameURI.getFragment() + ".wsdl\"\n";
		// TODO Graham - Verify hack - chops off last part of path assuming it
		// is the filename.owl part.
		/*
		 * String wsdlURI = nameURI.getScheme() + "://" + nameURI.getAuthority() +
		 * nameURI.getPath().substring(0, nameURI.getPath().lastIndexOf('/'));
		 * String wsdlNameSpace = wsdlURI + "/" + nameURI.getFragment() +
		 * ".wsdl"; str_bpel += "\txmlns:process=\"" + wsdlNameSpace + "\"\n";
		 */

		// get the namespace of the current process
		str_bpel += "\txmlns:process=\"" + this.wsdlURI + "\"\n";
		/*
		 * str_bpel += "\txmlns:process=\"" + wsdlURI + "/" +
		 * nameURI.getFragment() + ".wsdl\"\n";
		 */

		// add last namespace
		str_bpel += "\txmlns:xsd=\"http://www.w3.org/2001/XMLSchema\">\n\n";

		/*
		 * Import the client WSDL
		 */
		/*
		 * str_bpel += "\t<!-- Import the client WSDL -->\n"; str_bpel += "\t<import
		 * location=\"" + nameURI.getFragment() + ".wsdl\" namespace=\"" +
		 * this.wsdlURI + "\"\n"; str_bpel +=
		 * "\t\timportType=\"http://schemas.xmlsoap.org/wsdl/\" />\n\n";
		 */

		return str_bpel;
	}

	/**
	 * Generate the part of the BPEl script corresponding to the partner Links
	 * 
	 * @param services :
	 *            the list of services containing the operations
	 * @return the header of the BPELScript
	 * @return the list of partner links
	 */
	private String generateBPELPartnerLink(URI nameURI,
			Map<String, ServiceMap> services) {

		String str_bpel = "";
		str_bpel += "\t<partnerLinks>\n\n";

		// bpel process' partner link
		str_bpel += "\t\t<!--\n";
		str_bpel += "\t\tThe 'client' role represents the requester of this service. It is\n";
		str_bpel += "\t\tused for callback. The location and correlation information associated\n";
		str_bpel += "\t\twith the client role are automatically set using WS-Addressing.\n";
		str_bpel += "\t\t-->\n";
		str_bpel += "\t\t<partnerLink name=\"" + nameURI.getFragment() + "\"\n";
		str_bpel += "\t\t\tpartnerLinkType=\"process:" + nameURI.getFragment()
				+ "LinkType\"\n";
		str_bpel += "\t\t\tmyRole=\"" + nameURI.getFragment() + "Role\"/>\n\n";

		// store service already mention in pdd file to avoid doublon
		List<String> alreadyTreated = new ArrayList<String>();

		// services' partner link
		str_bpel += "\t\t<!-- The other required services -->\n";
		for (Iterator<Map.Entry<String, ServiceMap>> i = services.entrySet().iterator(); i.hasNext();) {
			Map.Entry<String, ServiceMap> entry = i.next();

			String servicePort = service_name.get(entry.getKey());
			/*
			 * DebugPrinter.print("ServicePort = " + servicePort,
			 * DebugPrinter.INFO); DebugPrinter.print(alreadyTreated,
			 * "alreadyTreated", DebugPrinter.INFO);
			 */

			if (!alreadyTreated.contains(servicePort)) {

				str_bpel += "\t\t<partnerLink name=\"" + servicePort + "\"\n";
				str_bpel += "\t\t\tpartnerLinkType=\""
						+ service_namespace.get(entry.getKey()) + ":"
						+ servicePort + "LinkType\"\n";
				str_bpel += "\t\t\tpartnerRole=\"" + servicePort + "Role\"/>\n";

				alreadyTreated.add(servicePort);
			}
		}

		str_bpel += "\n\t</partnerLinks>\n\n";
		return str_bpel;
	}

	/**
	 * Generate the part of the BPEL script concerning the variable
	 * 
	 * @param currentCapability :
	 *            the name of the requiredCapability currently treated
	 * @param servicesParsed :
	 *            the list of services (parsed) containing the operations
	 * @param matches :
	 *            the list of the selected operation that have to be composed
	 * @return the part of the scipt describing the variables
	 * @throws URISyntaxException
	 *             if a problem occurs during parsing of an URI
	 * @throws GenerationException
	 *             if no grounding is assiciated with one of capa
	 */
	private String generateBPELVariable(Capability currentCapability,
			Map<String, Service> servicesParsed, Map<String, String> matches)
			throws URISyntaxException, GenerationException {

		String str_bpel = "\t<variables>\n\n";

		/*
		 * Bpel process variables - from owl (mainTask)
		 */

		/* BPEL Comments */
		str_bpel += "\t\t<!-- ~~~~~~~~~~~~~~~~~~~~~~~~~~ -->\n";
		str_bpel += "\t\t<!-- Variables for bpel process -->\n";
		str_bpel += "\t\t<!-- ~~~~~~~~~~~~~~~~~~~~~~~~~~ -->\n\n";

		/* Input */
		str_bpel += "\t\t<!-- Reference to the message passed as input during initiation -->\n";
		String message = currentCapability.getTypeName() + "Input";
		str_bpel += "\t\t<variable name=\"" + message
				+ "_Variable\" messageType=\"process:" + message + "\"/>\n";

		/* Output */
		str_bpel += "\n\t\t<!--\n\t\tReference to the message that will be sent back to the\n";
		str_bpel += "\t\trequester during callback\n";
		str_bpel += "\t\t-->\n";
		message = currentCapability.getTypeName() + "Output";
		str_bpel += "\t\t<variable name=\"" + message
				+ "_Variable\" messageType=\"process:" + message + "\"/>\n";

		/* BPEL Comments */
		str_bpel += "\n\t\t<!-- ~~~~~~~~~~~~~~~~~~~~~~~~~~~ -->\n";
		str_bpel += "\t\t<!-- Variables for other process -->\n";
		str_bpel += "\t\t<!-- ~~~~~~~~~~~~~~~~~~~~~~~~~~~ -->\n\n";

		/*
		 * Bpel process variables for other defines services
		 */
		str_bpel += this.generateBPELVariableForOtherServices(
			currentCapability, servicesParsed, matches);

		str_bpel += "\n\t</variables>\n\n";
		return str_bpel;

	}

	/**
	 * Generate the part of the BPEL script concerning the variable of other
	 * services than Usertask
	 * 
	 * @param currentCapability :
	 *            the name of the requiredCapability currently treated
	 * @param servicesParsed :
	 *            the list of services (parsed) containing the operations
	 * @param matches :
	 *            the list of the selected operation that have to be composed
	 * @return the part of the scipt describing the variables
	 * @throws URISyntaxException
	 *             if an error occurs during the parsing of an URI
	 * @throws GenerationException
	 *             if no grounding is assiciated with one of capa
	 */
	private String generateBPELVariableForOtherServices(
			Capability currentCapability, Map<String, Service> servicesParsed,
			Map<String, String> matches) throws URISyntaxException,
			GenerationException {

		String str_bpel = "";

		// we will go trought all capabilities linked by the currentCapability.
		// if the currentCapability is Atomic, then we generate the variables.
		// Otherwise we add all the subcapabilies of this one to the list of
		// capabilities to visit.
		LinkedList<Capability> listCapaToVisit = new LinkedList<Capability>();
		listCapaToVisit.add(currentCapability);

		/* start of the search */
		while (listCapaToVisit.size() > 0) {
			Capability current = listCapaToVisit.poll();

			if (current.isAtomic()) {

				// DebugPrinter.print(matches,"matches",DebugPrinter.INFO);
				// DebugPrinter.print(servicesParsed,"servicesParsed",DebugPrinter.INFO);

				/*
				 * Get the corresponding service
				 */
				String serviceBase = matches.get(Tools.getSecondPart(current.getType()));
				// DebugPrinter.print("serviceBase =
				// "+serviceBase,DebugPrinter.INFO);
				Service s = servicesParsed.get(serviceBase.split("#")[0]);

				// DebugPrinter.print("current.getLocalName() =
				// "+current.getLocalName(),DebugPrinter.INFO);

				/*
				 * Get the corresponding Capability
				 */
				try {
					Capability realCapa = s.getCapabilityFromTypeName(
						current.getTypeName()).get(0);

					// DebugPrinter.print("realCapa =
					// "+realCapa,DebugPrinter.INFO);

					/*
					 * Generate Bpel code
					 */
					str_bpel += "\n";

					try {
						String service = this.getService_namespace().get(
							realCapa.getService().getBase());

						/* Input */
						String message = Tools.getSecondPart(realCapa.getGrounding().getWsdlInputMessage());
						str_bpel += "\t\t<variable name=\"" + message + "_"
								+ service + "_Variable\" messageType=\""
								+ service + ":" + message + "\"/>\n";

						/* Output */
						message = Tools.getSecondPart(realCapa.getGrounding().getWsdlOutputMessage());
						str_bpel += "\t\t<variable name=\"" + message + "_"
								+ service + "_Variable\" messageType=\""
								+ service + ":" + message + "\"/>\n";

					} catch (NullPointerException e) {
						DebugPrinter.print(e, DebugPrinter.WARN);
						throw new GenerationException(
								"No grounding is associated with the capability "
										+ realCapa.getCapabilityURI());
					}
				} catch (NullPointerException e) {
					DebugPrinter.print(e, DebugPrinter.ERROR);
					DebugPrinter.print(
						serviceBase.split("#")[0]
								+ " has not been found in the list of parsed services",
						DebugPrinter.ERROR);

					throw new GenerationException(e);
				}

			} else {
				// add all subcapabilities
				listCapaToVisit.addAll(current.getSubCapabilities());
			}

		}

		return str_bpel;
	}

	/**
	 * Generate the main part of the bpel script, ie the process description
	 * 
	 * @param myProcess
	 *            the process we need to generate
	 * @param nameURI :
	 *            the URI of the generated BPEL script
	 * @param sd
	 *            the list of services containing the operations
	 * @param matches
	 *            the list of the selected operation that have to be composed
	 * @param currentCapability
	 *            the name of the requiredCapability currently treated
	 * @param servicesParsed
	 *            the list of services (parsed) containing the operations
	 * @return the process description part of the bpel script
	 * @throws IOException
	 *             if an IO error occurs
	 * @throws URISyntaxException
	 *             if an error occurs during parsing of an URI
	 * @throws ParsingException
	 *             if an error occurs during the parsing of services
	 */
	private String generateMainProcess(ProcessInterface myProcess, URI nameURI,
			ServiceDiscovery sd, Map<String, String> matches,
			Capability currentCapability, Map<String, Service> servicesParsed)
			throws URISyntaxException, IOException, ParsingException {

		String str_bpel = "\t<sequence name=\"main\">\n\n";

		/*
		 * Receive information from client
		 */

		/* Comments */
		str_bpel += "\t\t<!-- Receive input from requestor. \n";
		str_bpel += "\t\t     Note: This maps to operation defined in "
				+ nameURI.getFragment() + ".wsdl\n";
		str_bpel += "\t\t-->\n";

		// receive information
		str_bpel += "\t\t<receive name=\"receiveInput\" partnerLink=\""
				+ nameURI.getFragment() + "\"\n";
		str_bpel += "\t\t\tportType=\"process:" + nameURI.getFragment()
				+ "PortType\"\n";

		// get operation name
		str_bpel += "\t\t\toperation=\"" + currentCapability.getTypeName()
				+ "\"\n";

		// add variables "variable=\"input\"\n";
		str_bpel += "\t\t\tvariable=\"" + currentCapability.getTypeName()
				+ "Input_Variable\"\n";

		str_bpel += "\t\t\tcreateInstance=\"yes\"/>\n\n";

		/*
		 * Start of the workingFlow
		 */
		str_bpel += "\t\t<!-- +++++++++++++++++++++++++ -->\n";
		str_bpel += "\t\t<!-- START OF THE WORKING FLOW -->\n";
		str_bpel += "\t\t<!-- +++++++++++++++++++++++++ -->\n";
		str_bpel += "\t\t<sequence>\n\n";

		/*
		 * Copy the receive parameter to the corresponding variable *
		 */

		// Bpel comments
		str_bpel += "\t\t\t<!-- ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ -->\n";
		str_bpel += "\t\t\t<!-- ASSIGNATION OF RECEIVE VARIABLES -->\n";
		str_bpel += "\t\t\t<!-- ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ -->\n\n";

		// DebugPrinter.print(matches, "matches", DebugPrinter.INFO);

		// generate bpel copy of variables
		str_bpel += copyBPELInputVariables(sd.getIOMapping(), "\t\t\t",
			currentCapability, matches);

		/*
		 * Generate main code for the task
		 */

		/* Comments */
		str_bpel += "\t\t\t<!-- ~~~~~~~~~~~~~~~~~~~~~~~~~~~~ -->\n";
		str_bpel += "\t\t\t<!-- START OF TASK IMPLEMENTATION -->\n";
		str_bpel += "\t\t\t<!-- ~~~~~~~~~~~~~~~~~~~~~~~~~~~~ -->\n";

		/* Generate the main code of the process */
		str_bpel += this.generateBPEL(currentCapability, matches,
			servicesParsed, "\t\t\t", this);

		/* Comments */
		str_bpel += "\t\t\t<!-- ~~~~~~~~~~~~~~~~~~~~~~~~~~~~ -->\n";
		str_bpel += "\t\t\t<!-- ~~~~~~~~~~~~~~~~~~~~~~~~~~~~ -->\n\n";

		/*
		 * Reply to the client
		 */

		// Bpel comments
		str_bpel += "\t\t\t<!-- ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ -->\n";
		str_bpel += "\t\t\t<!-- ASSIGNATION OF REPLY VARIABLES -->\n";
		str_bpel += "\t\t\t<!-- ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ -->\n\n";

		// generate bpel copy of variables
		str_bpel += copyBPELOutputVariables(sd.getIOMapping(), "\t\t\t",
			currentCapability, servicesParsed);

		/*
		 * End of the workingFlow
		 */
		str_bpel += "\t\t<!-- +++++++++++++++++++++++ -->\n";
		str_bpel += "\t\t<!-- END OF THE WORKING FLOW -->\n";
		str_bpel += "\t\t<!-- +++++++++++++++++++++++ -->\n";
		str_bpel += "\t\t</sequence>\n\n";

		/* Comments */
		str_bpel += "\t\t<!-- Reply output to requestor. \n";
		str_bpel += "\t\t     Note: This maps to operation defined in "
				+ nameURI.getFragment() + ".wsdl\n";
		str_bpel += "\t\t-->\n";

		// reply to the client
		str_bpel += "\t\t<reply name=\"replyOutput\" \n";
		str_bpel += "\t\t\tpartnerLink=\"" + nameURI.getFragment() + "\"\n";
		str_bpel += "\t\t\tportType=\"process:" + nameURI.getFragment()
				+ "PortType\"\n";
		str_bpel += "\t\t\toperation=\"" + currentCapability.getTypeName()
				+ "\"";

		// add reply variables
		str_bpel += "\n\t\t\tvariable=\"" + currentCapability.getTypeName()
				+ "Output_Variable\"";

		str_bpel += "/>\n\n";

		str_bpel += "\t</sequence>\n\n";

		return str_bpel;

	}

	/**
	 * This method will write the initialisation of variables (from value get
	 * into the SOAP body to the BPEL variable)
	 * 
	 * @param mapping
	 *            the mapping between UserTask IO and selected service IO
	 * @param prefix
	 *            the string to put befior each generated string
	 * @param currentCapability :
	 *            the name of the requiredCapability currently treated
	 * @param matches
	 *            the list of the selected operation that have to be composed
	 * @return the part of bpel script corresponding to variable initialisation
	 */
	private String copyBPELInputVariables(Map<IO, IO> mapping, String prefix,
			Capability currentCapability, Map<String, String> matches) {
		String bpel_str = "";
		bpel_str = prefix + "<assign>\n\n";

		// count the number of input found
		int nbrInputFound = 0;

		// go throuth all taskInput
		for (Input taskInput : currentCapability.getInputs().values()) {

			/*
			 * DebugPrinter.print(mapping, "mapping", DebugPrinter.INFO);
			 * 
			 * DebugPrinter.print("taskInput = " + taskInput,
			 * DebugPrinter.INFO);
			 * 
			 * DebugPrinter.print("taskInput.hashCode = " +
			 * taskInput.hashCode(), DebugPrinter.INFO);
			 * DebugPrinter.print("mapping.keySet().toArray()[0]= " +
			 * mapping.keySet().toArray()[0], DebugPrinter.INFO);
			 * 
			 * DebugPrinter.print("mapping.get(0).hashCode = " +
			 * mapping.get(mapping.keySet().toArray()[0]).hashCode(),
			 * DebugPrinter.INFO);
			 */

			// the corresponding input in a selected service
			Input serviceInput = null;

			/*
			 * Find the corresponding input in the selected service ! This input
			 * correspond to the one mapped in the SDResult object if the
			 * Capability is atomic. If the Capability is not atomic, we need to
			 * get the first atomic functionality that have an Input with such
			 * type
			 */
			if (currentCapability.isAtomic()) {
				serviceInput = (Input) mapping.get(taskInput);
			} else {
				Input requiredInput = currentCapability.getFirstAtomicSubCapaUsingSuchIO(taskInput);

				serviceInput = (Input) mapping.get(requiredInput);
			}

			/*
			 * DebugPrinter.print("serviceInput = " + serviceInput,
			 * DebugPrinter.INFO);
			 * DebugPrinter.print("serviceInput.getParentCapability() = " +
			 * serviceInput.getParentCapability(), DebugPrinter.INFO);
			 */

			// increase number of inputs found
			nbrInputFound++;

			// Bpel comment + open tag
			bpel_str += prefix + "\t<!-- " + taskInput.getTypeName() + " -->\n";
			bpel_str += prefix + "\t<copy>\n";

			// get source of the copy
			bpel_str += prefix + "\t\t<from part=\""
					+ Tools.getSecondPart(taskInput.getTypeName())
					+ "\" variable=\"" + currentCapability.getTypeName()
					+ "Input_Variable\"/>\n";

			// set the destination
			bpel_str += prefix
					+ "\t\t<to part=\""
					+ Tools.getSecondPart(serviceInput.getGroundingValue())
					+ "\" variable=\""
					+ Tools.getSecondPart(serviceInput.getParentCapability().getGrounding().getWsdlInputMessage())
					+ "_"
					+ this.getService_namespace().get(
						serviceInput.getParentCapability().getService().getBase())
					+ "_Variable\"/>\n";

			// close tag
			bpel_str += prefix + "\t</copy>\n\n";

		}

		bpel_str += prefix + "</assign>\n\n";

		// DebugPrinter.print(bpel_str, DebugPrinter.INFO);

		// check the number of inputs found to avoid empty assign tag
		if (nbrInputFound > 0)
			return bpel_str;
		else
			return "";
	}

	/**
	 * This method will write the initialisation of return variables (from the
	 * BPEL variable to value put into the SOAP body)
	 * 
	 * @param mapping :
	 *            the mapping between UserTask IO and selected service IO
	 * @param prefix :
	 *            the string to put befior each generated string
	 * @param currentCapability :
	 *            the name of the requiredCapability currently treated
	 * @param servicesParsed :
	 *            the list of services containing the operations
	 * 
	 * @return the part of bpel script corresponding to variable initialisation
	 */
	private String copyBPELOutputVariables(Map<IO, IO> mapping, String prefix,
			Capability currentCapability, Map<String, Service> servicesParsed) {
		String bpel_str = "";
		bpel_str = prefix + "<assign>\n\n";

		// count the number of input found
		int nbrOutputFound = 0;

		// DebugPrinter.print(
		// currentCapability.getOutputs(),"currentCapability.getOutputs()",DebugPrinter.INFO);

		// go throuth all taskOutput
		for (Output taskOutput : currentCapability.getOutputs().values()) {

			// DebugPrinter.print(mapping, "mapping", DebugPrinter.INFO);

			// DebugPrinter.print("taskOutput = " +
			// taskOutput,DebugPrinter.INFO);
			// DebugPrinter.print("currentCapability = " + currentCapability,
			// DebugPrinter.INFO);

			// the corresponding Output in a selected service
			Output serviceOutput = null;

			/*
			 * Find the corresponding input in the selected service ! This input
			 * correspond to the one mapped in the SDResult object if the
			 * Capability is atomic. If the Capability is not atomic, we need to
			 * get the first atomic functionality that have an Input with such
			 * type
			 */
			if (currentCapability.isAtomic()) {
				serviceOutput = (Output) mapping.get(taskOutput);
			} else {
				Output requiredOutput = currentCapability.getLastAtomicSubCapaUsingSuchIO(taskOutput);
				serviceOutput = (Output) mapping.get(requiredOutput);

				// DebugPrinter.print("requiredOutput = " + requiredOutput,
				// DebugPrinter.INFO);
			}

			// DebugPrinter.print("serviceOutput = " + serviceOutput,
			// DebugPrinter.INFO);
			// DebugPrinter.print("serviceInput.getParentCapability() = " +
			// serviceOutput.getParentCapability(), DebugPrinter.INFO);

			// increase number of inputs found
			nbrOutputFound++;

			// Bpel comment + open tag
			bpel_str += prefix + "\t<!-- " + taskOutput.getTypeName()
					+ " -->\n";
			bpel_str += prefix + "\t<copy>\n";

			// get source of the copy
			bpel_str += prefix
					+ "\t\t<from part=\""
					+ Tools.getSecondPart(serviceOutput.getGroundingValue())
					+ "\" variable=\""
					+ Tools.getSecondPart(serviceOutput.getParentCapability().getGrounding().getWsdlOutputMessage())
					+ "_"
					+ this.getService_namespace().get(
						serviceOutput.getParentCapability().getService().getBase())
					+ "_Variable\"/>\n";

			// set the destination
			bpel_str += prefix + "\t\t<to part=\""
					+ Tools.getSecondPart(taskOutput.getTypeName())
					+ "\" variable=\"" + currentCapability.getTypeName()
					+ "Output_Variable\"/>\n";

			// close tag
			bpel_str += prefix + "\t</copy>\n\n";

		}

		bpel_str += prefix + "</assign>\n\n";

		// DebugPrinter.print(bpel_str, DebugPrinter.INFO);

		// check the number of outputs found to avoid empty assign tag
		if (nbrOutputFound > 0)
			return bpel_str;
		else
			return "";
	}

	/*
	 * private String generateBPELAtomicProcess(CapabilityMap cm, Map<String,
	 * String> matches, Map<String, ServiceMap> services) throws Exception {
	 * String str_bpel = ""; // DebugPrinter.print("cm = " + cm, 1); //
	 * DebugPrinter.print(matches, "matches", 1); String realCapability =
	 * matches.get(cm.getType()); // DebugPrinter.print("real capability = " +
	 * realCapability, 1); // get parameter bindings of real capability
	 * 
	 * String realServiceURL = ""; String realCapabilityLocalName = "";
	 * 
	 * try { // DebugPrinter.print("realCapacity = " + realCapability, 1); URL
	 * capability_url = new URL(realCapability); // TODO Graham - Test bug fix -
	 * the code above // misses out the port number (:8080), so changed to //
	 * below. realServiceURL = capability_url.toExternalForm(); realServiceURL =
	 * realServiceURL.replaceFirst("#.*", ""); realCapabilityLocalName =
	 * realCapability.replaceFirst(".*#", ""); } catch (Exception e) {
	 * DebugPrinter.print(e.getStackTrace(), e.getMessage(), 3); }
	 * 
	 * ServiceMap realServiceMap = (ServiceMap) services.get(realServiceURL);
	 * String realServiceOWL = realServiceMap.getOwl(); String realServiceWSDL =
	 * realServiceMap.getWsdl();
	 * 
	 * Service realService = new Service(); // realService.read(realServiceOWL,
	 * realServiceURL); // give also parameter bindings via cm /* str_bpel +=
	 * realService.generateBpel(realCapabilityLocalName, cm, name_namespace);
	 */

	/*
	 * return str_bpel; }
	 */

	/**
	 * Generate the complete ".pdd" and store it it the given file
	 * 
	 * @param file :
	 *            the absolut path of the file where will be stored the
	 *            generated pdd file
	 * @param nameURI :
	 *            the URI of the generated BPEL script
	 * @param services :
	 *            the list of services containing the operations
	 * @throws IOException
	 *             if a problem occurs with the given file
	 * @throws SAXException
	 *             if a problem occurs during the parsing of the given file
	 */
	private void generatePDDFile(String file, URI nameURI,
			Map<String, ServiceMap> services) throws IOException, SAXException {
		// generate PDD header
		String str_pdd = this.generatePDDHeader(nameURI);

		// generate PDD PartnerLinks
		str_pdd += "<!-- START of partner link definition -->\n\n";
		str_pdd += this.generatePDDPartnerLink(nameURI, services);
		str_pdd += "<!-- END of partner link definition -->\n\n";

		// generate WSDL references
		str_pdd += "<!-- START of WSDL reference definition -->\n\n";
		str_pdd += this.generatePDDWSDLReference(nameURI, services);
		str_pdd += "<!-- END of WSDL reference definition -->\n\n";

		// generate the footer
		str_pdd += this.generatePDDFooter();

		// write contains in the given file
		writeFile(str_pdd, file);

	}

	/**
	 * Generate the header of .pdd file, containing xml header, definition of
	 * namespace, and the definition of the first PartnerLink (the task)
	 * 
	 * @param nameURI :
	 *            the URI of the generated BPEL script
	 * @return the header of the ".pdd" file
	 */
	private String generatePDDHeader(URI nameURI) {
		String str_pdd = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n";
		str_pdd += "<process name=\"bpelns:" + nameURI.getFragment() + "\"\n";
		str_pdd += "         location=\"bpel/" + nameURI.getFragment()
				+ ".bpel\"\n";
		str_pdd += "         xmlns=\"http://schemas.active-endpoints.com/pdd/2005/09/pdd.xsd\"\n";
		str_pdd += "         xmlns:wsa=\"http://schemas.xmlsoap.org/ws/2003/03/addressing\"\n";
		str_pdd += "         xmlns:bpelns=\"" + this.wsdlURI + "/"
				+ nameURI.getFragment() + "\">\n\n";

		return str_pdd;
	}

	/**
	 * Generate the list of ParnerLink for the ".pdd" file
	 * 
	 * @param nameURI :
	 *            the URI of the generated BPEL script
	 * 
	 * @param services :
	 *            the list of services containing the operations
	 * @return the part of ".pdd" file containing all the partner links
	 * @throws IOException
	 *             if a problem occurs with the given file
	 * @throws SAXException
	 *             if a problem occurs during the parsing of the given file
	 */
	private String generatePDDPartnerLink(URI nameURI,
			Map<String, ServiceMap> services) throws IOException, SAXException {
		String str_pdd = "";

		/* The first partner link (the task) */
		str_pdd += "<partnerLinks>\n";
		str_pdd += "\t  <partnerLink name=\"" + nameURI.getFragment() + "\">\n";
		str_pdd += "\t    <myRole service=\"" + nameURI.getFragment()
				+ "\" allowedRoles=\"\" binding=\"RPC\"/>\n";
		str_pdd += "\t  </partnerLink>\n\n";

		// store service already mention in pdd file to avoid doublon
		List<String> alreadyTreated = new ArrayList<String>();

		/* The other partner (all selected services */
		for (Iterator<Map.Entry<String, ServiceMap>> i = services.entrySet().iterator(); i.hasNext();) {
			Map.Entry<String, ServiceMap> entry = i.next();

			String serviceName = service_name.get(entry.getKey());

			if (!alreadyTreated.contains(serviceName)) {
				str_pdd += "\t<partnerLink name=\"" + serviceName + "\">\n";
				str_pdd += "\t  <partnerRole endpointReference=\"static\">\n";
				str_pdd += "\t    <wsa:EndpointReference\n";
				str_pdd += "\t         xmlns:"
						+ service_namespace.get(entry.getKey()) + "=\""
						+ service_uri.get(entry.getKey()) + "\">\n";
				str_pdd += "\t      <wsa:Address>"
						+ service_soapAddress.get(entry.getKey())
						+ "</wsa:Address>\n";
				str_pdd += "\t      <wsa:ServiceName\n";
				str_pdd += "\t           PortName=\""
						+ service_port.get(entry.getKey()) + "\">"
						+ service_namespace.get(entry.getKey()) + ":"
						+ serviceName + "</wsa:ServiceName>\n";
				str_pdd += "\t    </wsa:EndpointReference>\n";
				str_pdd += "\t  </partnerRole>\n";
				str_pdd += "\t</partnerLink>\n\n";

				alreadyTreated.add(serviceName);
			}

		}

		str_pdd += "</partnerLinks>\n\n";
		return str_pdd;
	}

	/**
	 * Generate the WsdlReference part of the ".pdd" file
	 * 
	 * @param nameURI :
	 *            the URI of the generated BPEL script
	 * 
	 * @param services :
	 *            the list of services containing the operations
	 * @return the part of the ".pdd" file corresponding to WSDL reference
	 */
	private String generatePDDWSDLReference(URI nameURI,
			Map<String, ServiceMap> services) {
		String str_pdd = "<wsdlReferences>\n";
		str_pdd += "  <wsdl namespace=\"" + nameURI.getScheme() + "://"
				+ nameURI.getHost() + "\"\n";
		/*
		 * str_pdd += " <wsdl namespace=\"" + nameURI.getScheme() + "://" +
		 * nameURI.getHost() + "/" + nameURI.getFragment() + ".wsdl\"\n";
		 */
		str_pdd += "        location=\"project:/wsdl/" + nameURI.getFragment()
				+ ".wsdl\"/>\n";

		// store service already mention in pdd file to avoid doublon
		List<String> alreadyTreated = new ArrayList<String>();

		for (Iterator<Map.Entry<String, ServiceMap>> i = services.entrySet().iterator(); i.hasNext();) {

			Map.Entry<String, ServiceMap> entry = i.next();
			String uri = service_uri.get(entry.getKey());

			if (!alreadyTreated.contains(uri)) {

				str_pdd += "  <wsdl namespace=\"" + uri + "\"\n";
				str_pdd += "        location=\"project:/wsdl/"
						+ service_name.get(entry.getKey()) + ".wsdl\"/>\n";

				alreadyTreated.add(uri);
			}

		}

		str_pdd += "</wsdlReferences>\n\n";
		return str_pdd;
	}

	/**
	 * Generate the ".pdd" footer
	 * 
	 * @return the footer of the .pdd file
	 */
	private String generatePDDFooter() {
		return "</process>\n";
	}

	/**
	 * Generate the content of the catalog and store it in the given file
	 * 
	 * @param file :
	 *            the path of the file where will be stored the generated
	 *            content
	 * @param nameURI :
	 *            the URI of the generated BPEL script
	 * 
	 * @param services :
	 *            the list of services containing the operations
	 * @throws IOException
	 *             if an error occurs during an IO access
	 */
	private void generateCatalogFile(String file, URI nameURI,
			Map<String, ServiceMap> services) throws IOException {
		String str_wsdlCatalog = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n";
		str_wsdlCatalog += "<wsdlCatalog>\n";

		// bpel process catalog

		str_wsdlCatalog += "\t<wsdlEntry location=\"project:/wsdl/"
				+ nameURI.getFragment() + ".wsdl\" classpath=\"wsdl/"
				+ nameURI.getFragment() + ".wsdl\"/>\n";

		// store service already mention in catalog file to avoid doublon
		List<String> alreadyTreated = new ArrayList<String>();

		// services catalog
		for (Iterator<Map.Entry<String, ServiceMap>> i = services.entrySet().iterator(); i.hasNext();) {

			Map.Entry<String, ServiceMap> entry = i.next();

			String serviceName = service_name.get(entry.getKey());

			if (!alreadyTreated.contains(serviceName)) {
				str_wsdlCatalog += "\t<wsdlEntry location=\"project:/wsdl/"
						+ serviceName + ".wsdl\" classpath=\"wsdl/"
						+ serviceName + ".wsdl\"/>\n";

				alreadyTreated.add(serviceName);
			}
		}

		str_wsdlCatalog += "</wsdlCatalog>\n";

		// write the content in a file
		writeFile(str_wsdlCatalog, file);

	}

	/**
	 * Clean all the used during generation HashMap
	 * 
	 */
	private void clearAll() {
		this.service_message.clear();
		this.service_name.clear();
		this.service_namespace.clear();
		this.name_namespace.clear();
		this.service_port.clear();
		this.service_uri.clear();
		this.messages.clear();
	}

	/**
	 * Write String to local file
	 * 
	 * @param doc :
	 *            the contains of the document
	 * @param fname :
	 *            the name of the file which will contain the document
	 * @throws IOException
	 *             if an error occurs during an IO access
	 */
	private static void writeFile(String doc, String fname) throws IOException {

		try {
			File f = new File(fname);
			if (f.exists() && f.canWrite())
				f.delete();

			f.createNewFile();
			FileWriter fw = new FileWriter(f);
			BufferedWriter buf = new BufferedWriter(fw);
			buf.write(doc);
			buf.close();
		} catch (IOException e) {
			DebugPrinter.print(e, DebugPrinter.ERROR);
			throw e;
		}
	}

	/**
	 * Communicate the generated Bpelscript to the Bpel engine
	 * 
	 * @param filepath :
	 *            the absolut path of the file to send to the bpel engine
	 * @param serviceName :
	 *            the name of the generated service
	 * @throws GenerationException
	 *             if an error occurs during the deployment of the bpel script
	 */
	private void sendFileToBpelServer(String filepath, String serviceName)
			throws GenerationException {
		org.apache.axis.client.Service service = new org.apache.axis.client.Service();

		try {
			org.apache.axis.client.Call call = (org.apache.axis.client.Call) service.createCall();

			String url = this.configManager.getBpelDeploymentServiceAddress();

			DebugPrinter.print("deployement URL = "+url,DebugPrinter.INFO);
			
			call.setTargetEndpointAddress(new URL(url));

			String namespace = url;

			call.setOperationName(new QName(namespace,
					this.configManager.BPEL_DEPLOYEMENT_OPERATIONAME));

			call.invoke(new Object[] { serviceName, fileToString(filepath) });

		} catch (Exception e) {
			/* Exception such as ServiceException occur */
			throw new GenerationException(e);
		}
	}

	/**
	 * 
	 * Transfer a file content to base64 format string.<BR>
	 * 
	 * @param file
	 * @return
	 * @throws IOException
	 */
	private String fileToString(String file) throws IOException {

		File f = new File(file);

		byte buf[] = new byte[(int) f.length()];

		FileInputStream fio = null;

		try {
			fio = new FileInputStream(f);

			int count = fio.read(buf);
		} catch (Exception e) {
			DebugPrinter.print(e, DebugPrinter.ERROR);
			throw new IOException("Read file error.");
		} finally {
			if (fio != null) {
				try {
					fio.close();
				} catch (IOException ioe) {
					DebugPrinter.print(ioe, DebugPrinter.ERROR);
				}
			}
		}

		return (new sun.misc.BASE64Encoder()).encode(buf);
	}

	/* ******************* */
	/* * BPEL Generation * */
	/* ******************* */

	/* ////////////////// */
	/* // For UserTask // */
	/* ////////////////// */

	/**
	 * THis method generate the part of the BPEl script corresponding to this
	 * Capability<br>
	 * Generally this method is called for a Capability corresponding to a
	 * UserTask. Indeed, this method will used the given map to find the good
	 * "real" method to invoke, and launch a recursion on the Capability
	 * corresponding to a Service.
	 * 
	 * @param matches :
	 *            the list of the selected operation that have to be composed
	 * @param servicesParsed :
	 *            the list of services containing the operations
	 * @param generator :
	 *            the BpelGenerator object that contains many assocation table
	 * @param prefix :
	 *            the prefix to write before each generated line (to have a
	 *            correct indented xml-file)
	 * @return the part of BPEL script corresponding to the invoke of the
	 *         capability
	 */
	private String generateBPEL(Capability capa, Map<String, String> matches,
			Map<String, Service> servicesParsed, String prefix,
			BpelGenerator generator) {

		// BPEL Comments
		String bpel_str = "\n" + prefix + "<!-- " + capa.getTypeName()
				+ " -->\n";

		// the stop condition of recurcivity
		if (capa.isAtomic()) {

			/*
			 * Get the corresponding service
			 */
			String serviceBase = matches.get(capa.getTypeName());
			Service s = servicesParsed.get(serviceBase.split("#")[0]);

			/*
			 * Get the corresponding Capability
			 */
			Capability realCapa = s.getCapabilityFromTypeName(
				capa.getTypeName()).get(0);

			/*
			 * Launch the generation on the real Capabilities
			 */
			bpel_str += realCapa.getFlow().generateBPELFragment(prefix,
				this.getService_name(), this.getService_portType(),
				this.getService_namespace());

		} else {
			/* Launch recursion depending to the Control Construct */
			// TODO implement for other ControlConstruct
			switch (capa.getCC().getValue()) {
			case ControlConstruct.SEQUENCE_VALUE:
				bpel_str += this.generateSequenceBPEL(capa, matches,
					servicesParsed, prefix, generator);
				break;
			default:
				DebugPrinter.print("ERROR : Control Construct Unknown",
					DebugPrinter.ERROR);
			}
		}
		return bpel_str;
	}

	/**
	 * Recursive call in case the Control Construct is a Sequence
	 * 
	 * @param matches :
	 *            the list of the selected operation that have to be composed
	 * @param services :
	 *            the list of services containing the operations
	 * @param prefix :
	 *            the prefix to write before each generated line (to have a
	 *            correct indented xml-file)
	 * @param generator :
	 *            the BpelGenerator object that contains many assocation table
	 * @return the part of BPEL script corresponding to the invoke of the
	 *         capability (sequence)
	 */
	private String generateSequenceBPEL(Capability capa,
			Map<String, String> matches, Map<String, Service> servicesParsed,
			String prefix, BpelGenerator generator) {
		String bpel_str = prefix + "<sequence>\n";

		// generate bpel script by recursion
		for (Iterator<Capability> it = capa.getSubCapabilities().iterator(); it.hasNext();)
			bpel_str += this.generateBPEL(it.next(), matches, servicesParsed,
				prefix + "\t", generator);

		bpel_str += prefix + "</sequence>\n\n";
		return bpel_str;
	}

	/* ////////////////// */
	/* // For Services // */
	/* ////////////////// */

	/**
	 * THis method generate the part of the BPEl script corresponding to this
	 * Capability<br>
	 * Generally this method is called for a Capability corresponding to a real
	 * Service
	 * 
	 * @param prefix :
	 *            the prefix to write before each generated line (to have a
	 *            correct indented xml-file)
	 * @param generator :
	 *            the BpelGenerator object that contains many assocation table
	 * @return the part of BPEL script corresponding to the invoke of the
	 *         capability
	 */
	/*
	 * private String generateBPEL(Capability capa, String prefix, BpelGenerator
	 * generator) { // DebugPrinter.print("Capa = "+capa,DebugPrinter.INFO);
	 * String bpel_str = prefix + "<!-- " + capa.getCapabilityURI() + "
	 * -->\n\n"; bpel_str += prefix +
	 * capa.getFlow().getValue().generateBPELFragment(prefix,this.getService_name(),this.getService_portType(),this.getService_namespace());
	 * 
	 * //if (capa.isAtomic()) { /*
	 * DebugPrinter.print(generator.getService_port(),"generator.getService_port()",1);
	 * DebugPrinter.print(generator.getService_namespace(),"generator.getService_namespace()",1);
	 * DebugPrinter.print(generator.getService_name(),"generator.getService_name()",1);
	 * DebugPrinter.print(generator.getService_uri(),"generator.getService_uri()",1);
	 * DebugPrinter.print(generator.getService_message(),"generator.getService_message()",1);
	 * 
	 * DebugPrinter.print("this.getService = ",1); this.getService().print();
	 */

	// String serviceBase = capa.getService().getBase();
	/*
	 * Invoke the service
	 */
	// bpel_str += prefix+"<!-- Invoke the service -->\n";
	/*
	 * bpel_str += prefix + "<invoke partnerLink=\"" +
	 * generator.getService_name().get(serviceBase) + "\"\n"; bpel_str += prefix +
	 * "\tportType=\"" + generator.getService_namespace().get(serviceBase) + ":" +
	 * generator.getService_portType().get(serviceBase) + "\"\n"; bpel_str +=
	 * prefix + "\toperation=\"" + Tools.getSecondPart(capa.getGrounding()
	 * .getOperationName()) + "\"\n"; // the input variable //
	 * if(this.getInputs().size()>0){ bpel_str += prefix + "\tinputVariable=\"" +
	 * Tools.getSecondPart(capa.getGrounding() .getWsdlInputMessage()) + "\"\n"; // } //
	 * the output variable // if(this.getOutputs().size()>0){ bpel_str += prefix +
	 * "\toutputVariable=\"" + Tools.getSecondPart(capa.getGrounding()
	 * .getWsdlOutputMessage()) + "\">\n"; // }
	 * 
	 * bpel_str += prefix + "</invoke>\n\n";
	 */

	/*
	 * Receive the result
	 */
	// bpel_str += prefix+"<!-- Get the reply -->\n";
	// receive information
	/*
	 * bpel_str += prefix+"<receive name=\"receiveInput\" partnerLink=\""+
	 * nameURI.getFragment() + "\"\n"; bpel_str +=
	 * prefix+"\tportType=\"process:" + nameURI.getFragment()+ "PortType\"\n"; //
	 * get operation name URI opeName = new URI(currentCapability.getName());
	 * bpel_str += prefix+"\toperation=\"" + opeName.getFragment() + "\"\n"; //
	 * add variables "variable=\"input\"\n"; for (Iterator<Map.Entry<String,
	 * Input>> it = currentCapability .getInputs().entrySet().iterator();
	 * it.hasNext();) { Input in = it.next().getValue(); URI inputURI = new
	 * URI(in.getURI()); bpel_str += "\t\t\tvariable=\"" +
	 * inputURI.getFragment() + "\"\n"; }
	 */

	// } else {
	/* Generate bpel flow from capability workflow */
	// bpel_str +=
	// capa.getFlow().getValue().generateBPELFragment(prefix,this.getService_name(),this.getService_portType(),this.getService_namespace());
	/*
	 * Old version
	 * 
	 */

	/* Launch recursion depending to the Control Construct */

	/*
	 * DebugPrinter.print("Service = ", 1); this.getService().print();
	 * DebugPrinter.print("CC = "+this.getCC(), 1);
	 * DebugPrinter.print("Capability = "+this.toString(), 1);
	 * DebugPrinter.print("CC.name() = "+this.getCC().name(), 1);
	 */
	/*
	 * switch (capa.getCC().getValue()) { case ControlConstruct.SEQUENCE_VALUE:
	 * bpel_str += this.generateSequenceBPEL(capa, prefix, generator); break;
	 * default: DebugPrinter.print("ERROR : Control Construct Unknown",
	 * DebugPrinter.ERROR); }
	 */

	// }
	/*
	 * return bpel_str; }
	 */

	/**
	 * Recursive call in case the Control Construct is a Sequence
	 * 
	 * @param prefix :
	 *            the prefix to write before each generated line (to have a
	 *            correct indented xml-file)
	 * @param generator :
	 *            the BpelGenerator object that contains many assocation table
	 * @return the part of BPEL script corresponding to the invoke of the
	 *         capability (sequence)
	 */
	/*
	 * private String generateSequenceBPEL(Capability capa, String prefix,
	 * BpelGenerator generator) { String bpel_str = prefix + "<sequence>\n"; //
	 * generate bpel script by recursion for (Iterator<Capability> it =
	 * capa.getSubCapabilities().iterator(); it .hasNext();) bpel_str +=
	 * this.generateBPEL(it.next(), prefix + "\t", generator);
	 * 
	 * bpel_str += prefix + "</sequence>\n\n"; return bpel_str; }
	 */

}

/**
 * 
 * BpelGenerator.java
 * 
 * FolderJarer provide a static method to jar a folder. This code is inspired
 * from one found on the net [@author pitchoun (found on
 * http://www.theserverside.com/discussions/thread.tss?thread_id=34906, Posted
 * by: Mohit Gupta on juillet 01, 2005)]
 * 
 * 
 * @author <a href="mailto:sebastien.bianco@inria.fr">sebastien bianco</a> - <a
 *         href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
 * 
 * Creation Date : Jan 2, 2007 1:53:13 PM
 */
class FolderJarer {

	/**
	 * Zip the srcFolder into the destFileZipFile. All the folder subtree of the
	 * src folder is added to the destZipFile archive.
	 * 
	 * 
	 * @param srcFolder
	 *            String, the path of the srcFolder
	 * @param destZipFile
	 *            String, the path of the destination zipFile. This file will be
	 *            created or erased.
	 * @throws JarException
	 *             if an error occurs during the creation of the jar file
	 */
	static public void jarFolder(String srcFolder, String destJarFile)
			throws JarException {
		JarOutputStream jar = null;
		FileOutputStream fileWriter = null;
		try {
			fileWriter = new FileOutputStream(destJarFile);
			jar = new JarOutputStream(fileWriter);
		} catch (Exception ex) {
			DebugPrinter.print(ex, DebugPrinter.FATAL);
			System.exit(0);
		}

		addFolderToJar("", srcFolder, jar, true);
		try {
			jar.flush();
			jar.close();
		} catch (Exception ex) {
			DebugPrinter.print(ex, DebugPrinter.ERROR);
			throw new JarException();
		}
	}

	/**
	 * Write the content of srcFile in a new jarEntry, named path+srcFile, of
	 * the jar stream. The result is that the srcFile will be in the path folder
	 * in the generated archive.
	 * 
	 * @param path
	 *            String, the relatif path with the root archive.
	 * @param srcFile
	 *            String, the absolute path of the file to add
	 * @param jar
	 *            jarOutputStram, the stream to use to write the given file.
	 * @throws JarException
	 *             if an error occurs during the creation of the jar file
	 */
	static private void addToJar(String path, String srcFile,
			JarOutputStream jar) throws JarException {

		File folder = new File(srcFile);
		if (folder.isDirectory()) {
			addFolderToJar(path, srcFile, jar, false);
		} else {
			// Transfer bytes from in to out
			byte[] buf = new byte[1024];
			int len;
			FileInputStream in = null;
			try {
				in = new FileInputStream(srcFile);
				jar.putNextEntry(new JarEntry(path + "/" + folder.getName()));
				while ((len = in.read(buf)) > 0) {
					jar.write(buf, 0, len);
				}
			} catch (IOException ex) {
				DebugPrinter.print(ex, DebugPrinter.ERROR);
				throw new JarException();
			} finally {
				if (in != null)
					try {
						in.close();
					} catch (IOException e) {
						DebugPrinter.print(e, DebugPrinter.ERROR);
					}
			}
		}
	}

	/**
	 * add the srcFolder to the jar stream.
	 * 
	 * @param path
	 *            String, the relatif path with the root archive.
	 * @param srcFile
	 *            String, the absolute path of the file to add
	 * @param jar
	 *            jarOutputStram, the stream to use to write the given file.
	 * @param firstlevel
	 *            indicate if it is the first level of recursion of not, to
	 *            permit to remove the first level of repository in the jar file
	 * @throws JarException
	 *             if an error occurs during the creation of the jar file
	 */
	static private void addFolderToJar(String path, String srcFolder,
			JarOutputStream jar, boolean firstlevel) throws JarException {
		File folder = new File(srcFolder);
		String fileListe[] = folder.list();
		try {
			for (int i = 0; i < fileListe.length; i++) {
				if (firstlevel) {
					addToJar("", srcFolder + "/" + fileListe[i], jar);

				} else {
					addToJar(path + "/" + folder.getName(), srcFolder + "/"
							+ fileListe[i], jar);
				}
			}
		} catch (Exception ex) {
			DebugPrinter.print(ex, DebugPrinter.ERROR);
			throw new JarException();
		}
	}

}
