/**
 ##############################################################################
 # Copyright (C) 2006-2010 INRIA
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 2 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ##############################################################################
 * 
 */

package fr.inria.icocoa.compositionengine;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import com.hp.hpl.jena.ontology.OntModel;

import fr.inria.icocoa.bpelgenerator.BpelGenerator;
import fr.inria.icocoa.common.ConfigManager;
import fr.inria.icocoa.common.DebugPrinter;
import fr.inria.icocoa.common.JenaStarter;
import fr.inria.icocoa.common.MeasurementTool;
import fr.inria.icocoa.common.PropertyManager;
import fr.inria.icocoa.common.Tools;
import fr.inria.icocoa.exception.ParsingException;
import fr.inria.icocoa.servicediscovery.ServiceDiscovery;
import fr.inria.icocoa.servicemodel.Capability;
import fr.inria.icocoa.servicemodel.UserTask;
import fr.inria.icocoa.servicemodel.parsers.jenaparser.QoSParser;
import fr.inria.icocoa.servicemodel.qos.QoSProperty;
import fr.inria.icocoa.servicemodel.qos.QoSRequirement;
import fr.inria.icocoa.servicemodel.qos.Type;

/**
 * iCOCOA Composition engine
 * <p>
 * The composition engine interacts with the service requestor and the
 * repository. <br>
 * It presents a composition interface to the service requestor with the
 * following operations:<br>
 * {@link deployProcessAsService}, {@link undeployProcess} and
 * {@link executeProcess}.
 * <p>
 * 
 * The {@link deployProcessAsService} and {@link executeProcess} operations take
 * as an argument a composite process defined in the SDL.<br>
 * The composite process should define a required functionality, which is the
 * composite process, composing several functionalities (composed service
 * functionalities).<br>
 * The composition engine then first access the repositoy and queries all
 * available services providing the composed functionalities.<br>
 * Then, it retrieves their semantic description, which includes their
 * conversations and their WSDL service grounding.<br>
 * The composition engine tries to find all suitable compositions among all the
 * matching service functionalities, by integrating their conversations.<br>
 * When at least one composition is found, it calls the makeConcrete operation
 * to generate an executable BPEL process.<br>
 * <p>
 * The {@link makeConcrete} operation reads the grounding part in the SDL
 * documents to generate a BPEL process calling the matching functionalities.<br>
 * Each provided functionality is replaced by the conversation in terms of WSDL
 * operations. <br>
 * This is defined in the Proces Model of the functionalities.<br>
 * Then, for each atomic process, the grounding part is read to generate the
 * proper invocations and parameter bindings.<br>
 * <p>
 * The generated files are:<br>
 * The composite process' BPEL file<br>
 * The compsite process' WSDL file<br>
 * The modified WSDL files of accessed services (original WSDL files with
 * partnerLinkTypes)<br>
 * A configuration file to deploy the BPEL process on top of the ActiveBpel
 * engine<br>
 * <p>
 * If the composition is initiated with the {@link deployProcessAsService}
 * operation, the BPEL process is deployed on an underlying BPEL engine, and the
 * WSDL file is returned to the service requestor.<br>
 * It is then up to the service requestor to make Web service calls to the
 * composite service to execute the process.<br>
 * The deployed service can be undeployed any time with the
 * {@link undeployProcess} operation.
 * <p>
 * If the composition is initiated with the {@link executeProcess} operation,
 * the BPEL process is deployed, and the deployed service is immediately invoked
 * internally. This assumes that all required input paramter values are given in
 * the initial process definition in SDL.<br>
 * The results are then returned to the service requestor and the service is
 * undeployed.
 * 
 * <p>
 * Management Interface:<br>
 * It is used by the composition engine user to perform administrative
 * operations on the engine<br>
 * There are three methods:<br>
 * {@link traceOn} to enable logging<br>
 * {@link traceOff} to diasble logging<br>
 * {@link getUnderlyingWFEngineURL} to get the administation URL of the
 * underlyinh BPEL engine<br>
 * 
 * @author <a href="mailto:Galip-Ferda.Tartanoglu@inria.fr">Ferda Tartanoglu</a> -
 *         <a href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
 */
public class CompositionEngine {

	// constants

	static String HOME = System.getProperty("user.dir");

	static String SEPARATOR = System.getProperty("file.separator");

	static String PREFIX = SEPARATOR + "tmp" + SEPARATOR;

	static String TEMP_FILE = "aggregated.owl";

	// main functionality ontology

	static String FUNCTIONALITY_NS = "http://www.dyonisos.org/functionalities.owl#";

	static String FUNCTIONALITY_LN = "ServiceFunctionality";

	// private String underlyingWFEngineURL;

	// initialize the repository service
	// private Repository repository = Repository.getInstance();

	/*
	 * The property manager used by this class
	 */
	/*
	 * private PropertyManager propertyManager = PropertyManager .getInstance();
	 */

	// private static String myuri = "";
	/**
	 * Initialize composition engine
	 * 
	 */
	public CompositionEngine() {

		// set the underlying BPEL engine URL
		// underlyingWFEngineURL = "http://localhost:8080";
	}

	/**
	 * Deploys a process as a new service. The process is an orchestration over
	 * required functionalities. Functionalities are described semantically
	 * using SDL, without binding/grounding to any concrete service. The method
	 * uses the repository to search for real services and composes the process
	 * taking into account the conversations of each service.
	 * 
	 * @param process
	 *            Abstract process orchestrating semantic functionalities
	 * @param expandProcess
	 *            set to false for forcing the the functionality to be replaced
	 *            with exactly one service operation.
	 * @return returns the list of WSDL document of the resulting composite Web
	 *         service
	 * @throws Exception
	 *             raised if composition or deployment fails
	 */
	public List<String> deployProcessAsService(UserTask userTask)
			throws Exception {

		// deploy

		/*
		 * Generated bpel files are in :
		 * 
		 * bpel/service/
		 * 
		 * deploying:
		 * 
		 * jar cf service.bpr *.pdd META-INF bpel wsdl cp service.bpr
		 * $CATALINA_HOME/bpr
		 * 
		 */
		return makeConcrete(userTask);
	}

	/**
	 * Undeploys the composite service given its URI
	 * 
	 * @param processURI
	 *            URI of the composite service to undeploy.
	 * @throws Exception
	 *             raised if the removal fails.
	 */
	public void undeployProcess(String processURI) throws Exception {

	}

	/**
	 * Execute a process by searching for matching services.<br>
	 * <font color="#FF0000">!!!! Not Tested, please don't use it !!!</font>
	 * 
	 * @param process
	 *            Abstract process orchestrating semantic functionalities
	 * @param expandProcess
	 *            set to false for forcing the the functionality to be replaced
	 *            with exactly one service operation
	 * @return output retrieved after the execution of the process
	 * @throws Exception
	 *             raised if the composition or execution fails
	 */
	public String executeProcess(UserTask userTask) throws Exception {

		String result = "";

		List<String> service = makeConcrete(userTask);

		// deploy

		/*
		 * Generated bpel files are in :
		 * 
		 * bpel/service/
		 * 
		 * deploying:
		 * 
		 * jar cf service.bpr *.pdd META-INF bpel wsdl cp service.bpr
		 * $CATALINA_HOME/bpr
		 * 
		 */

		// execute
		// TODO: read from process SDL
		String operation = "";
		String input = "";

		String url = ConfigManager.getInstance().getUnderlyingWFEngineURL()
				+ "/active-bpel/services/" + service + "?method=" + operation
				+ "&value=" + input;

		try {
			URL u = new URL(url);
			HttpURLConnection huc = (HttpURLConnection) u.openConnection();
			huc.setRequestMethod("GET");
			huc.connect();
			InputStream is = huc.getInputStream();
			int code = huc.getResponseCode();
			if (code == HttpURLConnection.HTTP_OK) {
				byte[] buffer = new byte[4096];

				OutputStream os = new ByteArrayOutputStream();

				int bytes = 0;
				while (true) {
					bytes = is.read(buffer);
					if (bytes <= 0)
						break;
					os.write(buffer, 0, bytes);
				}
				os.close();

			}
			huc.disconnect();

		} catch (IOException e) {
			e.printStackTrace();
		}

		// undeploy
		// undeployProcess(service);

		return result;
	}

	/**
	 * Transform an abstract process described in SDL to a BPEL concrete process
	 * Used by deploy.. and execute..
	 * 
	 * @param userTask
	 * @return the list of WSDL file content of the generated Service
	 */
	private List<String> makeConcrete(UserTask userTask) throws Exception {

		// Creation of a Service Discovery Object
		ServiceDiscovery sd = new ServiceDiscovery();

		// Creation of the Service Composition Object
		COCOA_CI cocoa = new COCOA_CI(sd);

		/*
		 * send to composer -> get results
		 */
		long startTime = System.currentTimeMillis();
		Map<String, List<String>> results = cocoa.compose(userTask);
		long estimatedTime = System.currentTimeMillis() - startTime;
		MeasurementTool.displayTime("The composition process :",estimatedTime);
		// DebugPrinter.print(results,"Composition result: \n",
		// DebugPrinter.INFO);

		// test if there is a least a composition
		if (results.size() == 0)
			throw new NoSolutionException(
					"There is no valid composition for this task");

		// Get the map of selected services from composition result
		Map<String, String> selected = this.extractSelectedServicesFromCompositionResult(
			results, sd);
		
		//DebugPrinter.print(selected,"selected : \n", DebugPrinter.INFO);

		// Map<String, String> best = performQoSSelection(selected, userTask,
		// sd);

		/*
		 * Call BPEL Process generator
		 */
		startTime = System.currentTimeMillis();
		BpelGenerator generator = new BpelGenerator();
		List<String> service = generator.generateBpelProcess(userTask,
			selected, sd);
		estimatedTime = System.currentTimeMillis() - startTime;
		MeasurementTool.displayTime("The generation process :",estimatedTime);

		// service =
		// generator.generateBpelProcess2(process,selected,sd.getServicesMap());
		return service;

	}

	private Map<String, String> performQoSSelection(
			Map<String, String> selected, UserTask userTask, ServiceDiscovery sd)
			throws IOException, ParsingException, NoSolutionException {
		// foreach compostion:
		// . foreach task req capa:
		// ... ensure qos reqs are respected by service qos props

		// we are selecting on a per capability basis, so select the one that is
		// best for each capability individually.
		// but, the problem is, the current impl. return a single mapping for
		// all task capabilities to all service capbilities.
		// in which case, we have try each composition, determine the qos for
		// all required capabilities in the task, sum them, and
		// select the compsition with the best score over all req capas.

		// for each selected {

		Map<String, List<QoSRequirement>> taskQoSReqs = extractFunctionalityLevelRequirements(userTask);
		// DebugPrinter.print(taskQoSReqs, "task qos reqs", 1);
		if (taskQoSReqs.isEmpty()) {
			DebugPrinter.print(
				"Task description contains no QoS requirements.",
				DebugPrinter.INFO);
			return selected;
		}

		Map<String, List<QoSProperty>> serviceSubCapabilitiesQoSProperties = extractServiceSubCapabilitiesQoSProperties(
			selected, sd);
		// DebugPrinter.print(serviceSubCapabilitiesQoSProperties, "props", 1);

		List<Map<String, String>> qosRespectingCompositions = extractQoSRespectingCompositions(
			selected, userTask, taskQoSReqs,
			serviceSubCapabilitiesQoSProperties);

		if (qosRespectingCompositions.isEmpty()) {
			throw new NoSolutionException(
					"None of the available compositions respect the task QoS requirements.");
		} else {
			DebugPrinter.print(
				"There is at least a composition that respects the task's QoS requirements.",
				1);
		}

		// for now, assume list is sorted with best at head - so get(0) returns
		// best.
		return qosRespectingCompositions.get(0);
	}

	private List<Map<String, String>> extractQoSRespectingCompositions(
			Map<String, String> selected, UserTask userTask,
			Map<String, List<QoSRequirement>> taskQoSReqs,
			Map<String, List<QoSProperty>> serviceSubCapabilitiesQoSProperties) {
		List<Map<String, String>> qosRespectingCompositions = new ArrayList<Map<String, String>>();
		List<Capability> taskRequiredCapabilities = userTask.getRequiredCapabilities();
		boolean allReqCapasAreRespected = true;
		for (Capability capability : taskRequiredCapabilities) {
			List<List<QoSProperty>> qosProps = new ArrayList<List<QoSProperty>>();
			for (int i = 0; i < Type.values().length; ++i) {
				qosProps.add(new ArrayList<QoSProperty>());
			}

			if (capability.isAtomic()) {
				List<QoSProperty> providedQoSProps = serviceSubCapabilitiesQoSProperties.get(capability.getLocalName());
				for (QoSProperty property : providedQoSProps) {
					qosProps.get(property.getType().ordinal()).add(property);
				}
			} else {
				for (Capability subCapability : capability.getSubCapabilities()) {
					List<QoSProperty> providedQoSProps = serviceSubCapabilitiesQoSProperties.get(subCapability.getLocalName());
					for (QoSProperty property : providedQoSProps) {
						qosProps.get(property.getType().ordinal()).add(property);
					}
				}
			}

			QoSProperty[] combinedQoSProps = new QoSProperty[Type.values().length];
			for (Type type : Type.values()) {
				combinedQoSProps[type.ordinal()] = QoSProperty.combine(
					qosProps.get(type.ordinal()), type);
			}

			List<QoSRequirement> capabailityQoSReqs = taskQoSReqs.get(capability.getLocalName());
			QoSRequirement[] orderedCapaQoSReqs = new QoSRequirement[Type.values().length];
			for (QoSRequirement req : capabailityQoSReqs) {
				orderedCapaQoSReqs[req.getProperty().getType().ordinal()] = req;
			}

			boolean allQosReqIsRespected = true;
			for (Type type : Type.values()) {
				if (!orderedCapaQoSReqs[type.ordinal()].isRespectedBy(combinedQoSProps[type.ordinal()])) {
					allQosReqIsRespected = false;
					break;
				}
			}

			if (!allQosReqIsRespected) {
				allReqCapasAreRespected = false;
				break;
			}
		}

		if (allReqCapasAreRespected) {
			qosRespectingCompositions.add(selected);
		}
		return qosRespectingCompositions;
	}

	private Map<String, List<QoSProperty>> extractServiceSubCapabilitiesQoSProperties(
			Map<String, String> selected, ServiceDiscovery sd)
			throws IOException, ParsingException {
		Map<String, List<QoSProperty>> serviceSubCapabilitiesQoSProperties = new HashMap<String, List<QoSProperty>>();
		List<String> selectedServiceCapabitilties = new ArrayList<String>(
				selected.values());
		Set<String> serviceNames = new HashSet<String>();
		for (String ssCapa : selectedServiceCapabitilties) {
			String serviceName = ssCapa.split("#")[0];
			serviceNames.add(serviceName);
			// DebugPrinter.print("service name: " + serviceName, 1);
		}
		for (String serviceName : serviceNames) {
			OntModel serviceModel = JenaStarter.getInstanceOfOntModel(false);
			serviceModel.read(Tools.createTempOwlFile(sd.getServicesMap().get(
				serviceName).getOwl()));
			QoSParser qosParser = new QoSParser(serviceModel);
			Map<String, List<QoSProperty>> serviceQoSProps = qosParser.parseSubFunctionalityLevelProperties();
			serviceSubCapabilitiesQoSProperties.putAll(serviceQoSProps);
		}
		return serviceSubCapabilitiesQoSProperties;
	}

	private Map<String, List<QoSRequirement>> extractFunctionalityLevelRequirements(
			UserTask userTask) {
		OntModel taskModel = JenaStarter.getInstanceOfOntModel(false);
		taskModel.read(userTask.getOwlDescriptionFile());
		QoSParser qosParser = new QoSParser(taskModel);
		Map<String, List<QoSRequirement>> taskQoSReqs = qosParser.parseFunctionalityLevelRequirements();
		return taskQoSReqs;
	}

	/**
	 * Create a map of association TaskCapabilityRDFType -> CapabilityURI (the
	 * URI correspond to Capability local name in the selected service) from the
	 * result of the composition.
	 * 
	 * @param compositionResult :
	 *            the Data structure corresponding to the result of the
	 *            composition
	 * @param sd :
	 *            the result of the service discovery
	 * @return the map of association TaskCapabilityRDFType -> CapabilityURI
	 */
	private Map<String, String> extractSelectedServicesFromCompositionResult(
			Map<String, List<String>> compositionResult, ServiceDiscovery sd) {
		Map<String, String> selected = new HashMap<String, String>();

		for (Iterator<Map.Entry<String, List<String>>> i = compositionResult.entrySet().iterator(); i.hasNext();) {

			Map.Entry<String, List<String>> entry = i.next();
			String required_str = entry.getKey();
			List<String> provided_all = entry.getValue();
			
			//DebugPrinter.print(provided_all,"provided_all",DebugPrinter.INFO);

			// select the first matching => the first transition
			String provided_str = provided_all.get(0);
			//DebugPrinter.print("provided_str = "+provided_str,DebugPrinter.INFO);
			

			// get the symbol of the transition (required)
			String[] required_parts = required_str.split(";");
			String required = required_parts[1];
			//DebugPrinter.print("required = "+required,DebugPrinter.INFO);

			String[] provided_parts = provided_str.split(";");
			String[] provided_parts_ = provided_parts[1].split("\\.");
			String provided = provided_parts_[1];

			// get the URI of the service
			//DebugPrinter.print(sd.getMapCapability_ServiceFile(),"sd.getMapCapability_ServiceFile()",DebugPrinter.INFO);
			//DebugPrinter.print("provided = "+provided,DebugPrinter.INFO);
			String providedURI = sd.getMapCapability_ServiceFile().get(provided);

			// store the result of the association
			selected.put(required, providedURI + "#" + provided);
		}

		return selected;
	}

}
