package fr.inria.icocoa.semrep;

import fr.inria.icocoa.semrep.excp.SemanticRepositoryException;

/**
 * @author Graham
 * 
 */
public interface SemanticRepository {

	/**
	 * Retrieve a service by its name.
	 * 
	 * @param serviceName
	 *            the name of the required service
	 * @return the Service object corresponding to the given name
	 */
	public String getServiceWithName(String serviceName);

	/**
	 * Retrieve the set of services that match a semantic type.
	 * 
	 * @param serviceType
	 *            the semantic type of the required services
	 * @return a list of Service objects corresponding to the given semantic
	 *         type
	 */
	public String[] getServicesWithType(String serviceType);

	/**
	 * Get the XML Fragment of a service description corresponding to its QoS
	 * properties.
	 * 
	 * @param serviceName :
	 *            the name of the service we required the QoS properties
	 * @return an XML fragment containing QoS properties associated with the
	 *         service.
	 */
	public String getQoSProperties(String serviceName);

	/**
	 * Retrieve a set of services that match the task description. If no service
	 * match exactly, repository will attempt to compose and deploy a match
	 * service from the services currently registered with the repository.
	 * 
	 * @param taskURI :
	 *            the URI of the file containing the description of the required
	 *            task
	 * @return a list of matching services if a match has been found, an empty
	 *         list if no match is possible, or <i>null</i> if it was not
	 *         possible to process the request
	 * @throws SemanticRepositoryException :
	 *             if an error occurs during the parsing of the service, or if
	 *             the task description contains no required capability
	 */
	public String[] getServicesForTask(String taskURI)
			throws SemanticRepositoryException;

	/**
	 * This method returns the associated WSDL description for the specified
	 * service.
	 * 
	 * @param serviceName :
	 *            the service
	 * @return The WSDL description of the specified service, or null if the
	 *         service is not registered with the repository
	 */
	public String getWsdlForService(String serviceName);

	/**
	 * Calling this method will add the specified local ontology to the
	 * repository. This method allows local copies of ontologies to be used,
	 * when required ontologies are not available over the network, e.g. when
	 * the application is run behind a firewall, or the ontologies are still in
	 * development and not yet published.
	 * 
	 * @param filename
	 *            the path to the local ontology file
	 * @throws SemanticRepositoryException
	 *             if an exception occurred processing the ontology file
	 */
	public void addOntology(String filename) throws SemanticRepositoryException;

	/**
	 * Register a service in the repository. Adds the semantic description of
	 * the service together with its associated WSDL document.
	 * <p>
	 * Returns the URI of the service, as declared in the SDL description. This
	 * URI can further be used to remove the service using the removeService
	 * method.
	 * 
	 * @param providedService
	 *            SDL description of the service
	 * @param WSDLDocument
	 *            WSDL Document of the service
	 * @return URI of the service (unique identifier to be used with
	 *         removeService)
	 * @throws SemanticRepositoryException :
	 *             if an error occurs during the parsing of the service, if an
	 *             error occurs with a file, or if a similar service as ever
	 *             been registered.
	 */
	public String addService(String providedService, String WSDLDocument)
			throws SemanticRepositoryException;

	/**
	 * Unregister a service in the repository. Removes the semantic description
	 * of the service. The service is identified by its URI, which is the
	 * identifier that is returned by the
	 * {@link #addService(java.lang.String, java.lang.String)} method.
	 * 
	 * @param serviceURI
	 *            URI of the service to un-register.
	 * @throws SemanticRepositoryException
	 *             an exception if the removal of the service fails. E.g, if
	 *             there is no registered service referenced by the given URI
	 */
	public void removeService(String serviceURI)
			throws SemanticRepositoryException;

	/**
	 * Get the semantic type of a service from its name.
	 * 
	 * @param serviceName :
	 *            the name of the service that we want to know the name
	 * @return the semantic type of the service, <i>null</i>
	 */
	public String getServiceTypeFromServiceName(String serviceName);

	/**
	 * Remove all services and service descriptions from the repository.
	 * <p>
	 * Note: This method is intended for developers to provide a convenient
	 * means to clear the repository during development and testing. It is not
	 * recommend to call this method on a live repository.
	 */
	public void clearRepository();

	/**
	 * Get a list of all service object managed by the repository.
	 * 
	 * @return the list of all services, <i>empty list</i> if no services are manages
	 *         by the repository.
	 */
	public String[] getAllServices();

	/**
	 * Set the type of matcher to be used by the repository.
	 * 
	 * @param matcher
	 *            the name of the matcher to be used.
	 * @throws SemanticRepositoryException
	 *             if the name of the matcher is not recognised.
	 */
	public void setMatcher(String matcher) throws SemanticRepositoryException;

}