/**
 ##############################################################################
 # Copyright (C) 2006-2010 INRIA
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 2 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ##############################################################################
 * 
 */
package fr.inria.icocoa.semrep;

import fr.inria.icocoa.common.ConfigManager;
import fr.inria.icocoa.common.DebugPrinter;
import fr.inria.icocoa.semrep.excp.MatcherLoadingException;
import fr.inria.icocoa.servicemodel.matching.Matcher;
import fr.inria.icocoa.servicemodel.matching.SyntacticMatcher;

/**
 * This class permit to have an acess to an instance of SemanticRepository or to
 * an instance of SemanticRepositoryForCompositionEngine.<br>
 * The instance of SemanticRepository is single ! If the matcher is different,
 * then the matcher is updated.<br>
 * <br>
 * You can indicate the Matcher you want to use during the algorithm, by giving
 * the Matcher object as parameter of the constructor. If you don't give any
 * Matcher, the SyntaxicMatcher will be used. <br>
 * This indication could be done by using "matcher.classname". To set it, please
 * run jvm with option <i>-Dmatcher.classname="YourCompleteClassName"</i>. If
 * the classname is false, then default matcher will be used (SyntacticMatcher) !
 * This will be indicate in log traces.
 * 
 * @author <a href="mailto:sebastien.bianco@inria.fr">sebastien bianco</a> - <a
 *         href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
 * 
 * Creation Date : Oct 1, 2007 1:50:51 PM
 */
public class SemanticRepositoryFactory {

	/** current instance of SemanticRepositoryForCompositionEngine */
	private static SemanticRepositoryImpl instance = null;

	/**
	 * get an instance of SemanticRepository
	 * 
	 * @param m
	 *            the matcher that will be used by the repository to do matching
	 *            algorithm
	 */
	public static SemanticRepositoryForCompositionEngine getInstance(Matcher m) {
		if (instance == null) {
			instance = new SemanticRepositoryImpl(m);
		} else if (!instance.getMatcher().equals(m)) {
			instance.setMatcher(m);
		}
		return instance;
	}

	/**
	 * get an instance of SemanticRepositoryForCompositionEngine
	 * 
	 * @param m
	 *            the matcher that will be used by the repository to do matching
	 *            algorithm
	 */
	public static SemanticRepositoryForCompositionEngine getInstanceCE(Matcher m) {
		return getInstance(m);
	}

	/**
	 * get an instance of SemanticRepository that use a SyntaxicMatcher
	 */
	public static SemanticRepository getInstance() {
		return getInstance(SemanticRepositoryFactory.getMatcher());
	}

	/**
	 * get an instance of SemanticRepositoryForCompositionEngine that use a
	 * SyntaxicMatcher
	 */
	public static SemanticRepositoryForCompositionEngine getInstanceCE() {
		return getInstanceCE(SemanticRepositoryFactory.getMatcher());
	}

	/**
	 * Check the environment variable "matcher.classname". If this variable
	 * exists, then use the given class name to load the Matcherer, else use a
	 * SyntacticMatcher
	 * 
	 * @return the loaded matcher
	 */
	@SuppressWarnings("unchecked")
	private static Matcher getMatcher() {
		//load the config manager to be sur about system properties
		ConfigManager.getInstance();
		
		Matcher result = new SyntacticMatcher();

		// load Matcher class name configuration file
		String className = System.getProperty("matcher.classname");

		if (className != null) {

			try {
				// Dynamic load of the Matcher class
				try {
					Class matcherClass = Class.forName(className);
					Matcher matcherInstance = (Matcher) matcherClass.newInstance();

					// return the loaded matcher
					return matcherInstance;

				} catch (ClassNotFoundException e) {
					DebugPrinter.print(e, DebugPrinter.WARN);
					throw new MatcherLoadingException(e);
				} catch (InstantiationException e) {
					DebugPrinter.print(e, DebugPrinter.WARN);
					throw new MatcherLoadingException(e);
				} catch (IllegalAccessException e) {
					DebugPrinter.print(e, DebugPrinter.WARN);
					throw new MatcherLoadingException(e);
				} catch (ClassCastException e) {
					DebugPrinter.print(e, DebugPrinter.WARN);
					throw new MatcherLoadingException(e);
				}
			} catch (MatcherLoadingException e) {
				DebugPrinter.print(e, DebugPrinter.WARN);
				DebugPrinter.print(
					"An error occurs during dynamic load of the Matcher. SyntacticMatcher will be used.",
					DebugPrinter.WARN);
			}
		}

		return result;

	}

}
