package fr.inria.icocoa.semrep;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.ObjectInput;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.List;
import java.util.UUID;

import com.thoughtworks.xstream.XStream;

import fr.inria.icocoa.common.ConfigManager;
import fr.inria.icocoa.common.DebugPrinter;
import fr.inria.icocoa.common.DeleteDir;
import fr.inria.icocoa.common.JenaStarter;
import fr.inria.icocoa.common.Tools;
import fr.inria.icocoa.exception.MatchException;
import fr.inria.icocoa.exception.ParsingException;
import fr.inria.icocoa.semrep.dataset.ServiceBag;
import fr.inria.icocoa.semrep.dataset.ServiceBagFactory;
import fr.inria.icocoa.semrep.excp.SemanticRepositoryException;
import fr.inria.icocoa.semrep.excp.ServiceAlreadyRegisteredException;
import fr.inria.icocoa.servicemodel.Capability;
import fr.inria.icocoa.servicemodel.Service;
import fr.inria.icocoa.servicemodel.matching.CapabilityMatch;
import fr.inria.icocoa.servicemodel.matching.ExactSemanticMatcher;
import fr.inria.icocoa.servicemodel.matching.Matcher;
import fr.inria.icocoa.servicemodel.matching.SyntacticMatcher;
import fr.inria.icocoa.servicemodel.matching.WeakSemanticMatcher;
import fr.inria.icocoa.servicemodel.parsers.ServiceParserFactory;

/**
 * You can indicate the Matcher you want to use during the algorithm, by giving
 * the Matcher object as parameter of the constructor. If you don't give any
 * Matcher, the SyntaxicMatcher will be used.
 * 
 * @author <a href="mailto:Graham.Thomson@inria.fr">Graham Thomson</a> - <a
 *         href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
 */
public class SemanticRepositoryImpl implements
		SemanticRepositoryForCompositionEngine {

	/** the file where the Repository is stored */
	private final File servicesStore;

	/** The data corresponding to registered services */
	private ServiceBag serviceBag;

	/** The Xml stream marshaller/unmarshaller */
	private XStream xstream = new XStream();

	/**
	 * Create a SemanticRepository object (initialize content from saved file)
	 * 
	 * @param matcher
	 *            the matcher that will be used to match services/functionality
	 */
	SemanticRepositoryImpl(Matcher matcher) {
		super();
		this.serviceBag = ServiceBagFactory.getInstance().getServiceBag();
		this.serviceBag.setMatcher(matcher);
		createRepositoryDirectories();
		servicesStore = new File(ConfigManager.getInstance()
				.getRepositoryIndexFolder()
				+ File.separatorChar + "services.obj");
		loadServicesFromStore();
	}

	/**
	 * Create a SemanticRepository object (initialize content from saved file)
	 * and initialize Matcher with a SyntaxicMatcher
	 */
	SemanticRepositoryImpl() {
		this(new SyntacticMatcher());
	}

	/**
	 * @return the serviceBag
	 */
	public ServiceBag getServiceBag() {
		return serviceBag;
	}

	
	/**
	 * Get a specific Service object thanks its name
	 * 
	 * @param serviceName :
	 *            the name of the requested service object
	 *            (ex:http://www.hitech-projects.com/euprojects/amigo/ServiceA.owl#ServiceA)
	 * @return the service object corresponding with the given name, <i>null</i>
	 *         if no object with such names corresponds.
	 */
	public String getServiceWithName(String serviceName) {
		Service s= this.getServiceBag().getServiceObjectWithName(serviceName);
		if(s!=null){	
			return s.getQualifiedName();
		}
		return null;
	}

	/**
	 * Get a list of Service object that type correspond to the given parameter
	 * 
	 * @param serviceType :
	 *            the requested type of the service
	 *            (ex:http://www.hitech-projects.com/euprojects/amigo/MyServiceTypes.owl#Type1).
	 * @return a list of service corresponding to the requested type, an empty
	 *         list if no result is found.
	 */
	public String[] getServicesWithType(String serviceType) {
		return this.getServiceBag().getServicesWithType(serviceType);
	}

	/**
	 * Get a list of match between services contained into the RepositoryOld and
	 * the task given by the User.
	 * 
	 * @param taskURI :
	 *            the URI of the file containing the semantic description
	 *            (amigo-s) of the task required by the User
	 *            (ex:file:/afs/inria.fr/rocq/home/arles/bianco/workspace/sdsdcae_repository/amigo-s/TaskA.owl)
	 * @return a list of match object, empty list if no match are found.
	 * @throws ParsingException :
	 *             this exception is thrown when a problem occurs during parsing
	 * @throws MatchException :
	 *             this exception is thrown when a problem occurs during the
	 *             match process
	 */
	public String[] getServicesForTask(String taskURI)
			throws SemanticRepositoryException {
		return this.getServiceBag().getServicesForTask(taskURI);
	}

	/**
	 * Go through the services contained into the repository to select the one
	 * that contains. <br>
	 * This alogrithm is o(n*m) where n=number of services, m= average number of
	 * functionalities per service
	 * 
	 * @param requiredCapa
	 *            the required Capability
	 * @return the match between the required Capability and the found one,
	 *         <i>empty list</i> if no match is found
	 */
	public List<CapabilityMatch> getServicesContainingCapability(
			Capability requiredCapa) {
		return this.getServiceBag().getServicesContainingCapability(
				requiredCapa);
	}

	public String[] getAllServices() {
		return this.getServiceBag().getAllServices();
	}

	public String getWsdlForService(String serviceName) {
		WsdlResult result = null;
		Service service = this.getServiceBag().getServiceObjectWithName(
				serviceName);
		if (service != null) {
			if (service.isComposedService()) {
				result = new WsdlResult(service.getWsdlDescriptionFile());
			} else {
				result = new WsdlResult(service.getWsdlDescriptionFile(),
						service.getBase(), service.getProvided_Capabilities());
			}
			return xstream.toXML(result);
		}
		return null;
	}

	public void addOntology(String filename) throws SemanticRepositoryException {
		try {
			JenaStarter.addOntologyFile(Tools.readFromFile(filename));
		} catch (Exception e) {
			throw new SemanticRepositoryException(e);
		}
	}

	/**
	 * Registration of a service so as to be manage by the RepositoryOld. This
	 * service could now appear in the results of a query.
	 * 
	 * @param semanticDescriptionURI :
	 *            the URI of the semantic description file (amigo-s) of the
	 *            service (ex:./amigo-s/ServiceA.owl)
	 * @param wsdlDocumentURI :
	 *            the URI of the syntaxic description file (wsdl) of the service
	 *            (ex:./wsdl/ServiceA.wsdl)
	 * @return the complete URI of the Service object that has been registered
	 *         (ex:
	 *         http://www.hitech-projects.com/euprojects/amigo/ServiceA.owl#ServiceA)
	 * @throws ServiceAlreadyRegisteredException :
	 *             this exception is throws if another service with the same URI
	 *             has been already registered
	 * @throws ParsingException
	 *             :this exception is thrown if a error appear during the
	 *             parsing of a file
	 */
	public String addService(String semanticDescriptionURI,
			String wsdlDocumentURI) throws SemanticRepositoryException {

		//DebugPrinter.print("add Service - owlFile = \n"+semanticDescriptionURI+"\nwsdlFile = \n"+wsdlDocumentURI+"\n",DebugPrinter.INFO);
		
		
		String owlFilename = UUID.randomUUID().toString();
		String wsdlFilename = UUID.randomUUID().toString();

		//DebugPrinter.print("write owl file = "+owlFilename,DebugPrinter.INFO);
		
		// read files
		String owlsData = Tools.readFromFile(semanticDescriptionURI);
		String wsdlData = Tools.readFromFile(wsdlDocumentURI);
		
		
		//DebugPrinter.print("read owl file = \n"+owlsData,DebugPrinter.INFO);
		
		

		String owlQName = copyToRepository(owlsData, owlFilename);
		String wsdlQName = copyToRepository(wsdlData, wsdlFilename);

		String owlQNameURI = Tools.getURIStringFromFilePath(owlQName);
		String wsdlQNameURI = Tools.getURIStringFromFilePath(wsdlQName);
		
		//DebugPrinter.print("owlQNameURI = \n"+owlQNameURI,DebugPrinter.INFO);
		

		Service service;
		try {
			service = ServiceParserFactory.getInstance().getParser()
					.parseProcess(owlQNameURI, wsdlQNameURI);
			service.setWsdlContent(wsdlData);
			
			//DebugPrinter.print("ParsedService = "+service, DebugPrinter.INFO);
			
		} catch (ParsingException e) {
			throw new SemanticRepositoryException(
					"Unable to parse semantic description.("
							+ e.getLocalizedMessage() + ")", e);
		}

		if (this.getServiceBag().getServiceObjectWithName(
				service.getQualifiedName()) != null) {
			deleteFromRepository(owlQName);
			deleteFromRepository(wsdlQName);

			throw new SemanticRepositoryException("Service '"
					+ service.getQualifiedName() + "' already registered.");
		}

		this.getServiceBag().addService(service);

		updateServicesStore();

		return owlFilename;
	}

	/**
	 * Unregister a Service from the RepositoryOld. This service could not
	 * appear anymore in the results of a query.
	 * 
	 * @param serviceUUID :
	 *            the complete URI of the service object that you want
	 *            unregister (ex:
	 *            http://www.hitech-projects.com/euprojects/amigo/ServiceA.owl#ServiceA)
	 */
	public void removeService(String serviceUUID) {

		Service delete = this.getServiceBag().removeService(serviceUUID);
		if (delete != null) {

			deleteFromRepository(delete.getOwlDescriptionFile());
			deleteFromRepository(delete.getWsdlDescriptionFile());
		}
		updateServicesStore();
	}

	

	/***************************************************************************
	 * Delete all content from the Repository
	 * 
	 */
	public void clearRepository() {

		File dataFolder = new File(ConfigManager.getInstance()
				.getRepositoryDataFolder());
		File indexFolder = new File(ConfigManager.getInstance()
				.getRepositoryIndexFolder());
		DeleteDir.deleteDirectory(dataFolder);
		DeleteDir.deleteDirectory(indexFolder);

		this.getServiceBag().clear();

		// call the garbage collector
		System.runFinalization();

		createRepositoryDirectories();
	}

	/**
	 * Get the XML Fragment of the service description corresponding to the QOS
	 * 
	 * @param serviceName :
	 *            the name of the service we required the QoS properties
	 * @return an XML fragment containing QoS properties associated with the
	 *         service.
	 */
	public String getQoSProperties(String serviceName) {
		return this.getServiceBag().getServiceObjectWithName(serviceName)
				.getQos().getQoSXMLPart();
	}

	/**
	 * Get the Semantic Type of a Service from its name.
	 * 
	 * @param serviceName :
	 *            the name of the service that we want to know the name
	 * @return the semantic type of the service, <i>null</i>
	 */
	public String getServiceTypeFromServiceName(String serviceName) {
		Service s = this.getServiceBag().getServiceObjectWithName(serviceName);
		if (s != null) {
			return s.getType();
		} else {
			return null;
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see fr.inria.icocoa.semrep.SemanticRepositoryForCompositionEngine#getServiceObjectWithLocalName(java.lang.String)
	 */
	public Service getServiceObjectWithLocalName(String serviceName) {
		return this.getServiceBag().getServiceObjectWithName(serviceName);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see fr.inria.icocoa.semrep.SemanticRepositoryForCompositionEngine#displayAllServices()
	 */
	public void displayAllServices() {
		for (Service s : this.getServiceBag().getAllServiceObjects()) {
			s.print(System.out);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see fr.inria.icocoa.semrep.SemanticRepositoryForCompositionEngine#getServiceObjectFromOwlFileURI(java.lang.String)
	 */
	public Service getServiceObjectFromOwlFileURI(String owlFileURI) {
		for (Service s : this.getServiceBag().getAllServiceObjects()) {
			if (s.getOwlDescriptionFile().equalsIgnoreCase(owlFileURI)) {
				return s;
			}
		}
		return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see fr.inria.icocoa.semrep.SemanticRepositoryForCompositionEngine#getServiceObjectFromBaseURL(java.lang.String)
	 */
	public Service getServiceObjectFromBaseURL(String baseURL) {
		for (Service s : this.getServiceBag().getAllServiceObjects()) {
			if (s.getBase().equalsIgnoreCase(baseURL)) {
				return s;
			}
		}
		return null;
	}

	public void setMatcher(String matcher) throws SemanticRepositoryException {
		String m = matcher.toLowerCase();
		if (m.equals("syntactic")) {
			setMatcher(new SyntacticMatcher());
		} else if (m.equals("exact")) {
			setMatcher(new ExactSemanticMatcher());
		} else if (m.startsWith("weak ")) {
			int radius = Integer.parseInt(m.substring(5));
			setMatcher(new WeakSemanticMatcher(radius));
		} else {
			throw new SemanticRepositoryException("Unrecognised matcher: "
					+ matcher);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see fr.inria.icocoa.semrep.SemanticRepositoryForCompositionEngine#getMatcher()
	 */
	public Matcher getMatcher() {
		return this.getServiceBag().getMatcher();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see fr.inria.icocoa.semrep.SemanticRepositoryForCompositionEngine#setMatcher(fr.inria.icocoa.servicemodel.matching.Matcher)
	 */
	public void setMatcher(Matcher matcher) {
		this.getServiceBag().setMatcher(matcher);
	}
	
	/**
	 * Copy the content of a file into another file
	 * @param filedata the file content to copy
	 * @param filename the name of the destination file
	 * @return the complete name of the file that have been created
	 */
	private String copyToRepository(String filedata, String filename) {
		String qname = ConfigManager.getInstance().getRepositoryDataFolder()
				+ filename;
		try {
			BufferedWriter bw = new BufferedWriter(new FileWriter(new File(
					qname)));
			bw.write(filedata);
			bw.close();
		} catch (IOException e) {
			DebugPrinter.print("Failed to write file: " + e.getMessage(),
					DebugPrinter.ERROR);
		}
		return qname;
	}
	
	/**
	 * Store the list of all service objects contained into the ServiceBag
	 * 
	 */
	private void updateServicesStore() {
		try {
			ObjectOutputStream objOut = new ObjectOutputStream(
					new BufferedOutputStream(
							new FileOutputStream(servicesStore)));
			objOut.writeObject(this.serviceBag.getAllServiceObjects());
			objOut.close();
		} catch (IOException e) {
			DebugPrinter.print("Unable to update serivces store: " + e,
					DebugPrinter.ERROR);
		}
	}
	
	/** 
	 * Delete a file from the repository
	 * @param filename the name of the file to delete
	 * @return <i>true</i> if no error occurs, <i>false</i> otherwise.
	 */
	private boolean deleteFromRepository(String filename) {
		return new File(filename).delete();
	}
	
	/**
	 * Create the required folder if those doesn't exist
	 */
	private void createRepositoryDirectories() {
		// The following folders are accessed to invoke their side effect -
		// the directory is created if it does not already exist.
		ConfigManager.getInstance().getRepositoryFolder();
		ConfigManager.getInstance().getRepositoryIndexFolder();
		ConfigManager.getInstance().getRepositoryDataFolder();
	}

	/**
	 * Load the services from the file where they are stored, and recreate the associated ServiceBag object
	 */
	@SuppressWarnings("unchecked")
	private void loadServicesFromStore() {
		if (servicesStore.exists() && servicesStore.canRead()) {
			try {
				ObjectInput objIn = new ObjectInputStream(
						new BufferedInputStream(new FileInputStream(
								servicesStore)));
				List<Service> services = (List<Service>) objIn.readObject();
				objIn.close();

				for (Service service : services) {
					this.getServiceBag().addService(service);
				}
			} catch (Exception e) {
				DebugPrinter.print("Failed to load services from store: " + e,
						DebugPrinter.ERROR);
			}
		}
	}


}
