package fr.inria.icocoa.semrep.dataset;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Serializable;
import java.net.URL;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;

import fr.inria.icocoa.common.ConfigManager;
import fr.inria.icocoa.common.DebugPrinter;
import fr.inria.icocoa.common.Tools;
import fr.inria.icocoa.compositionengine.CompositionEngine;
import fr.inria.icocoa.exception.MatchException;
import fr.inria.icocoa.exception.ParsingException;
import fr.inria.icocoa.exception.SemanticTypeNotKnowException;
import fr.inria.icocoa.semrep.excp.SemanticRepositoryException;
import fr.inria.icocoa.servicemodel.Capability;
import fr.inria.icocoa.servicemodel.Service;
import fr.inria.icocoa.servicemodel.UserTask;
import fr.inria.icocoa.servicemodel.matching.CapabilityMatch;
import fr.inria.icocoa.servicemodel.matching.Matcher;
import fr.inria.icocoa.servicemodel.matching.ServiceMatch;
import fr.inria.icocoa.servicemodel.parsers.ServiceParserFactory;
import fr.inria.icocoa.servicemodel.semantictype.SemanticType;

/**
 * 
 * This object contains all data of the repository in ordered or unordered way
 *
 * @author <a href="mailto:sebastien.bianco@inria.fr">sebastien bianco</a> - <a href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
 * 
 * Creation Date : Aug 28, 2008 11:31:07 AM
 */
public abstract class ServiceBag implements Serializable{

	/** The matcher that will be used to match services/functionality */
	protected Matcher matcher = null;
	

	/**
	 * The map of services that have been stored
	 */
	protected Map<String, Service> nameToServiceMap = new HashMap<String, Service>();


	// ///////////// //
	// abstract part //
	// ///////////// //
	
	/**
	 * Go through the services contained into the repository to select the one
	 * that contains. <br>
	 * This alogrithm is o(n*m) where n=number of services, m= average number of
	 * functionalities per service
	 * 
	 * @param requiredCapa
	 *            the required Capability
	 * @return the match between the required Capability and the found one,
	 *         <i>empty list</i> if no match is found
	 */
	public abstract List<CapabilityMatch> getServicesContainingCapability(Capability requiredCapa);


	/**
	 * Clear the service bag
	 * 
	 */
	public abstract void clear();

	/**
	 * registered a new service in the bag
	 * 
	 * @param service
	 *            the service to add
	 */
	public abstract void addService(Service service);

	/**
	 * Unregister a Service from the RepositoryOld. This service could not
	 * appear anymore in the results of a query.
	 * 
	 * @param serviceUUID :
	 *            the complete URI of the service object that you want
	 *            unregister (ex:
	 *            http://www.hitech-projects.com/euprojects/amigo/ServiceA.owl#ServiceA)
	 * @return the remove service object, <b>null</b> if none is found
	 */
	public abstract Service removeService(String serviceUUID);

	/**
	 * Get a list of Service object that type correspond to the given parameter
	 * 
	 * @param serviceType :
	 *            the requested type of the service
	 *            (ex:http://www.hitech-projects.com/euprojects/amigo/MyServiceTypes.owl#Type1).
	 * @return a list of service corresponding to the requested type, an empty
	 *         list if no result is found.
	 */
	public abstract String[] getServicesWithType(String serviceType);

	/**
	 * Get the list of services that contains a given semantic type (collected and sorted by Semantic Type)
	 * @return the typeToServiceMap the map serviceType <-> List of services
	 */
	public abstract Map<SemanticType, List<Service>> getTypeToServiceMap();

	/**
	 * Insert a service in the specific object that will store all services (bag of services)
	 * 
	 * @param service
	 *            the service to insert
	 */	
	protected abstract void addServiceInBag(Service service);

	// ******************************************************************************************

	public ServiceBag() {
		super();
	}

	/**
	 * @return the matcher
	 */
	public Matcher getMatcher() {
		return matcher;
	}

	/**
	 * @param matcher the matcher to set
	 */
	public void setMatcher(Matcher matcher) {
		this.matcher = matcher;		
	}

	
	/**
	 * @return the nameToServiceMap
	 */
	public Map<String, Service> getNameToServiceMap() {
		return nameToServiceMap;
	}
	
	/**
	 * Get a specific Service object thanks its name
	 * 
	 * @param serviceName :
	 *            the name of the requested service object
	 *            (ex:http://www.hitech-projects.com/euprojects/amigo/ServiceA.owl#ServiceA)
	 * @return the service object corresponding with the given name, <i>null</i>
	 *         if no object with such names corresponds.
	 */
	public Service getServiceObjectWithName(String serviceName) {
		DebugPrinter.print(nameToServiceMap,"nameToServiceMap", DebugPrinter.INFO);
		Service service = nameToServiceMap.get(serviceName);
		if (service != null) {
			return service;
		} else {
			return null;
		}
	}
	
	/**
	 * Get a service object thanks to the URL
	 * 
	 * @param baseURL
	 *            the URL of the service
	 * @return the required Service object
	 */
	public Service getServiceObjectFromBaseURL(String baseURL) {
		for (Service s : this.nameToServiceMap.values()) {
			if (s.getBase().equalsIgnoreCase(baseURL)) {
				return s;
			}
		}
		return null;
	}

	/**
	 * Get a table containing the Qualified Name of all services registered
	 * 
	 * @return the list of registered services, <i>empty list</i> if no
	 *         services are registered
	 */
	public String[] getAllServices() {
		if (nameToServiceMap != null) {
			String[] result = new String[nameToServiceMap.keySet().size()];
			int i = 0;
			for (String serviceName : nameToServiceMap.keySet()) {
				result[i++] = serviceName;
			}
			return result;
		} else {
			return new String[0];
		}
	}

	/**
	 * Get the list of all service object stored in the SzeviceBag
	 * 
	 * @return a list of Service object
	 */	
	public List<Service> getAllServiceObjects() {
		return new ArrayList<Service>(this.getNameToServiceMap().values());
	}
	
	/**
	 * Get a list of match between services contained into the RepositoryOld and
	 * the task given by the User.
	 * 
	 * @param taskURI :
	 *            the URI of the file containing the semantic description
	 *            (amigo-s) of the task required by the User
	 *            (ex:file:/afs/inria.fr/rocq/home/arles/bianco/workspace/sdsdcae_repository/amigo-s/TaskA.owl)
	 * @return a list of match object, empty list if no match are found.
	 * @throws ParsingException :
	 *             this exception is thrown when a problem occurs during parsing
	 * @throws MatchException :
	 *             this exception is thrown when a problem occurs during the
	 *             match process
	 */
	public String[] getServicesForTask(String taskURI)
			throws SemanticRepositoryException{
		UserTask task;
		try {
			task = new UserTask(ServiceParserFactory.getInstance().getParser()
					.parseProcess(taskURI, false));
		} catch (ParsingException e) {
			throw new SemanticRepositoryException("Unable to parse task", e);
		}

		if (task.getRequired_Capabilities().size() < 1) {
			DebugPrinter.print(
					"The TaskParser File contains no Required capabilities", 3);
			throw new SemanticRepositoryException(
					"The TaskParser File contains no Required capabilities.");
		}

		// First scan for existing or already composed services that match the
		// task.
		List<String> result = new ArrayList<String>();
		for (Service service : this.nameToServiceMap.values()) {
			// DebugPrinter.print("test with "+service.getLocalName(),1);
			try {
				ServiceMatch sm = service.match(task, this.getMatcher());
				if (sm != null) {
					result.add(sm.getService().getQualifiedName());
				}
			} catch (SemanticTypeNotKnowException e) {
				DebugPrinter.print(e.getMessage(), DebugPrinter.WARN);
			}
		}

		if (!result.isEmpty()) {
			return result.toArray(new String[result.size()]);
		}

		// If no existing services match, try to compose a service.
		CompositionEngine ce = new CompositionEngine();
		List<String> deployedServices;
		try {
			deployedServices = ce.deployProcessAsService(task);
			DebugPrinter.print(deployedServices, "deployedServices",
					DebugPrinter.INFO);
		} catch (Exception e) {
			DebugPrinter.print(e, DebugPrinter.ERROR);
			throw new SemanticRepositoryException(e);
		}

		// Add new composed services to the repository, so that they are
		// available for general use.
		for (int i = 0; i < deployedServices.size(); ++i) {
			String composedServiceQname = task.getQualifiedName()
					+ "Composition" + (i + 1);
			result.add(composedServiceQname);

			String composedServiceWsdl;
			try {
				composedServiceWsdl = Tools.readFromUrl(new URL(
						deployedServices.get(i)));
			} catch (Exception e) {
				throw new SemanticRepositoryException(e);
			}
			String wsdlFilename = UUID.randomUUID().toString();
			String wsdlQName = copyStringToRepository(composedServiceWsdl,
					wsdlFilename);
			String wsdlQNameURI = Tools.getURIStringFromFilePath(wsdlQName);

			Service composedService = new Service(composedServiceQname, task
					.getType(), wsdlQNameURI, true);
			addServiceByName(composedService);
			if (composedService.getType() != null
					&& !composedService.getType().trim().equals("")) {
				addServiceInBag(composedService);
			}
		}

		return result.toArray(new String[result.size()]);
	}

	
	// ***************************************************************************************************
	
	/**
	 * remove the service from the map nameToServiceMap
	 * 
	 * @param service
	 *            the service object to remove from
	 */
	protected void removeServiceByName(Service service) {
		nameToServiceMap.remove(service.getQualifiedName());
	}

	/**
	 * Add a service in the map nameToServiceMap
	 * 
	 * @param service
	 *            the service to add
	 */
	protected void addServiceByName(Service service) {
		//DebugPrinter.print("Add service by name- Service =\n"+service, DebugPrinter.INFO);
		nameToServiceMap.put(service.getQualifiedName(), service);
	}
	
	/**
	 * write a string into a file
	 * 
	 * @param data
	 *            the string to write
	 * @param filename
	 *            the name of the file to write into
	 * @return the complete path of the written file
	 */
	protected String copyStringToRepository(String data, String filename) {
		String qname = ConfigManager.getInstance().getRepositoryDataFolder()
				+ filename;
		try {
			BufferedWriter bw = new BufferedWriter(new FileWriter(new File(
					qname)));
			bw.write(data);
			bw.close();
		} catch (IOException e) {
			DebugPrinter.print("Failed to write file: " + e.getMessage(),
					DebugPrinter.ERROR);
		}
		return qname;
	}
	
}