/**
 ##############################################################################
 # Copyright (C) 2006-2010 INRIA
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 2 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ##############################################################################
 * 
 */
package fr.inria.icocoa.semrep.dataset;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;

import fr.inria.icocoa.common.ConfigManager;
import fr.inria.icocoa.common.DebugPrinter;

/**
 * This class is the entry point so as to get a ServiceBag object. 
 * <br>
 * You can indicate the implementation of the service bag you want, by setting the system property using "servicebag.classname". 
 * To set it, please run jvm with option <i>-Dservicebag.classname="YourCompleteClassName"</i>. If
 * the classname is false, then default matcher will be used (ServiceBagImplementation) !
 * This will be indicate in log traces.
 *
 * @author <a href="mailto:sebastien.bianco@inria.fr">sebastien bianco</a> - <a href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
 * 
 * Creation Date : Sep 11, 2008 1:44:11 PM
 */
public class ServiceBagFactory {
	
	static{
		//load config manager so as to be sur that system properties will be set
		ConfigManager.getInstance();
	}
	
	/**
	 * Single instance of the Factory
	 */
	private static ServiceBagFactory instance = null;

	
	/**
	 * the System property value to look at to set the class that need to be
	 * used during execution
	 */
	private final static String PROPERTYNAME = "servicebag.classname";

	/**
	 * The default class that will be load in case of error
	 */
	private final static Class DEFAULTCLASS = ServiceBagImplementation.class;
	
	/**
	 * The constructor that will be used to return instance of OntologieGraphGenerator
	 */
	private Constructor<ServiceBag> noParametersConstructor = null;
	
	/**
	 * The name of the class to load as Graph Generator
	 */
	private String classToLoadName = DEFAULTCLASS.getName();
	
	
	/**
	 * Default constructor of the class. Initialise the private field <i>noParametersConstructor</i>
	 * thanks to system property <b>graphgenerator.classname</b>
	 */
	@SuppressWarnings("unchecked")
	private ServiceBagFactory(){
		
		
		String classToLoad = System.getProperty(PROPERTYNAME);
		
		//check if property has been defined
		if(classToLoad == null){
			DebugPrinter.print("The system property "+PROPERTYNAME+" has not been set. The class "+DEFAULTCLASS.getName()+" will be used has ServiceBag", DebugPrinter.WARN);
			classToLoad = DEFAULTCLASS.getName();
		}else{
			this.classToLoadName = classToLoad;
		}
				
		try {
			//load the class
			Class<ServiceBag> loadedClass = (Class<ServiceBag>) Class.forName(classToLoad);
			
			//initialise constructor (default constructor)
			this.noParametersConstructor = loadedClass.getConstructor(new Class[]{});
		} catch (ClassNotFoundException e) {
			DebugPrinter.print(e, DebugPrinter.ERROR);
		} catch (SecurityException e) {
			DebugPrinter.print(e, DebugPrinter.ERROR);
		} catch (NoSuchMethodException e) {
			DebugPrinter.print(e, DebugPrinter.ERROR);
		}
	}
	
	/**
	 * Get the single instance of the Factory
	 * @return the OntologiesGraphGeneratorFactory instance
	 */
	public static ServiceBagFactory getInstance(){
		if(instance==null)
			instance = new ServiceBagFactory();
		return instance;
	}

	/**
	 * Get a Graph generator instance 
	 * @return a graph generator instance, <b>null</b> if an error occurs
	 */
	public ServiceBag getServiceBag(){
		try {
			if(noParametersConstructor!=null){
				DebugPrinter.print("getServiceBag => create new Instance",DebugPrinter.INFO);
				return noParametersConstructor.newInstance();
			}else{
				DebugPrinter.print("noParametersConstructor is null ! (for "+this.classToLoadName+")",DebugPrinter.ERROR);
			}
		
		} catch (IllegalArgumentException e) {
			DebugPrinter.print(e,DebugPrinter.ERROR);
		} catch (InstantiationException e) {
			DebugPrinter.print(e,DebugPrinter.ERROR);
		} catch (IllegalAccessException e) {
			DebugPrinter.print(e,DebugPrinter.ERROR);
		} catch (InvocationTargetException e) {
			DebugPrinter.print(e,DebugPrinter.ERROR);
		} catch (Exception e){
			DebugPrinter.print(e,DebugPrinter.ERROR);
		}
		return null;
	}

}
