/**
 ##############################################################################
 # Copyright (C) 2006-2010 INRIA
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 2 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ##############################################################################
 * 
 */
package fr.inria.icocoa.semrep.dataset;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import fr.inria.icocoa.common.DebugPrinter;
import fr.inria.icocoa.common.Tools;
import fr.inria.icocoa.exception.SemanticTypeNotKnowException;
import fr.inria.icocoa.servicemodel.Capability;
import fr.inria.icocoa.servicemodel.Service;
import fr.inria.icocoa.servicemodel.matching.CapabilityMatch;
import fr.inria.icocoa.servicemodel.matching.Matcher;
import fr.inria.icocoa.servicemodel.semantictype.SemanticType;
import fr.inria.icocoa.servicemodel.semantictype.SemanticTypeFactory;

/**
 * ServiceBag.java
 * 
 * @author <a href="mailto:sebastien.bianco@inria.fr">sebastien bianco</a> - <a
 *         href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
 * 
 * Creation Date : May 16, 2008 3:00:25 PM
 */
public class ServiceBagImplementation extends ServiceBag {

	/**
	 * serial ID
	 */
	private static final long serialVersionUID = 1L;

	/**
	 * The index storing list of services corresponding to a given SemanticType
	 */
	private Map<SemanticType, List<Service>> typeToServiceMap = new HashMap<SemanticType, List<Service>>();


	/**
	 * @param matcher
	 */
	public ServiceBagImplementation(Matcher matcher) {
		this.matcher = matcher;
	}

	/**
	 * Default Constructor
	 */
	public ServiceBagImplementation() {}
	

	/**
	 * @return the typeToServiceMap
	 */
	@Override
	public Map<SemanticType, List<Service>> getTypeToServiceMap() {
		return typeToServiceMap;
	}

	/**
	 * Get a list of Service object that type correspond to the given parameter
	 * 
	 * @param serviceType :
	 *            the requested type of the service
	 *            (ex:http://www.hitech-projects.com/euprojects/amigo/MyServiceTypes.owl#Type1).
	 * @return a list of service corresponding to the requested type, an empty
	 *         list if no result is found.
	 */
	@Override
	public String[] getServicesWithType(String serviceType) {
		SemanticType _serviceType = SemanticTypeFactory.getInstance()
				.getObject(Tools.getSecondPart(serviceType),
						Tools.getFirstPart(serviceType));
		if (typeToServiceMap.containsKey(_serviceType)) {
			List<Service> services = typeToServiceMap.get(_serviceType);
			String[] result = new String[services.size()];
			for (int i = 0; i < services.size(); ++i) {
				result[i] = services.get(i).getQualifiedName();
			}
			return result;
		} else {
			return new String[0];
		}
	}

	

	

	/**
	 * Remove a service of the map typeToServiceMap
	 * 
	 * @param service
	 *            the service to remove
	 * @param serviceUUID
	 *            the UUID associeted to this service
	 */
	private void removeServiceByType(Service service, String serviceUUID) {
		List<Service> services = typeToServiceMap.get(service.getType());
		if (services != null) {
			for (Iterator<Service> itr = services.iterator(); itr.hasNext();) {
				Service s = itr.next();

				if (s.getOwlDescriptionFile().endsWith("/" + serviceUUID)) {
					itr.remove();
				}
			}
		}
	}

	

	/**
	 * Insert a service in the map typeToServiceMap
	 * 
	 * @param service
	 *            the service to insert
	 */
	@Override
	protected void addServiceInBag(Service service) {
		SemanticType type = service.getSemanticType();
		if (!typeToServiceMap.containsKey(type)) {
			typeToServiceMap.put(type, new ArrayList<Service>());
		}
		typeToServiceMap.get(type).add(service);
	}

	/**
	 * Unregister a Service from the RepositoryOld. This service could not
	 * appear anymore in the results of a query.
	 * 
	 * @param serviceUUID :
	 *            the complete URI of the service object that you want
	 *            unregister (ex:
	 *            http://www.hitech-projects.com/euprojects/amigo/ServiceA.owl#ServiceA)
	 * @return the remove service object, <b>null</b> if none is found
	 */
	@Override
	public Service removeService(String serviceUUID) {
		for (Iterator<Service> itr = this.getAllServiceObjects().iterator(); itr
				.hasNext();) {
			Service service = itr.next();
			if (service.getOwlDescriptionFile().endsWith("/" + serviceUUID)) {
				itr.remove();
				removeServiceByName(service);
				removeServiceByType(service, serviceUUID);

				return service;
			}
		}
		return null;
	}

		

	/**
	 * registered a new service in the bag
	 * 
	 * @param service
	 *            the service to add
	 */
	@Override
	public void addService(Service service) {
		this.addServiceByName(service);
		this.addServiceInBag(service);
	}

	/**
	 * Clear the service bag
	 * 
	 */
	@Override
	public void clear() {
		this.nameToServiceMap.clear();
		this.typeToServiceMap.clear();
	}



	/**
	 * Go through the services contained into the repository to select the one
	 * that contains. <br>
	 * This alogrithm is o(n*m) where n=number of services, m= average number of
	 * functionalities per service
	 * 
	 * @param requiredCapa
	 *            the required Capability
	 * @return the match between the required Capability and the found one,
	 *         <i>empty list</i> if no match is found
	 */
	@Override
	public List<CapabilityMatch> getServicesContainingCapability(
			Capability requiredCapa) {
		List<CapabilityMatch> result = new ArrayList<CapabilityMatch>();

		/*
		 * Iterate all services
		 */
		Iterator<Service> serviceIterator = this.nameToServiceMap.values()
				.iterator();
		while (serviceIterator.hasNext()) {
			Service currentService = serviceIterator.next();

			/*
			 * Iterate all capability
			 */
			Iterator<Capability> listOfCapaIterator = currentService
					.getAllCapabilities().iterator();
			while (listOfCapaIterator.hasNext()) {
				Capability currentCapa = listOfCapaIterator.next();

				// match the Capability
				try {
					// DebugPrinter.print("match
					// "+currentCapa.getSemanticType()+" vs
					// "+requiredCapa.getSemanticType(), DebugPrinter.INFO);

					CapabilityMatch match = this.getMatcher().capabilityMatch(
							currentCapa, requiredCapa);

					// DebugPrinter.print("match = "+match, DebugPrinter.INFO);

					// test the match => if not null then add the match in the
					// list of results
					if (match != null)
						result.add(match);

				} catch (SemanticTypeNotKnowException e) {
					DebugPrinter.print(e.getStackTrace(), e.getMessage(),
							DebugPrinter.WARN);
				}
			}
		}
		return result;
	}

	

}
