/**
 ##############################################################################
 # Copyright (C) 2006-2010 INRIA
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 2 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ##############################################################################
 * 
 */
package fr.inria.icocoa.semrep.dataset;

import java.util.Iterator;
import java.util.List;
import java.util.Map;

import fr.inria.icocoa.semrep.dataset.dag.CapabilitiesDAG;
import fr.inria.icocoa.servicemodel.Capability;
import fr.inria.icocoa.servicemodel.Service;
import fr.inria.icocoa.servicemodel.matching.CapabilityMatch;
import fr.inria.icocoa.servicemodel.matching.Matcher;
import fr.inria.icocoa.servicemodel.semantictype.SemanticType;

/**
 * This class complete the implementation of ServiceBag by using a DAG to order
 * services into the repository
 * 
 * @author <a href="mailto:sebastien.bianco@inria.fr">sebastien bianco</a> - <a
 *         href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
 * 
 * Creation Date : Aug 27, 2008 10:09:52 AM
 */
public class ServiceDAGImplementation extends ServiceBag {

	/**
	 * class's version ID for serialization
	 */
	private static final long serialVersionUID = 1L;

	/**
	 * The dag that will contains all data
	 */
	private CapabilitiesDAG dagOfCapabilities = null;

	/**
	 * Default constructor. Initialise the DAg with the current semantic matcher
	 */
	public ServiceDAGImplementation() {
		this.dagOfCapabilities = new CapabilitiesDAG(this.getMatcher());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see fr.inria.icocoa.semrep.dataset.ServiceBag#addService(fr.inria.icocoa.servicemodel.Service)
	 */
	@Override
	public void addService(Service service) {
		// insert all capabilities in the dag
		for (Capability capa : service.getAllCapabilities()) {
			this.dagOfCapabilities.insertCapability(capa);
		}

		// update map of inserted service
		this.addServiceByName(service);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see fr.inria.icocoa.semrep.dataset.ServiceBag#clear()
	 */
	@Override
	public void clear() {
		this.dagOfCapabilities.clear();
		this.nameToServiceMap.clear();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see fr.inria.icocoa.semrep.dataset.ServiceBag#getServicesContainingCapability(fr.inria.icocoa.servicemodel.Capability)
	 */
	@Override
	public List<CapabilityMatch> getServicesContainingCapability(
			Capability requiredCapa) {
		return this.dagOfCapabilities.getAllCapabilityMatch(requiredCapa);
	}

	

	/* (non-Javadoc)
	 * @see fr.inria.icocoa.semrep.dataset.ServiceBag#addServiceByType(fr.inria.icocoa.servicemodel.Service)
	 */
	@Override
	protected void addServiceInBag(Service service) {
		//insert all capabilities of the service in the dag
		for(Capability capa : service.getAllCapabilities()){
			this.dagOfCapabilities.insertCapability(capa);
		}		
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see fr.inria.icocoa.semrep.dataset.ServiceBag#getServicesWithType(java.lang.String)
	 */
	@Override
	public String[] getServicesWithType(String serviceType) {
		
		//get the map of type
		Map<SemanticType, List<Service>> mapOfServicesWithType = this.getTypeToServiceMap();
		
		//get the right type by exploring the map of service
		for(SemanticType semanticType : mapOfServicesWithType.keySet()){
			if(semanticType.getType().equals(serviceType)){
				
				//get the list of service corresponding to the required semantic type
				List<Service> services = mapOfServicesWithType.get(semanticType);
				
				//format and return the result
				String[] result = new String[services.size()];
				for (int i = 0; i < services.size(); i++) {
					result[i] = services.get(i).getQualifiedName();
				}
				return result;
			}
		}
		
		
		//no result has been found
		return new String[0];
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see fr.inria.icocoa.semrep.dataset.ServiceBag#getTypeToServiceMap()
	 */
	@Override
	public Map<SemanticType, List<Service>> getTypeToServiceMap() {
		return this.dagOfCapabilities.getTypeToServiceMap();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see fr.inria.icocoa.semrep.dataset.ServiceBag#removeService(java.lang.String)
	 */
	@Override
	public Service removeService(String serviceUUID) {
		
		/*
		 * List the all list of servie to get the one with right UUID
		 */
		for (Iterator<Service> itr = this.getAllServiceObjects().iterator(); itr
				.hasNext();) {
			Service service = itr.next();
			if (service.getOwlDescriptionFile().endsWith("/" + serviceUUID)) {
				itr.remove();
				removeServiceByName(service);

				/*
				 * Remove each relative Capability of the dag
				 */
				for (Capability serviceCapa : service.getAllCapabilities()) {
						this.dagOfCapabilities.removeACapability(serviceCapa);
				}

				// return the removed service
				return service;
			}
		}
		return null;
	}
	
	/**
	 * @param matcher the matcher to set
	 */
	@Override
	public void setMatcher(Matcher matcher) {
		this.matcher = matcher;		
		this.dagOfCapabilities.setMatcher(matcher);
	}


}
