/**
 ##############################################################################
 # Copyright (C) 2006-2010 INRIA
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 2 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ##############################################################################
 * 
 */
package fr.inria.icocoa.semrep.dataset.dag;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.HashMap;
import java.util.Map;
import java.util.Stack;

import fr.inria.icocoa.common.DebugPrinter;
import fr.inria.icocoa.common.Tools;
import fr.inria.icocoa.exception.SemanticTypeNotKnowException;
import fr.inria.icocoa.servicemodel.Capability;
import fr.inria.icocoa.servicemodel.Service;
import fr.inria.icocoa.servicemodel.matching.CapabilityMatch;
import fr.inria.icocoa.servicemodel.matching.Matcher;
import fr.inria.icocoa.servicemodel.semantictype.SemanticType;
import fr.inria.icocoa.servicemodel.toolsobjects.Couple;

/**
 * This class implements a Directed Acyclic Graph (DAG) that will be used to
 * store and sort Capabilities so as to make the service discovery faster (EASY
 * model)
 * 
 * @author <a href="mailto:sebastien.bianco@inria.fr">sebastien bianco</a> - <a
 *         href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
 *         Creation Date : May 21, 2008 10:47:34 AM
 */
public class CapabilitiesDAG implements Serializable {

	/**
	 * the ID for Serialization
	 */
	private static final long serialVersionUID = 1L;

	/**
	 * The entry points of the CapabilitiesDAG (roots)
	 */
	private Map<SemanticType, CapabilitiesDAGVertex> roots;

	/**
	 * The matcher that will be use to insert object into the
	 */
	private Matcher matcher;

	/**
	 * The number of vertex of the dag
	 */
	private int size = 0;

	/**
	 * Create a CapabilitiesDAG object
	 * 
	 * @param matcher
	 *            the matcher that will be used to ordered nodes
	 */
	public CapabilitiesDAG(Matcher matcher) {
		this.roots = new HashMap<SemanticType, CapabilitiesDAGVertex>();
		this.matcher = matcher;
	}

	/**
	 * Insert a new Capability into the DAG of capabilities
	 * 
	 * @param capability
	 *            the capability to insert
	 * @return <i>true</i> if the insertion succeed, <i>false</i> if a problem
	 *         occurs
	 */
	public boolean insertCapability(Capability capability) {
		try {

			this.size++;// update the size of the dag. We consider that each
						// insert of Capability will create a new vertex. If
						// it's not the case (only if equivalent) then the size
						// will be corrected.

			// create Vertex pour this capability
			CapabilitiesDAGVertex newVertex = new CapabilitiesDAGVertex(
					capability, capability.getSemanticType(), this.getMatcher());

			// for the first element to insert
			if (this.getRoots().isEmpty()) {
				this.addARoot(newVertex);
				return true;
			}

			// if the new Vertex is added into the graph, an edge as to be
			// delete
			// => the boudaries of this edges must be stored
			List<CapabilitiesDAGVertex> fromVertex_EdgeToDelete = new ArrayList<CapabilitiesDAGVertex>();
			List<CapabilitiesDAGVertex> toVertex_EdgeToDelete = new ArrayList<CapabilitiesDAGVertex>();

			// Find predecessor of Capability in the graph
			// the algortihm ended if the vertex has been added as a root of the
			// DAG
			// If a new edge has been added inside the graph, then delete the
			// old edge
			boolean ended = this.findPredecessor(newVertex,
					fromVertex_EdgeToDelete);
			if (!ended) {

				// Find successors of Capability in the graph
				this.findSuccessor(newVertex, toVertex_EdgeToDelete);

				// check that an edge has been add in the middle of a sub-graph
				if (!fromVertex_EdgeToDelete.isEmpty()
						&& !toVertex_EdgeToDelete.isEmpty()) {

					// delete all edges
					for (CapabilitiesDAGVertex fromVertex_EdgeToDelete_item : fromVertex_EdgeToDelete) {
						for (CapabilitiesDAGVertex toVertex_EdgeToDelete_item : toVertex_EdgeToDelete) {
							this.deleteEdge(fromVertex_EdgeToDelete_item,
									toVertex_EdgeToDelete_item);
						}
					}
				}

				return true;
			} else {
				return true;
			}
		} catch (SemanticTypeNotKnowException e) {
			DebugPrinter.print(e, DebugPrinter.WARN);
			return false;
		}
	}

	/**
	 * Find the right place to insert the newVertex from roots of the DAG to its
	 * leaves. Update the referende fromVertex_EdgeToDelete with the first
	 * boundary of the edge that has been draw.
	 * 
	 * @param newVertex
	 *            the Vertex to add into the DAG
	 * @param fromVertex_EdgeToDelete
	 *            the list of nodes that starts the boundary of the edges that
	 *            will be delete
	 * @return <i>true</i> if the new vertex has been inserted as an equivalent
	 *         of another node, <i>false</i> otherwise
	 * @throws SemanticTypeNotKnowException
	 */
	private boolean findPredecessor(CapabilitiesDAGVertex newVertex,
			List<CapabilitiesDAGVertex> fromVertex_EdgeToDelete)
			throws SemanticTypeNotKnowException {
		// Stack of all node to visit
		// The couple contains as first element the current node to visit,
		// and as second element, the Predecessor of this Vertex
		Stack<Couple<CapabilitiesDAGVertex, CapabilitiesDAGVertex>> nodeToVisit = new Stack<Couple<CapabilitiesDAGVertex, CapabilitiesDAGVertex>>();

		// initialise the stack of node to visit with all roots of the DAG
		for (CapabilitiesDAGVertex root : this.getRoots().values()) {

			// initiate the Stack with the root (predecessor == null)
			nodeToVisit
					.add(new Couple<CapabilitiesDAGVertex, CapabilitiesDAGVertex>(
							root, null));
		}

		// visit node from node to successors until the match holds
		while (!nodeToVisit.isEmpty()) {

			// get the node to check
			Couple<CapabilitiesDAGVertex, CapabilitiesDAGVertex> coupleCurrentlyVisited = nodeToVisit
					.pop();
			CapabilitiesDAGVertex currenltyVisitedVertex = coupleCurrentlyVisited
					.getFirst();

			// check inclusiveCapabilityMatch for successors until it break
			if (this.getMatcher().inclusiveCapabilityMatch(
					currenltyVisitedVertex.getAssociatedObject(),
					newVertex.getAssociatedObject())) {

				// Check if the node has children, if not, then add a new leaf
				if (currenltyVisitedVertex.hasChildrenVertex()) {

					// if the inclusiveMatch holds, all the children must be
					// push at the top of the stack of nodes to visit
					for (CapabilitiesDAGVertex successor : currenltyVisitedVertex
							.getOutputEdges().values()) {
						nodeToVisit
								.push(new Couple<CapabilitiesDAGVertex, CapabilitiesDAGVertex>(
										successor, currenltyVisitedVertex));
					}
				} else {
					this.drawAnEdge(currenltyVisitedVertex, newVertex);
					return false;
				}

			} else {
				// the root of the subgraph doesn't match
				// => create a new root and end algorithm
				if (coupleCurrentlyVisited.getSecond() == null) {
					// if the list of rootToVisit is empty, add a new Root,
					// otherwise, check the other root
					if (nodeToVisit.isEmpty()) {
						this.addARoot(newVertex);
						return false;
					}
				} else {
					// check if the predecessor has an exact Match with the Node
					// => if yes it will share the same node and the algorithm
					// ended
					if (this.getMatcher().exactCapabilityMatch(
							coupleCurrentlyVisited.getSecond()
									.getAssociatedObject(),
							newVertex.getAssociatedObject())) {
						coupleCurrentlyVisited.getSecond()
								.addNewEquivalentCapability(
										newVertex.getAssociatedObject());
						this.size--;// update the size. We suppose that each
									// capability added create a new vertex, so,
									// as equivalent capability doesn't create
									// any edge, the size must be updated
						return true;
					} else {

						// We reach the level in the tree were the edge need to
						// be added
						// => add the new Edge and end to find predecessor
						// without empty the stack
						// => initialise the stating node of the edge to delete
						this.drawAnEdge(coupleCurrentlyVisited.getSecond(),
								newVertex);
						fromVertex_EdgeToDelete.add(coupleCurrentlyVisited
								.getSecond());
						return false;
					}
				}
			}
		}

		return false;
	}

	/**
	 * Find the right place to insert the newVertex from leaves of the DAG to
	 * its roots. Update the referende toVertex_EdgeToDelete with the second
	 * boundary of the edge that has been draw.
	 * 
	 * @param newVertex
	 *            the Vertex to add into the DAG
	 * @param toVertex_EdgeToDelete
	 *            the list of node that ends the boundary of the edges that will
	 *            be delete
	 * @return <i>true</i> if the new vertex has been inserted as a root of the
	 *         DAG, <i>false</i> otherwise
	 * @throws SemanticTypeNotKnowException
	 */
	private boolean findSuccessor(CapabilitiesDAGVertex newVertex,
			List<CapabilitiesDAGVertex> toVertex_EdgeToDelete)
			throws SemanticTypeNotKnowException {

		// Stack of all node to visit
		// The couple contains as first element the current node to visit,
		// and as second element, the Successors of this Vertex
		Stack<Couple<CapabilitiesDAGVertex, CapabilitiesDAGVertex>> nodeToVisit = new Stack<Couple<CapabilitiesDAGVertex, CapabilitiesDAGVertex>>();

		// initialise the stack of node to visit with all leaves of the DAG
		for (CapabilitiesDAGVertex leaf : this.getAllLeaves()) {

			// initiate the Stack with the root (predecessor == null)
			nodeToVisit
					.add(new Couple<CapabilitiesDAGVertex, CapabilitiesDAGVertex>(
							leaf, null));
		}

		// visit node from node to successors until the match holds
		while (!nodeToVisit.isEmpty()) {

			// get the node to check
			Couple<CapabilitiesDAGVertex, CapabilitiesDAGVertex> coupleCurrentlyVisited = nodeToVisit
					.pop();
			CapabilitiesDAGVertex currenltyVisitedVertex = coupleCurrentlyVisited
					.getFirst();

			// check that the leaf is not the vertex we want to add
			if (currenltyVisitedVertex != newVertex) {

				// check inclusiveCapabilityMatch for predecessors until it
				// break
				if (this.getMatcher().inclusiveCapabilityMatch(
						newVertex.getAssociatedObject(),
						currenltyVisitedVertex.getAssociatedObject())) {

					// if the inclusiveMatch holds, all the parents must be
					// push at the top of the stack of nodes to visit
					// BE CAREFULL : If the node is a root, then change the root
					if (currenltyVisitedVertex.isARoot()) {
						this.changeARoot(currenltyVisitedVertex, newVertex);
						this.drawAnEdge(newVertex, currenltyVisitedVertex);
					} else {
						for (CapabilitiesDAGVertex predecessor : currenltyVisitedVertex
								.getInputEdges().values()) {
							nodeToVisit
									.push(new Couple<CapabilitiesDAGVertex, CapabilitiesDAGVertex>(
											predecessor, currenltyVisitedVertex));
						}
					}

				} else {

					// We reach the level in the tree were the edge need to
					// be added
					// => add the new Edge and end algorithm
					// without empty the stack
					// => initialise the ended node of the edge to delete
					// BE CAREFULL :
					// * if the currentVertex is a leaf, it means
					// that the node must not be added here, so do nothing
					// * if the nod is a Root, the list of root must be updated
					if (coupleCurrentlyVisited.getSecond() != null) {

						/*
						 * if(currenltyVisitedVertex.isARoot()){
						 * DebugPrinter.print("reach a Root ",
						 * DebugPrinter.INFO);
						 * this.changeARoot(coupleCurrentlyVisited .getSecond(),
						 * newVertex); }
						 */
						this.drawAnEdge(newVertex, coupleCurrentlyVisited
								.getSecond());
						toVertex_EdgeToDelete.add(coupleCurrentlyVisited
								.getSecond());
					} else {
					}

				}
			} else {
				// the node to insert is a leaf
				// => check the next leave
				// => Do nothing
			}
		}

		return false;
	}

	/**
	 * Add a new root in the list of Root
	 * 
	 * @param newVertex
	 *            the vertex to add
	 */
	public void addARoot(CapabilitiesDAGVertex newVertex) {
		this.getRoots().put(newVertex.getVertexName(), newVertex);
		newVertex.setARoot(true);
	}

	/**
	 * Change a root in the DAG (remove the old one, and add the new one)
	 * 
	 * @param oldVertex
	 *            the old vertex that must be remove from the list of root
	 * @param newVertex
	 *            the new vertes that must be add into the list of root
	 */
	public void changeARoot(CapabilitiesDAGVertex oldVertex,
			CapabilitiesDAGVertex newVertex) {
		this.removeARoot(oldVertex);
		this.addARoot(newVertex);
	}

	/**
	 * Remove a node from the list of roots <br>
	 * 
	 * <font color="#FF0000"><b><u>WARNING:</u></b></font> this method
	 * don't remove the vertex from the dag, but just remove the vertex from the
	 * list of roots
	 * 
	 * @param toRemove
	 *            the Vertex to remove
	 */
	public void removeARoot(CapabilitiesDAGVertex toRemove) {
		// DebugPrinter.print(this.getRoots(), "remove A Root start",
		// DebugPrinter.INFO);
		this.getRoots().remove(toRemove.getVertexName());
		// DebugPrinter.print(this.getRoots(), "remove A Root end",
		// DebugPrinter.INFO);
		toRemove.setARoot(false);
	}

	/**
	 * Get all leaves of the DAG
	 * 
	 * @return a list of vertex that have no outgoing edge
	 */
	public List<CapabilitiesDAGVertex> getAllLeaves() {
		List<CapabilitiesDAGVertex> result = new ArrayList<CapabilitiesDAGVertex>();
		for (CapabilitiesDAGVertex root : this.getRoots().values()) {
			// if the root has no successors then it is a leaf, otherwise, we
			// need to check if its successors are leaves
			if (root.getSuccessors().isEmpty()) {
				result.add(root);
			} else
				result.addAll(root.getAllLeaves());
		}
		return result;
	}

	/**
	 * Draw a directed Edge between to node
	 * 
	 * @param nodeParent
	 *            the parent node
	 * @param nodeChild
	 *            the child node
	 */
	public void drawAnEdge(CapabilitiesDAGVertex nodeParent,
			CapabilitiesDAGVertex nodeChild) {
		// DebugPrinter.print("draw an edge " + nodeParent.getVertexName() +
		// "->"
		// + nodeChild.getVertexName(), DebugPrinter.INFO);
		nodeParent.addNewChildNode(nodeChild);
		nodeChild.addNewParentNode(nodeParent);
	}

	/**
	 * Delete an edge of the DAG
	 * 
	 * @param fromVertex_EdgeToDelete
	 *            the starting boundary of the edge
	 * @param toVertex_EdgeToDelete
	 *            the ending boundary of the edge
	 */
	public void deleteEdge(CapabilitiesDAGVertex fromVertex_EdgeToDelete,
			CapabilitiesDAGVertex toVertex_EdgeToDelete) {
		// DebugPrinter.print("delete an edge "
		// + fromVertex_EdgeToDelete.getVertexName() + "->"
		// + toVertex_EdgeToDelete.getVertexName(), DebugPrinter.INFO);

		fromVertex_EdgeToDelete.getOutputEdges().remove(
				toVertex_EdgeToDelete.getVertexName());
		toVertex_EdgeToDelete.getInputEdges().remove(
				fromVertex_EdgeToDelete.getVertexName());
	}

	/**
	 * Clean the dag
	 * 
	 */
	public void clear() {
		this.roots.clear();
	}

	/**
	 * Get the vertex corresponding to a given Capability
	 * 
	 * @param required
	 *            the resuired Capability
	 * @return the vertex object corresponding to the required capability,
	 *         <b>null</b> if no such capability is contained into the dag.
	 */
	public CapabilitiesDAGVertex getVertex(Capability required) {

		// search the element root after root in the forest
		for (CapabilitiesDAGVertex root : this.getRoots().values()) {
			// check that the required in on the current sub-dag
			CapabilitiesDAGVertex found = root.getVertex(required);
			if (found != null)
				return found;
		}

		return null;
	}

	/**
	 * Get all Match with the capability
	 * 
	 * @param required
	 *            the capability to match with
	 * @return he match between the required Capability and the found one,
	 *         <i>empty list</i> if no match is found
	 */
	public List<CapabilityMatch> getAllCapabilityMatch(Capability required) {
		List<CapabilityMatch> result = new ArrayList<CapabilityMatch>();

		// search the element root after root in the forest
		for (CapabilitiesDAGVertex root : this.getRoots().values()) {
			// get all match in the corresponding sub-dag
			result.addAll(root.getAllCapabilityMatch(required));
		}

		return result;
	}

	/**
	 * Remove a Capability of the dag
	 * 
	 * @param serviceCapa
	 *            the Capability to remove
	 */
	public void removeACapability(Capability serviceCapa) {
		if (serviceCapa.getService() != null) {
			this.removeATypeOfCapability(serviceCapa.getSemanticType(),
					serviceCapa.getService().getServiceName());
		} else {
			this.removeATypeOfCapability(serviceCapa.getSemanticType(), "*");
		}
	}

	/**
	 * Remove a type of capability in the dag depending the name of the service.<br>
	 * "*" means that all capabilities with such type will be remove. "*xxx"
	 * means that all capabilities of services which names ends with xxx will be
	 * deleted. "xxx" means that only capabilities of service with name xxx will
	 * be deleted.
	 * 
	 * @param typeToRemove
	 *            the type of capability to remove
	 * @param nameOfServiceTemplate
	 *            the template use as name of service to remove.
	 */
	public void removeATypeOfCapability(SemanticType typeToRemove,
			String nameOfServiceTemplate) {

		// initialise the list of vertex to remove
		List<CapabilitiesDAGVertex> listOfVertexToRemove = new ArrayList<CapabilitiesDAGVertex>();

		// go through the roots to remove the capability
		for (CapabilitiesDAGVertex root : this.getRoots().values()) {
			listOfVertexToRemove.addAll(root.removeATypeOfCapability(
					typeToRemove, nameOfServiceTemplate));
		}

		// delete all useless vertex
		for (CapabilitiesDAGVertex vertexToDelete : listOfVertexToRemove) {
			this.removeAVertex(vertexToDelete);
		}

	}

	/**
	 * Remove a Vertex of the DAG
	 * 
	 * @param vertexToDelete
	 *            the vertex to remove
	 */
	public void removeAVertex(CapabilitiesDAGVertex vertexToDelete) {

		/*
		 * Check if the vertex to remove is a root or not - if it's a root,
		 * remove it from the list of root and add successors in the list of
		 * roots - if it's not a root, all successors of the vertex must be add
		 * to the lis of successors of all predecessors
		 */
		if (vertexToDelete.isARoot()) {
			// get the list of successors and add it as new root of the dag
			for (CapabilitiesDAGVertex successors : vertexToDelete
					.getSuccessors()) {
				this.addARoot(successors);
			}

			// remove the vertex from the list of roots
			this.removeARoot(vertexToDelete);
		} else {
			// put all successors of the vertex to delete as successors of all
			// predecessors of the vertex to delete
			for (CapabilitiesDAGVertex predecessor : vertexToDelete
					.getPredecessors()) {
				// add new successors
				for (CapabilitiesDAGVertex successor : vertexToDelete
						.getSuccessors()) {
					this.drawAnEdge(predecessor, successor);
				}
			}
		}

		/*
		 * destroy the object
		 */

		// delete all incoming/outgoing edges
		for (CapabilitiesDAGVertex successor : vertexToDelete.getSuccessors()) {
			this.deleteEdge(vertexToDelete, successor);
		}
		for (CapabilitiesDAGVertex predecessor : vertexToDelete
				.getPredecessors()) {
			this.deleteEdge(predecessor, vertexToDelete);
		}

		// delete object
		vertexToDelete = null;

		// update size of the dag
		this.size--;

	}
	
	/**
	 * Get the list of services that contains a given semantic type (collected and sorted by Semantic Type)
	 * @return the typeToServiceMap the map serviceType <-> List of services
	 */
	public Map<SemanticType, List<Service>> getTypeToServiceMap(){
		Map<SemanticType, List<Service>> result = new HashMap<SemanticType, List<Service>>();
		
		/*
		 * Go through all sub-dag to collect the list of services associated to each Capability
		 */
		for(CapabilitiesDAGVertex root : this.getRoots().values()){
			result.putAll(root.collectTypeToServiceMap());
		}
		
		return result;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public int hashCode() {
		final int PRIME = 31;
		int result = 1;
		result = PRIME * result
				+ ((getRoots() == null) ? 0 : getRoots().hashCode());
		return result;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		final CapabilitiesDAG other = (CapabilitiesDAG) obj;
		if (getRoots() == null) {
			if (other.getRoots() != null)
				return false;
		} else if (!Tools.compareMap(getRoots(), other.getRoots()))
			return false;
		return true;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		String result = "";

		for (CapabilitiesDAGVertex root : this.getRoots().values()) {
			result += " - " + root.toString("  ") + "\n";
		}

		return result;
	}

	/**
	 * @return the roots
	 */
	public Map<SemanticType, CapabilitiesDAGVertex> getRoots() {
		return roots;
	}

	/**
	 * @param roots
	 *            the roots to set
	 */
	public void setRoots(Map<SemanticType, CapabilitiesDAGVertex> roots) {
		this.roots = roots;
	}

	/**
	 * @return the matcher
	 */
	public Matcher getMatcher() {
		return matcher;
	}

	/**
	 * @param matcher
	 *            the matcher to set
	 */
	public void setMatcher(Matcher matcher) {
		this.matcher = matcher;
	}

	/**
	 * @return the size
	 */
	public Object getSize() {
		return this.size;
	}

}
