/**
 ##############################################################################
 # Copyright (C) 2006-2010 INRIA
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 2 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ##############################################################################
 * 
 */
package fr.inria.icocoa.semrep.dataset.dag;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.regex.Pattern;

import fr.inria.icocoa.common.DebugPrinter;
import fr.inria.icocoa.common.Tools;
import fr.inria.icocoa.exception.SemanticTypeNotKnowException;
import fr.inria.icocoa.servicemodel.Capability;
import fr.inria.icocoa.servicemodel.Service;
import fr.inria.icocoa.servicemodel.matching.CapabilityMatch;
import fr.inria.icocoa.servicemodel.matching.Matcher;
import fr.inria.icocoa.servicemodel.semantictype.SemanticType;

/**
 * This class represents a Node of a CapabilitiesDAG.
 * 
 * @author <a href="mailto:sebastien.bianco@inria.fr">sebastien bianco</a> - <a
 *         href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
 * 
 * 
 * Creation Date : May 21, 2008 10:22:02 AM
 */
public class CapabilitiesDAGVertex implements Serializable {

	/* ******************** */
	/* *** Class Fields *** */
	/* ******************** */

	/**
	 * the ID for Serialization
	 */
	private static final long serialVersionUID = 1L;

	/**
	 * The object associated to the node
	 */
	private Capability associatedObject;

	/**
	 * The list of entering edge
	 */
	private Map<SemanticType, CapabilitiesDAGVertex> inputEdges;

	/**
	 * The list of outgoing edge
	 */
	private Map<SemanticType, CapabilitiesDAGVertex> outputEdges;

	/**
	 * The assocated name of the Vertex
	 */
	private SemanticType vertexName;

	/**
	 * The matcher that will be use to insert object into the
	 */
	private Matcher matcher;

	/**
	 * list of equivalent capabilities (corresponding probably to other
	 * services)
	 */
	private List<Capability> listOfEquivalentCapability;

	/**
	 * Indicate if the Vertex is a root of the DAG
	 */
	private boolean isARoot = false;

	/* ******************** */
	/* *** Constructors *** */
	/* ******************** */

	/**
	 * Create a new Vertex (not a root)
	 * 
	 * @param associatedObject
	 *            the object associated to this node
	 * @param vertexName
	 *            the associated name of the vertex
	 * @param matcher
	 *            the matcher that will be used to ordered nodes
	 */
	public CapabilitiesDAGVertex(Capability associatedObject,
			SemanticType vertexName, Matcher matcher) {
		this(associatedObject, vertexName, matcher, false);
	}

	/**
	 * Create a new Vertex
	 * 
	 * @param associatedObject
	 *            the object associated to this node
	 * @param vertexName
	 *            the associated name of the vertex
	 * @param matcher
	 *            the matcher that will be used to ordered nodes
	 * @param aRoot
	 *            indicate if the new Vertex is aRoot or not
	 */
	public CapabilitiesDAGVertex(Capability associatedObject,
			SemanticType vertexName, Matcher matcher, boolean aRoot) {
		this.associatedObject = associatedObject;
		this.vertexName = vertexName;
		this.inputEdges = new HashMap<SemanticType, CapabilitiesDAGVertex>();
		this.outputEdges = new HashMap<SemanticType, CapabilitiesDAGVertex>();
		this.matcher = matcher;
		this.listOfEquivalentCapability = new LinkedList<Capability>();

		// add the associatedObject in the list of equivalents
		this.listOfEquivalentCapability.add(associatedObject);
		this.isARoot = aRoot;
	}

	/* ******************** */
	/* *** Main methods *** */
	/* ******************** */

	/**
	 * Add a new Capability that share the same node (ie
	 * ExactCapabilityMatch(capability,this.associatedObject)==true)
	 * 
	 * @param capability
	 *            the capability to add as equivalent
	 * @return <i>true</i> if the Capability has been add, <i>false</i> if
	 *         those capability was already in the list of equivalent
	 */
	public boolean addNewEquivalentCapability(Capability capability) {
		// check that the list doesn't contain already the object so as to avoid
		// doublon in the list
		if (this.getListOfEquivalentCapability().contains(capability)) {
			return false;
		} else {
			return this.getListOfEquivalentCapability().add(capability);
		}
	}

	/**
	 * Add a new Node in the list of outgoing vertex (check if no similar edge
	 * exists)
	 * 
	 * @param nodeChild
	 *            the outgoing vertex to add
	 */
	public void addNewChildNode(CapabilitiesDAGVertex nodeChild) {
		if (!this.getOutputEdges().containsKey(nodeChild.getVertexName())) {
			this.getOutputEdges().put(nodeChild.getVertexName(), nodeChild);
		}

	}

	/**
	 * Add a new Node in the list of entering vertex (check if no similar edge
	 * exists)
	 * 
	 * @param nodeParent
	 *            the entering vertex to add
	 */
	public void addNewParentNode(CapabilitiesDAGVertex nodeParent) {
		if (!this.getInputEdges().containsKey(nodeParent.getVertexName())) {
			this.getInputEdges().put(nodeParent.getVertexName(), nodeParent);
		}

	}

	/**
	 * Check if the current node has some children
	 * 
	 * @return <i>true</i> if the current Node has outgoing edge, <i>false</i>
	 *         otherwise.
	 */
	public boolean hasChildrenVertex() {
		return !this.getOutputEdges().isEmpty();
	}

	/**
	 * Get all leaves of the sub-graph starting with the current vertex
	 * 
	 * @return the list of leaves (a list contraining only the current vertex if
	 *         the current vertex is a leaf)
	 */
	public List<CapabilitiesDAGVertex> getAllLeaves() {
		List<CapabilitiesDAGVertex> result = new ArrayList<CapabilitiesDAGVertex>();
		if (this.hasChildrenVertex()) {
			for (CapabilitiesDAGVertex child : this.getOutputEdges().values()) {
				result.addAll(child.getAllLeaves());
			}
		} else {
			result.add(this);
		}
		return result;
	}

	/**
	 * Get the list of successors of the current vertex
	 * 
	 * @return a list of succesors, an empty list if the current node is a leaf;
	 */
	public List<CapabilitiesDAGVertex> getSuccessors() {
		return new ArrayList<CapabilitiesDAGVertex>(this.getOutputEdges()
				.values());
	}

	/**
	 * Get the list of predecessors of the current vertex
	 * 
	 * @return a list of predecessors, an empty list if the current node is a
	 *         root;
	 */
	public List<CapabilitiesDAGVertex> getPredecessors() {
		return new ArrayList<CapabilitiesDAGVertex>(this.getInputEdges()
				.values());
	}

	/* **************************** */
	/* *** Miscellaneous methods *** */
	/* **************************** */

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		return this.toString("");
	}

	/**
	 * Get a string representation of the current object
	 * 
	 * @param prefix
	 *            the string to put at the beginning of every line
	 * @return a String that represent the contain of the current object
	 */
	public String toString(String prefix) {
		String result = ((this.isARoot()) ? "(R) " : "")
				+ this.getVertexName().toString() + " ("
				+ this.getAssociatedObject().getLocalName() + ")\n";

		for (Capability equivalent : this.getListOfEquivalentCapability()) {
			result += prefix + " = " + equivalent.getLocalName() + "\n";
		}

		for (CapabilitiesDAGVertex outgoing : this.getOutputEdges().values())
			result += prefix + "   +--> " + outgoing.toString(prefix + "   ");

		return result;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public int hashCode() {
		final int PRIME = 31;
		int result = 1;
		// result = PRIME * result + ((getAssociatedObject() == null) ? 0 :
		// getAssociatedObject().hashCode());
		// result = PRIME * result + ((getInputEdges() == null) ? 0 :
		// getInputEdges().hashCode());
		result = PRIME * result + (isARoot() ? 1231 : 1237);
		result = PRIME
				* result
				+ ((getListOfEquivalentCapability() == null) ? 0
						: getListOfEquivalentCapability().hashCode());
		result = PRIME
				* result
				+ ((getOutputEdges() == null) ? 0 : getOutputEdges().hashCode());
		result = PRIME * result
				+ ((getVertexName() == null) ? 0 : getVertexName().hashCode());
		return result;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		final CapabilitiesDAGVertex other = (CapabilitiesDAGVertex) obj;
		/*
		 * if (getAssociatedObject() == null) { if (other.getAssociatedObject() !=
		 * null) return false; } else if
		 * (!getAssociatedObject().equals(other.getAssociatedObject())) return
		 * false;
		 */
		if (isARoot() != other.isARoot())
			return false;
		if (getListOfEquivalentCapability() == null) {
			if (other.getListOfEquivalentCapability() != null)
				return false;
		} else if (!Tools.compareList(getListOfEquivalentCapability(), other
				.getListOfEquivalentCapability()))
			return false;
		if (getVertexName() == null) {
			if (other.getVertexName() != null)
				return false;
		} else if (!getVertexName().equals(other.getVertexName()))
			return false;

		/*
		 * Check recursivly
		 */
		if (getOutputEdges() == null) {
			if (other.getOutputEdges() != null)
				return false;
		} else if (!Tools.compareList(new ArrayList<SemanticType>(
				getOutputEdges().keySet()), new ArrayList<SemanticType>(other
				.getOutputEdges().keySet())))
			return false;

		Iterator<SemanticType> keysIteraotor = getOutputEdges().keySet()
				.iterator();
		while (keysIteraotor.hasNext()) {
			SemanticType key = keysIteraotor.next();
			if (!getOutputEdges().get(key).equals(
					other.getOutputEdges().get(key)))
				return false;
		}

		return true;
	}

	/* ************************* */
	/* *** Getters / Setters *** */
	/* ************************* */

	/**
	 * @return the associatedObject
	 */
	public Capability getAssociatedObject() {
		return associatedObject;
	}

	/**
	 * @param associatedObject
	 *            the associatedObject to set
	 */
	public void setAssociatedObject(Capability associatedObject) {
		this.associatedObject = associatedObject;
	}

	/**
	 * @return the inputEdges
	 */
	public Map<SemanticType, CapabilitiesDAGVertex> getInputEdges() {
		return inputEdges;
	}

	/**
	 * @param inputEdges
	 *            the inputEdges to set
	 */
	public void setInputEdges(
			Map<SemanticType, CapabilitiesDAGVertex> inputEdges) {
		this.inputEdges = inputEdges;
	}

	/**
	 * @return the outputEdges
	 */
	public Map<SemanticType, CapabilitiesDAGVertex> getOutputEdges() {
		return outputEdges;
	}

	/**
	 * @param outputEdges
	 *            the outputEdges to set
	 */
	public void setOutputEdges(
			Map<SemanticType, CapabilitiesDAGVertex> outputEdges) {
		this.outputEdges = outputEdges;
	}

	/**
	 * @return the vertexName
	 */
	public SemanticType getVertexName() {
		return vertexName;
	}

	/**
	 * @param vertexName
	 *            the vertexName to set
	 */
	public void setVertexName(SemanticType vertexName) {
		this.vertexName = vertexName;
	}

	/**
	 * @return the matcher
	 */
	public Matcher getMatcher() {
		return matcher;
	}

	/**
	 * @param matcher
	 *            the matcher to set
	 */
	public void setMatcher(Matcher matcher) {
		this.matcher = matcher;
	}

	/**
	 * @return the listOfEquivalentCapability
	 */
	public List<Capability> getListOfEquivalentCapability() {
		return listOfEquivalentCapability;
	}

	/**
	 * @param listOfEquivalentCapability
	 *            the listOfEquivalentCapability to set
	 */
	public void setListOfEquivalentCapability(
			List<Capability> listOfEquivalentCapability) {
		this.listOfEquivalentCapability = listOfEquivalentCapability;
	}

	/**
	 * @return the isARoot
	 */
	public boolean isARoot() {
		return isARoot;
	}

	/**
	 * @param isARoot
	 *            the isARoot to set
	 */
	public void setARoot(boolean isARoot) {
		this.isARoot = isARoot;
	}

	/**
	 * Get the vertex with the given capability
	 * 
	 * @param required
	 *            the required capabiliy
	 * @return the vertex that correspond to the capability, <b>null</b> if the
	 *         current dag doesn't contains it, or if there is a trouble
	 *         (SemanticTypeException)
	 */
	public CapabilitiesDAGVertex getVertex(Capability required) {

		try {

			// get the match between concept
			CapabilityMatch match = this.getMatcher().capabilityMatch(
					this.getAssociatedObject(), required);

			// check that there is a match
			if (match == null)
				return null;

			/* check the match to see if it is an exact match */

			// check current node
			if (match.isExact())
				return this;

			// check if the required capability could be in the current dag (=>
			// inclusive match)
			else if (match.isInclusive()) {
				// if there is an inclusive match, try to get exact match in one
				// of the successors
				for (CapabilitiesDAGVertex succededVertex : this
						.getSuccessors()) {
					CapabilitiesDAGVertex result = succededVertex
							.getVertex(required);
					if (result != null)
						return result;
				}

			}
			// it doesn't match neither inclusively neither exactly but it match
			// (very weak match)
			return this;

		} catch (SemanticTypeNotKnowException e) {
			DebugPrinter.print(e, DebugPrinter.INFO);
			return null;
		}
	}

	/**
	 * Get all Match with the capability
	 * 
	 * @param required
	 *            the capability to match with
	 * @return he match between the required Capability and the found one,
	 *         <i>empty list</i> if no match is found, <b>null</b> if a
	 *         SemanticTypeException is thrown
	 */
	public List<CapabilityMatch> getAllCapabilityMatch(Capability required) {
		try {

			// get the match between concept
			//TODO Check change
			/*CapabilityMatch match = this.getMatcher().capabilityMatch(required,
					this.getAssociatedObject());*/
			CapabilityMatch match = this.getMatcher().capabilityMatch(
					this.getAssociatedObject(),required);

			// check that there is a match
			if (match == null)
				return new ArrayList<CapabilityMatch>();

			// create the table of matching and add this first matching
			List<CapabilityMatch> result = new ArrayList<CapabilityMatch>();
			result.add(match);

			// check if the required capability could be in the current dag (=>
			// inclusive match)
			for (CapabilitiesDAGVertex succededVertex : this.getSuccessors()) {
				result.addAll(succededVertex.getAllCapabilityMatch(required));
			}

			// return the list of collected match
			return result;

		} catch (SemanticTypeNotKnowException e) {
			DebugPrinter.print(e, DebugPrinter.INFO);
			return null;
		}
	}

	/**
	 * Remove a type of capability in the dag depending the name of the service.<br>
	 * "*" means that all capabilities with such type will be remove. "*xxx"
	 * means that all capabilities of services which names ends with xxx will be
	 * deleted. "xxx" means that only capabilities of service with name xxx will
	 * be deleted.
	 * 
	 * @param typeToRemove
	 *            the type of capability to remove
	 * @param nameOfServiceTemplate
	 *            the template use as name of service to remove.
	 * @return a list of Vertex with such Capability (so vertex have to be
	 *         removed), <i>empty list</i> if the subgraph contains no vertex
	 *         with such capability
	 */
	public List<CapabilitiesDAGVertex> removeATypeOfCapability(
			SemanticType typeToRemove, String nameOfServiceTemplate) {

		// create the list of vertex to remove
		List<CapabilitiesDAGVertex> vertexToRemove = new ArrayList<CapabilitiesDAGVertex>();

		/*
		 * Launch recursive delete of the capability
		 */
		for (CapabilitiesDAGVertex nextVertex : this.getSuccessors()) {
			vertexToRemove.addAll(nextVertex.removeATypeOfCapability(
					typeToRemove, nameOfServiceTemplate));
		}

		/*
		 * Check current Vertex and remove Capa
		 */
		if (this.getAssociatedObject().getSemanticType().equals(typeToRemove)) {

			/*
			 * remove capability one after the other until the owner of vertex
			 * change
			 */
			boolean aChangeOfOwner = false;// indicate if the owner of the
			// vertex has changed during the
			// loop

			do {
				/*
				 * Check the current Capability object
				 */
				if (this.getAssociatedObject().getService() == null
						|| this.checkTheNameOfTheService(this
								.getAssociatedObject().getService()
								.getLocalName(), nameOfServiceTemplate)) {
					/*
					 * it's the right capability to delete if there is no
					 * equivalent capability, remove the capability (and
					 * corresponding node), else change the "owner" of the
					 * vertex with the first equivalent
					 */
					if (this.getListOfEquivalentCapability().isEmpty()) {
						vertexToRemove.add(this);
						aChangeOfOwner = false;
					} else {
						this.setAssociatedObject(this
								.getListOfEquivalentCapability().get(0));// associate
						// new
						// object
						this.getListOfEquivalentCapability().remove(0);// delete
						// element
						// from
						// the
						// the
						// list
						// of
						// equivalent
						aChangeOfOwner = true; // update the indicator so as to
						// make another run of the loop
					}
				}
			} while (aChangeOfOwner);

			/*
			 * Check equivalents
			 */
			Iterator<Capability> equivalentIterator = this
					.getListOfEquivalentCapability().iterator();
			while (equivalentIterator.hasNext()) {
				Capability iteratedCapa = equivalentIterator.next();
				// if the name of the service corresponds to the template,
				// remove the element from the list of equivalent, otherwise, do
				// nothing
				if (this.checkTheNameOfTheService(iteratedCapa.getService()
						.getLocalName(), nameOfServiceTemplate)) {
					equivalentIterator.remove();
				}
			}

		}

		// return the list of vertex to remove
		return vertexToRemove;

	}

	/**
	 * Check if the name of the Service match the local name or not
	 * @param localName the name of the service
	 * @param nameOfServiceTemplate the template to match with
	 * @return <b>true</b> if the name of the service match the template, <b>false</b> otherwise.
	 */
	private boolean checkTheNameOfTheService(String localName,
			String nameOfServiceTemplate) {
		//check if the template is "*", in that case all services match with the template
		if (nameOfServiceTemplate.equals("*"))
			return true;

		//convert the template into a pattern of a regular expression then try to match the name of the service		
		Pattern p = Pattern.compile(nameOfServiceTemplate);
		java.util.regex.Matcher m = p.matcher(localName);
		return m.matches();
	}

	/**
	 * Get the list of services that contains a given semantic type (collected and sorted by Semantic Type)
	 * @return result the map where list of services must be collected
	 */
	public Map<SemanticType, List<Service>> collectTypeToServiceMap() {

		//create map of result
		Map<SemanticType, List<Service>> result = new HashMap<SemanticType, List<Service>>();

		/*
		 * Launch recursive walk-though
		 */
		for (CapabilitiesDAGVertex sucessors : this.getSuccessors()) {
			result.putAll(sucessors.collectTypeToServiceMap());
		}

		/*
		 * treat current vertex
		 */
		SemanticType currentType = this.getAssociatedObject().getSemanticType();

		//check if the semantic type already exist in the map of result. If it's the case, get the list of service and complete it, otherwise create the list of service
		List<Service> services = null;
		if (result.containsKey(currentType)) {
			services = result.get(currentType);
		} else {
			services = new ArrayList<Service>();
			result.put(currentType, services);
		}

		//add the service associated to the current capability in the list of service if it's not contained yet (and is not null)
		Service currentService = this.getAssociatedObject().getService();
		if (currentService != null && !services.contains(currentService)) {
			services.add(currentService);
		}

		// Check equivalent capability
		for (Capability equivalentCapa : this.getListOfEquivalentCapability()) {
			//add the service associated to the current capability in the list of service if it's not contained yet (and is not null)
			Service equivalentService = equivalentCapa.getService();
			if (equivalentService != null
					&& !services.contains(equivalentService)) {
				services.add(equivalentService);
			}
		}
		
		

		//return the map of result
		return result;
	}

}
