package fr.inria.icocoa.semrep.osgi;

import org.apache.log4j.Logger;
import org.ungoverned.gravity.servicebinder.Lifecycle;

import com.francetelecom.amigo.core.AmigoException;
import com.francetelecom.amigo.core.AmigoExportedService;
import com.francetelecom.amigo.core.AmigoLdapLookup;
import com.francetelecom.amigo.core.AmigoReference;
import com.francetelecom.amigo.core.AmigoServiceExporter;

import fr.inria.icocoa.semrep.SemanticRepository;
import fr.inria.icocoa.semrep.SemanticRepositoryFactory;

/**
 * 
 * The core of the implementation of the OSGi Service
 * 
 * @author <a href="mailto:sebastien.bianco@inria.fr">sebastien bianco</a> - <a
 *         href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
 * 
 * Creation Date : Aug 27, 2008 10:27:10 AM
 */
public class SemanticRepositoryComponent implements Lifecycle {

	/**
	 * The service exporter
	 */
	private AmigoServiceExporter serviceExporter;

	/**
	 * The log engine
	 */
	private Logger log;

	/**
	 * The reference to the Amigo LDAP database use to lookup other services
	 */
	private AmigoLdapLookup lookup;

	/**
	 * A reference to the current service exporter
	 */
	private AmigoExportedService myService;

	/**
	 * Default constructor (initialise logger)
	 * 
	 */
	public SemanticRepositoryComponent() {
		super();
		log = Logger.getLogger(this.getClass().getName());
	}

	/**
	 * This method is called by the OSGi engine so as to bind the current
	 * service
	 * 
	 * @param serviceExporter
	 *            the current service exporter
	 */
	public void bindServiceExporter(AmigoServiceExporter serviceExporter) {
		this.serviceExporter = serviceExporter;
	}

	/**
	 * This method is called by the OSGi engine so as to unbind the current
	 * service
	 * 
	 * @param serviceExporter
	 *            the current service exporter
	 */
	public void unbindServiceExporter(AmigoServiceExporter serviceExporter) {
		if (this.serviceExporter == serviceExporter) {
			this.serviceExporter = null;
		}
	}

	/**
	 * This method is called by the OSGi engine so as to bind the current lookup
	 * service
	 * 
	 * @param lookup
	 *            the current lookup service of the LDAP amigo database
	 */
	public void bindLookupService(AmigoLdapLookup lookup) {
		this.lookup = lookup;
		if (myService != null) {
			try {
				lookup.registerService(myService);
			} catch (AmigoException e) {
				log
						.warn(
								"Unable to bind lookup service for Amigo Semantic Repository.",
								e);
			}
		}
	}

	/**
	 * This method is called by the OSGi engine so as to unbind the current
	 * lookup service
	 * 
	 * @param lookup
	 *            the current lookup service of the LDAP amigo database
	 */
	public void unbindLookupService(AmigoLdapLookup lookup) {
		if (this.lookup == lookup) {
			if (myService != null) {
				try {
					lookup.unregisterService(myService);
				} catch (AmigoException e) {
					log
							.warn(
									"Unable to unbind lookup service for Amigo Semantic Repository.",
									e);
				}
			}
			this.lookup = null;
		}
	}

	public void activate() {
		log.info("Activating Amigo Semantic Repository...");
		SemanticRepository semrep = SemanticRepositoryFactory.getInstance();

		myService = serviceExporter.createService(semrep);
		try {
			log.info("Exporting Amigo Semantic Repository...");
			myService.addProperty("Scope", "urn:amigo");
			myService.addProperty("oid", "AmigoSemanticRepository");
			myService.addProperty("serviceType", "SemanticServiceRepository");
			myService.exportInterface(AmigoReference.DEFAULT,
					SemanticRepository.class);
			log.info("Exporting Amigo Semantic Repository with reference "
					+ myService.getReference());
		} catch (AmigoException e) {
			log.warn("Unable to export Amigo Semantic Repository.", e);
		}
		myService.addProperty("ServiceType", "SemanticServiceRepository");

		if (lookup != null) {
			try {
				lookup.registerService(myService);
				log.info("myService has been registered (lookup =" + lookup
						+ ", myService=" + myService + ")");
			} catch (AmigoException e) {
				log.warn("Unable to register Amigo Semantic Repository.", e);
			}
		}

	}

	public void deactivate() {
		log.info("Desactivating Amigo Semantic Repository...");
		if (myService != null) {
			if (lookup != null) {
				try {
					lookup.unregisterService(myService);
				} catch (AmigoException e) {
					log.warn("Unable to unregister Amigo Semantic Repository.",
							e);
				}
			}
		}
	}

}
