/**
 ##############################################################################
 # Copyright (C) 2006-2010 INRIA
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 2 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ##############################################################################
 * 
 */
package fr.inria.icocoa.servicediscovery;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import fr.inria.icocoa.common.Tools;
import fr.inria.icocoa.servicemodel.Capability;
import fr.inria.icocoa.servicemodel.IO;
import fr.inria.icocoa.servicemodel.Input;
import fr.inria.icocoa.servicemodel.Output;
import fr.inria.icocoa.servicemodel.Service;
import fr.inria.icocoa.servicemodel.UserTask;
import fr.inria.icocoa.servicemodel.flow.CapabilityFlow;
import fr.inria.icocoa.servicemodel.matching.CapabilityMatch;
import fr.inria.icocoa.servicemodel.matching.IOMatch;
import fr.inria.icocoa.servicemodel.toolsobjects.ServiceMap;

/**
 * This class store all information gathered during the Service Discovery
 * process. It contains the generated flow for the task, the list of services
 * that had been found to complete the task, and all mapping informations. <br>
 * 
 * @author <a href="mailto:sebastien.bianco@inria.fr">sebastien bianco</a> - <a
 *         href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
 * 
 * Creation Date : Sep 26, 2007 2:34:05 PM
 */
public class SDResult {

	/**
	 * The services gathered during the SD process, that will be used to perform
	 * the UserTask (stored in a map, using as index the service name)
	 */
	private Map<String, Service> services = new HashMap<String, Service>();

	/**
	 * The map nameOfUserTaskCapability->Match that store information concerning
	 * the matching between the UserTask Capability and a Service Capability
	 */
	private Map<String, List<CapabilityMatch>> matches = new HashMap<String, List<CapabilityMatch>>();

	/**
	 * The map nameOfUserTaskCapability->Flow that store the final flow (and the
	 * relative services) that will be used during TaskPerform
	 */
	private Map<String, CapabilityFlow> flows = new HashMap<String, CapabilityFlow>();

	/**
	 * the usertask used to get this SDResult object
	 */
	private UserTask usertask = null;

	/**
	 * Complete data structure of SDResult object from a list of matches
	 * 
	 * @param matches
	 *            the list of matches for Discovery
	 */
	public SDResult(List<CapabilityMatch> matches) {
		this(matches, null);
	}

	/**
	 * Complete data structure of SDResult object from a list of matches
	 * 
	 * @param matches
	 *            the list of matches for Discovery
	 * @param task
	 *            the initial user task from which the SDResult object is the
	 *            result !
	 */
	public SDResult(List<CapabilityMatch> matches, UserTask task) {

		this.usertask = task;

		/*
		 * go through all match to extract information and fill SDResult
		 * datastructure
		 */
		for (CapabilityMatch match : matches) {

			// add the service relative to the match in the list of services
			Service capabilityService = match.getServiceCapability().getService();
			this.getServices().put(capabilityService.getLocalName(),
				capabilityService);

			// add the match in the map "matches"
			List<CapabilityMatch> alreadyFound = this.getMatches().get(
				match.getTaskCapabilityLocalName());
			if (alreadyFound != null) {
				alreadyFound.add(match);
			} else {
				List<CapabilityMatch> tmp = new ArrayList<CapabilityMatch>();
				tmp.add(match);
				this.getMatches().put(match.getTaskCapabilityLocalName(), tmp);
			}
		}
	}

	/**
	 * @return the flows
	 */
	public Map<String, CapabilityFlow> getFlows() {
		return flows;
	}

	/**
	 * @param flows
	 *            the flows to set
	 */
	public void setFlows(Map<String, CapabilityFlow> flows) {
		this.flows = flows;
	}

	/**
	 * @return the matches
	 */
	public Map<String, List<CapabilityMatch>> getMatches() {
		return matches;
	}

	/**
	 * @param matches
	 *            the matches to set
	 */
	public void setMatches(Map<String, List<CapabilityMatch>> matches) {
		this.matches = matches;
	}

	/**
	 * @return the services
	 */
	public Map<String, Service> getServices() {
		return services;
	}

	/**
	 * @param services
	 *            the services to set
	 */
	public void setServices(Map<String, Service> services) {
		this.services = services;
	}

	@Override
	public String toString() {
		String result = "SDResult(" + super.toString() + ") = \n";

		/*
		 * Display key of the map of services
		 */
		result += "\nList of selected services:\n--------------------------\n";
		for (String key : this.getServices().keySet()) {
			result += "\t* " + key + "\n";
		}

		/*
		 * Display map of matches
		 */
		result += "\nMatches:\n--------\n";
		// list of required capability
		for (Entry<String, List<CapabilityMatch>> entry : this.getMatches().entrySet()) {
			result += "\t* " + entry.getKey() + " ("
					+ entry.getValue().get(0).getType() + ")\n";

			// display the required Capability
			if (this.getUsertask() != null) {
				Capability required = this.getUsertask().getCapabilityFromLocalName(
					entry.getKey());
				if(required !=null){
				// display Input
				for (Input requiredInput : new ArrayList<Input>(
						required.getInputs().values())) {
					result += "\t\t\t~ RI: " + requiredInput.getTypeName()
							+ "\n";
				}
				// display Output
				for (Output requiredOutput : new ArrayList<Output>(
						required.getOutputs().values())) {
					result += "\t\t\t~ RO: " + requiredOutput.getTypeName()
							+ "\n";
				}
				}

			}

			// list of match for this required
			for (CapabilityMatch match : entry.getValue()) {
				// Display CapabilityMatch
				result += "\t\t+ "
						+ match.getServiceCapabilityLocalName()
						+ " -> "
						+ match.getServiceCapability().getService().getLocalName()
						+ "\n";

				// Display Input IOMatch
				for (IOMatch iomatch : match.getInputsMapping()) {
					result += "\t\t\t- (I:"
							+ Tools.getSecondPart(iomatch.getTypeName())
							+ ") "
							+ Tools.getSecondPart(iomatch.getTaskLocalName())
							+ " <-> "
							+ Tools.getSecondPart(iomatch.getServiceLocalName())
							+ "\n";
				}

				// Display Output IOMatch
				for (IOMatch iomatch : match.getOutputsMapping()) {
					result += "\t\t\t- (O:"
							+ Tools.getSecondPart(iomatch.getTypeName())
							+ ") "
							+ Tools.getSecondPart(iomatch.getTaskLocalName())
							+ " <-> "
							+ Tools.getSecondPart(iomatch.getServiceLocalName())
							+ "\n";
				}
			}
		}

		return result;
	}

	/**
	 * This equals method is not a depth equals method. it just check that list
	 * of selected services are the same, and that for each functionality of the
	 * UT, the same Match has been selected)
	 * 
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		final SDResult other = (SDResult) obj;

		/*
		 * checke that for each functionality of the UT, the same Match has been
		 * selected
		 */
		if (this.getMatches() == null) {
			if (other.getMatches() != null)
				return false;
		} else if (!this.lightCompare(this.getMatches(), other.getMatches()))
			return false;

		/*
		 * Check that the list of selected services
		 */
		if (this.getServices() == null) {
			if (other.getServices() != null)
				return false;
		} else if (!Tools.compareList(new ArrayList<String>(
				this.getServices().keySet()), new ArrayList<String>(
				other.getServices().keySet())))
			return false;

		return true;
	}

	/**
	 * Compare two map of matches by a easy way (<=> two CapabilityMatch are
	 * considered as equals if the methode CapabilityMatch.lightEquals return
	 * true)
	 * 
	 * @param map1
	 *            the first map
	 * @param map2
	 *            the second map
	 * @return <i>true</i> if both map are considering as equivalent, <i>false</i>
	 *         otherwise
	 */
	private boolean lightCompare(Map<String, List<CapabilityMatch>> map1,
			Map<String, List<CapabilityMatch>> map2) {
		// compare list of keys
		if (Tools.compareList(new ArrayList<String>(map1.keySet()),
			new ArrayList<String>(map2.keySet()))) {

			// compare each list of CapabilityMatch
			for (Entry<String, List<CapabilityMatch>> entry : map1.entrySet()) {
				List<CapabilityMatch> tmpList = map2.get(entry.getKey());
				boolean tmpCompare = this.compareList(tmpList, entry.getValue());
				if (!tmpCompare)
					return false;
			}
			return true;
		}
		return false;
	}

	/**
	 * Compare two list of Capabilities using the lightEquals method of
	 * CapabilityMatch
	 * 
	 * @param list1
	 *            the first list
	 * @param list2
	 *            the second list
	 * @return <i>true</i> if both lists are considering as equivalent,
	 *         <i>false</i> otherwise
	 */
	private boolean compareList(List<CapabilityMatch> list1,
			List<CapabilityMatch> list2) {

		// if both list have the same size...
		if (list1.size() == list2.size()) {

			// ...for each element of list1...
			for (CapabilityMatch matchList1 : list1) {

				// ...find an equivalent in the second list
				boolean found = false;
				for (int i = 0; !found && i < list2.size(); i++) {
					found = matchList1.lightEquals(list2.get(i));
				}

				if (!found)
					return false;
			}
			return true;
		}
		return false;
	}

	/**
	 * Get the list of Selected services
	 * 
	 * @return the list of selected service, <i>empty list</i> if no services
	 *         have been selected
	 * 
	 */
	public List<Service> getListOfService() {
		return new ArrayList<Service>(this.getServices().values());
	}

	/**
	 * This method return a map CapabilityName -> Service File URL. This map is
	 * generated during call of the SD method) So, you have to call it after the
	 * call of SD()
	 * 
	 * @return the map said, or null if the SD has never be called before !
	 */
	public Map<String, String> getMapCapability_ServiceFile() {
		// Creation of the map Capability-> Service File
		Map<String, String> capability_service = new HashMap<String, String>();

		/*
		 * go through map of Matches to collect get the Base URI of services
		 * corresponding to the capability that match
		 */
		for (List<CapabilityMatch> listCapa : new ArrayList<List<CapabilityMatch>>(
				this.getMatches().values())) {
			for (CapabilityMatch match : listCapa) {
				capability_service.put(Tools.getSecondPart(match.getType()),
					match.getServiceCapability().getService().getBase());

			}
		}

		return capability_service;
	}

	/**
	 * Get map of correspondance between serviceURI and corresponding
	 * ServiceMap. This map is generated during call of the SD method) So, you
	 * have to call it after the call of SD()
	 * 
	 * @return the map said, or null if the SD has never be called before !
	 */
	public Map<String, ServiceMap> getServicesMap() {
		// create map of result
		Map<String, ServiceMap> result = new HashMap<String, ServiceMap>();

		/*
		 * go through the selected services list to get the URI of the WSDL/OWL
		 * files. Then read its and create a ServiceMap object from theses
		 */
		for (Service s : this.getListOfService()) {
			//try {
				// get WSDL/OWL files's URI
				//File fowl = new File(new URI(s.getOwlDescriptionFile()));
				//File fwsdl = new File(new URI(s.getWsdlDescriptionFile()));

				// create ServiceMap object and put it in the list of results
				/*result.put(s.getBase(), new ServiceMap(Tools.readFromFile(fowl
						.getAbsolutePath()), Tools.readFromFile(fwsdl
						.getAbsolutePath())));*/
				/*result.put(s.getBase(), new ServiceMap("useless", Tools.readFromFile(fwsdl
						.getAbsolutePath())));*/
				result.put(s.getBase(), new ServiceMap("useless", s.getWsdlContent()));
			/*} catch (URISyntaxException e) {

				DebugPrinter.print(e.getStackTrace(), "The URI "
						+ s.getOwlDescriptionFile() + " or the URI "
						+ s.getWsdlDescriptionFile() + " is not correct !",
					DebugPrinter.WARN);
			}*/
		}
		return result;
	}

	/**
	 * Get the map of correspondance IO uri of Usertask and IO object of
	 * selected services
	 * 
	 * @return the map said, or null if the sd function has never be called
	 *         before ! This map could be empty, if there is no matching.
	 */
	public Map<IO, IO> getIOMapping() {
		// create map of result
		Map<IO, IO> result = new HashMap<IO, IO>();

		/* go through map of Matches to collect IOMatch. Then update result map */
		for (List<CapabilityMatch> listCapa : new ArrayList<List<CapabilityMatch>>(
				this.getMatches().values())) {
			for (CapabilityMatch match : listCapa) {

				// collect ioMatch
				for (IOMatch iomatch : match.getInputsMapping()) {
					result.put(iomatch.getTaskIO(), iomatch.getServiceIO());
				}
				for (IOMatch iomatch : match.getOutputsMapping()) {
					result.put(iomatch.getTaskIO(), iomatch.getServiceIO());
				}
			}
		}

		return result;
	}

	/**
	 * @return the usertask
	 */
	public UserTask getUsertask() {
		return usertask;
	}

}
