/**
 ##############################################################################
 # Copyright (C) 2006-2010 INRIA
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 2 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ##############################################################################
 * 
 */
package fr.inria.icocoa.servicediscovery;

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Queue;

import fr.inria.icocoa.common.DebugPrinter;
import fr.inria.icocoa.semrep.SemanticRepositoryFactory;
import fr.inria.icocoa.semrep.SemanticRepositoryForCompositionEngine;
import fr.inria.icocoa.semrep.excp.MatcherLoadingException;
import fr.inria.icocoa.servicemodel.Capability;
import fr.inria.icocoa.servicemodel.IO;
import fr.inria.icocoa.servicemodel.UserTask;
import fr.inria.icocoa.servicemodel.matching.CapabilityMatch;
import fr.inria.icocoa.servicemodel.matching.Matcher;
import fr.inria.icocoa.servicemodel.matching.SyntacticMatcher;
import fr.inria.icocoa.servicemodel.toolsobjects.ServiceMap;

/**
 * This class contains the algorithm that will be used to discover services
 * concidering a task.<br>
 * The algotithm is quite simple. For each RequiredCapability, the repository is
 * asked for services containing those Capability. If a match is possible and if
 * it is a ProvidedCapability of the selected Service, the match and the result
 * is add in the SDResult object. If no match is found,and if the
 * RequiredCapability is composed, then the subfunctionality will be searched.
 * <br>
 * 
 * <br>
 * You can indicate the Matcher you want to use during the algorithm, by giving
 * the Matcher object as parameter of the constructor. If you don't give any
 * Matcher, the SyntaxicMatcher will be used. <br>
 * This indication could be done by using "matcher.classname". To set it, please
 * run jvm with option <i>-Dmatcher.classname="YourCompleteClassName"</i>. If
 * the classname is false, then default matcher will be used (SyntacticMatcher) !
 * This will be indicate in log traces.
 * 
 * 
 * 
 * @author <a href="mailto:sebastien.bianco@inria.fr">sebastien bianco</a> - <a
 *         href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
 * 
 * Creation Date : Sep 26, 2007 3:13:16 PM
 */
public class ServiceDiscovery {

	/** The matcher that will be used during the algorithm */
	private Matcher matcher = null;

	/** The repository in which we will get informations */
	private SemanticRepositoryForCompositionEngine repository = null;

	/** the last result deliver by the service discovery system */
	private SDResult lastResult = null;

	/**
	 * Check the environment variable "matcher.classname". If this variable
	 * exists, then use the given class name to load the Matcherer, else use a
	 * SyntacticMatcher
	 */
	public ServiceDiscovery() {

		// load Matcher class name configuration file
		String className = System.getProperty("matcher.classname");

		if (className == null) {
			Matcher matcher = new SyntacticMatcher();
			this.matcher = matcher;
			this.repository = SemanticRepositoryFactory.getInstanceCE(matcher);
		} else {

			try {
				// Dynamic load of the Matcher class
				try {
					Class matcherClass = Class.forName(className);
					Matcher matcherInstance = (Matcher) matcherClass.newInstance();

					// update matcher and repository
					this.matcher = matcherInstance;
					this.repository = SemanticRepositoryFactory.getInstanceCE(matcherInstance);

				} catch (ClassNotFoundException e) {
					DebugPrinter.print(e, DebugPrinter.WARN);
					throw new MatcherLoadingException(e);
				} catch (InstantiationException e) {
					DebugPrinter.print(e, DebugPrinter.WARN);
					throw new MatcherLoadingException(e);
				} catch (IllegalAccessException e) {
					DebugPrinter.print(e, DebugPrinter.WARN);
					throw new MatcherLoadingException(e);
				} catch (ClassCastException e) {
					DebugPrinter.print(e, DebugPrinter.WARN);
					throw new MatcherLoadingException(e);
				}
			} catch (MatcherLoadingException e) {
				DebugPrinter.print(e, DebugPrinter.WARN);
				DebugPrinter.print(
					"An error occurs during dynamic load of the Matcher. SyntacticMatcher will be used.",
					DebugPrinter.WARN);

				Matcher matcher = new SyntacticMatcher();
				this.matcher = matcher;
				this.repository = SemanticRepositoryFactory.getInstanceCE(matcher);
			}

		}
	}

	/**
	 * @param matcher
	 *            the matcher to use during algorithm
	 */
	public ServiceDiscovery(Matcher matcher) {
		this.matcher = matcher;
		this.repository = SemanticRepositoryFactory.getInstanceCE(matcher);
	}

	/**
	 * Execute a ServiceDiscovery process to find services that will be able to
	 * perform the UserTask. The algorithm could return a null result if at
	 * least one Capability (required, or subcapability of a required capability
	 * not found) has not been found.
	 * 
	 * @param task
	 *            a UserTask object that describe the set of functionality
	 *            required
	 * @return a SDResult object containing the main part of information that
	 *         will be used during composition and BPEL generation, <i>null</i>
	 *         if the service discovery has no result.
	 */
	public SDResult discoverServices(UserTask task) {

		/*
		 * Create a queue and initialise it with the functionality to find =
		 * RequiredFunctionality of the UserTask
		 */
		Queue<Capability> toFind = new LinkedList<Capability>();
		toFind.addAll(task.getRequiredCapabilities());

		/*
		 * Create the list where the match corresponding to a result will be
		 * stored
		 */
		List<CapabilityMatch> matches = new ArrayList<CapabilityMatch>();

		/*
		 * Search all required Capabilities
		 */
		while (!toFind.isEmpty()) {
			// search the first element of the queue (this one is remove from
			// the queue)
			Capability capaToFind = toFind.poll();
			List<CapabilityMatch> subMatches = this.discoverCapability(capaToFind);

			// if not null (<=> a match has been found) add to the list of
			// results
			if (subMatches != null) {
				matches.addAll(subMatches);
			}

			// a required Capability has not found any match => no match has
			// been found for the UserTask
			else {
				//DebugPrinter.print("a capability ("+capaToFind.getLocalName()+") has not been found and has no subcapabilities...", DebugPrinter.WARN);
				return null;
			}
		}

		// create SDResult object
		SDResult result = new SDResult(matches, task);

		this.lastResult = result;
		return result;
	}

	/**
	 * This method try to find a match for the Capability. If it is an composite
	 * Capability, if the Capability is not found directly, then all
	 * subcapabilities of the current one will be searched.
	 * 
	 * @param capability
	 *            the Capability to match with
	 * @return a list of matches. The list could have more than one elements in
	 *         the case of Capability has not be found as an Atomic or Required
	 *         capabilities, but found as match of subcapabilities ; or in the
	 *         case of many Services offering the same Capabilities.
	 */
	private List<CapabilityMatch> discoverCapability(Capability capability) {

		/*
		 * try to find directly the Capability
		 */
		List<CapabilityMatch> matches = this.getRepository().getServicesContainingCapability(
			capability);

		// DebugPrinter.print("CapabilityTYpe = " +
		// capability.getType(),DebugPrinter.INFO);
		// DebugPrinter.print(matches, "matches", DebugPrinter.INFO);

		/*
		 * If not found, try to find each subcapabilities if not Atomic,
		 * otherwise return null;
		 */
		if (capability.isAtomic()) {
			if (!matches.isEmpty()) {
				return matches;
			} else{
				DebugPrinter.print("an atomic capability ("+capability.getLocalName()+" - "+capability.getSemanticType()+") has not been found...", DebugPrinter.WARN);
				return null;
			}
		} else {

			// Create a queue and initialise it with the functionality to
			// find = RequiredFunctionality of the UserTask
			Queue<Capability> toFind = new LinkedList<Capability>();
			toFind.addAll(capability.getSubCapabilities());

			// Create the list where the match corresponding to a result
			// will be stored
			List<CapabilityMatch> capaMatches = new ArrayList<CapabilityMatch>();

			// add the already found match, in case of one found
			if (!matches.isEmpty())
				capaMatches.addAll(matches);

			/*
			 * Search all required Capabilities
			 */
			while (!toFind.isEmpty()) {
				// search the first element of the queue (this one is remove
				// from the queue)
				List<CapabilityMatch> subMatches = this.discoverCapability(toFind.poll());

				// if not null (<=> a match has been found) add to the list
				// of results
				if (subMatches != null) {
					capaMatches.addAll(subMatches);
				}

				// a required Capability has not found any match => no match
				// has been found for the Capability
				else{
					//DebugPrinter.print("a capability ("+capability.getLocalName()+") has not been found...", DebugPrinter.WARN);
					return null;
				}
			}

			return capaMatches;
		}

	}

	/**
	 * @return the matcher
	 */
	public Matcher getMatcher() {
		return matcher;
	}

	/**
	 * @return the repository
	 */
	public SemanticRepositoryForCompositionEngine getRepository() {
		return repository;
	}

	/**
	 * This method return a map CapabilityName -> Service File URL. This map is
	 * generated during call of the SD method) So, you have to call it after the
	 * call of SD()
	 * 
	 * @return the map said, or <i>null</i> if the SD has never be called
	 *         before !
	 */
	public Map<String, String> getMapCapability_ServiceFile() {
		return (this.lastResult != null) ? this.lastResult.getMapCapability_ServiceFile()
				: null;
	}

	/**
	 * Get map of correspondance between serviceURI and corresponding
	 * ServiceMap. This map is generated during call of the SD method) So, you
	 * have to call it after the call of SD()
	 * 
	 * @return the map said, or <i>null</i> if the SD has never be called
	 *         before !
	 */
	public Map<String, ServiceMap> getServicesMap() {
		return (this.lastResult != null) ? this.lastResult.getServicesMap()
				: null;
	}

	/**
	 * Get the map of correspondance IO uri of Usertask and IO object of
	 * selected services
	 * 
	 * @return the map said, or <i>null</i> if the sd function has never be
	 *         called before ! This map could be empty, if there is no matching.
	 */
	public Map<IO, IO> getIOMapping() {
		return (this.lastResult != null) ? this.lastResult.getIOMapping()
				: null;
	}

	/**
	 * @return the lastResult
	 */
	public SDResult getLastResult() {
		return lastResult;
	}

}
