package fr.inria.icocoa.test.generator;

import java.io.FileNotFoundException;
import java.io.PrintWriter;
import java.util.List;
import java.util.Map;

import javax.wsdl.Binding;
import javax.wsdl.Definition;
import javax.wsdl.Input;
import javax.wsdl.Message;
import javax.wsdl.Operation;
import javax.wsdl.Output;
import javax.wsdl.Part;
import javax.wsdl.Port;
import javax.wsdl.PortType;
import javax.wsdl.Service;
import javax.wsdl.WSDLException;
import javax.wsdl.factory.WSDLFactory;
import javax.wsdl.xml.WSDLReader;
import javax.xml.namespace.QName;

/**
 * <p>
 * This is a program to convert a WSDL service description into an almost
 * complete Amigo-S semantic service description.
 * </p>
 * <p>
 * Usage:
 * <tt>java fr.inria.icocoa.semrep.tools.amigosgen WSDL_URI OUTPUT_FILENAME</tt>
 * </p>
 * <p>
 * The Amigo-S semantic service description will be written to the location
 * given in OUTPUT_FILENAME. The description is almost complete, with just a few
 * simple name have to be added before the semantic service description is ready
 * to use. Below is a list of strings that must be replaced in the file:
 * <ul>
 * <li>@@ WSDL FILE URI @@ - replace these for the URI of the WSDL file, e.g.
 *      http://amigo.gforge.inria.fr/wsdl/MyService.wsdl.</li>
 * <li>@@ SEMANTIC SERVICE NAME @@ - replace these for the identifier you wish
 *      to give the semantic service, e.g. MySemanticService.</li>
 * <li>@@ OWL FILE URI @@ - replace these with the URI for the resulting
 *      semantic description file, e.g.
 *      http://amigo.gforge.inria.fr/owl/MySemanticService.owl.</li>
 * <li>@@ SERVICE TYPE SEMANTIC URI @@ - replace these the the URI pointing to
 *      the appropriate service semantic, e.g.
 *      http://amigo.gforge.inria.fr/owl/ServiceTypes.owl#MyServiceType.</li>
 * <li>@@ TEXT DESCRIPTION @@ - replace this with a textual description of the
 *      semantic service.</li>
 * <li>@@ TEXT NAME @@ - replace this with a textual (human readable) name for
 *      the semantic service, e.g. My Semantic Service.</li>
 * <li>@@ OPERATION-X CAPABILITY SEMANTIC @@ - replace this with the local name
 *      of the semantic for the capability, e.g. AddUser. The capability
 *      corresponds to the WSDL operation called 'OPERATION-X'.</li>
 * <li>@@ OPERATION-X-INPUT-Y INPUT SEMANTIC URI @@ - replace this with the URI
 *      pointing to the semantic for the current input, e.g.
 *      http://amigo.gforge.inria.fr/owl/Devices.owl#OperatingSystem. The input
 *      corresponds to 'INPUT-Y' of 'OPERATION-X'.</li>
 * <li>@@ OPERATION-X-OUTPUT-Z OUTPUT SEMANTIC URI @@ - replace this with the
 *      URI pointing to the semantic for the current output, e.g.
 *      http://amigo.gforge.inria.fr/owl/Devices.owl#AgentPlatform. The output
 *      corresponds to 'OUTPUT-Z' of 'OPERATION-X'.</li>
 *      </ul>
 *      </p>
 * 
 * @author <a href="mailto:Graham.Thomson@inria.fr">Graham Thomson</a> - <a
 *         href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
 */
public class AmigoSGenerator {

	private static String capitalise(String s) {
		StringBuilder sb = new StringBuilder();
		for (int i = 0; i < s.length(); ++i) {
			if (i == 0) {
				sb.append(Character.toUpperCase(s.charAt(i)));
			} else {
				sb.append(s.charAt(i));
			}
		}
		return sb.toString();
	}

	/**
	 * Generate the amigo-S description from WSDL document and other args
	 * 
	 * @param args
	 *            the filespaths
	 *            <p>
	 *            <ul>
	 *            <li><i>args[0] </i>- the path of the WSDL file</li>
	 *            <li><i>args[1] </i>- the path of the generated Amigo-S file</li>
	 *            </ul>
	 *            </p>
	 * @param otherargs
	 *            the semantic parameter to write during generation
	 *            <p>
	 *            <ul>
	 * <li><i>otherargs[0] </i>- @@ WSDL FILE URI @@</li>
	 * <li><i>otherargs[1] </i>- @@ SEMANTIC SERVICE NAME @@</li>
	 * <li><i>otherargs[2] </i>- @@ OWL FILE URI @@</li>
	 * <li><i>otherargs[3] </i>- @@ SERVICE TYPE SEMANTIC URI @@</li>
	 * <li><i>otherargs[4] </i>- @@ TEXT DESCRIPTION @@</li>
	 * <li><i>otherargs[5] </i>- @@ TEXT NAME @@</li>
	 *            </ul>
	 *            </p>
	 * @param serviceID
	 *            the Id of the service
	 * @return the path of the generated Amigo-S file
	 * @throws FileNotFoundException
	 * @throws WSDLException
	 */
	@SuppressWarnings("unchecked")
	public String generateAmigoS(String[] args, String[] otherargs,
			int serviceID) throws FileNotFoundException, WSDLException {
		WSDLFactory factory = WSDLFactory.newInstance();
		WSDLReader reader = factory.newWSDLReader();
		reader.setFeature("javax.wsdl.verbose", true);
		reader.setFeature("javax.wsdl.importDocuments", true);
		Definition wsdlDef = reader.readWSDL(null, args[0]);

		Service service = null;
		PortType portType = null;
		List<Operation> wsdlOperations = null;

		Map<QName, Service> services = wsdlDef.getServices();
		if (services.size() != 1) {
			throw new RuntimeException(
					"The WSDL file must describe a single service, currently it describes "
							+ services.size() + ".");
		}
		for (Map.Entry<QName, Service> servicesEntry : services.entrySet()) {
			service = servicesEntry.getValue();
			Map<QName, Port> ports = service.getPorts();
			if (services.size() != 1) {
				throw new RuntimeException(
						"The WSDL file must describe a single port, currently it describes "
								+ ports.size() + ".");
			}
			for (Map.Entry<QName, Port> portsEntry : ports.entrySet()) {
				Port p = portsEntry.getValue();
				Binding b = p.getBinding();
				portType = b.getPortType();
				wsdlOperations = portType.getOperations();
			}
		}

		String[] semanticargs = new String[] { "@@ WSDL FILE URI @@",
				"@@ SEMANTIC SERVICE NAME @@", "@@ OWL FILE URI @@",
				"@@ SERVICE TYPE SEMANTIC URI @@", "@@ TEXT DESCRIPTION @@",
				"@@ TEXT NAME @@" };

		/**
		 * Make sure that all args will exist => copy input parameter inside the
		 * tab that will be used
		 */
		for (int i = 0; i < otherargs.length; i++) {
			semanticargs[i] = otherargs[i];
		}

		String wsdlFileURI = semanticargs[0];
		String semanticServiceName = semanticargs[1];
		String owlFileURI = semanticargs[2];
		String serviceTypeURI = semanticargs[3];
		String textDescription = semanticargs[4];
		String textName = semanticargs[5];

		String wsdlNamespace = service.getQName().getNamespaceURI();

		String semanticServiceProfileName = semanticServiceName + "Profile";
		String semanticServiceGrounding = semanticServiceName + "Grounding";

		PrintWriter pout = new PrintWriter(args[1]);

		pout.println("<?xml version=\"1.0\"?>");
		pout.println("");
		pout.println("<!DOCTYPE uridef[");
		pout.println("  <!ENTITY xsd \"http://www.w3.org/2001/XMLSchema\">");
		pout.println("  <!ENTITY process \"http://www.daml.org/services/owl-s/1.1/Process.owl\">");
		pout.println("  <!ENTITY objList \"http://www.daml.org/services/owl-s/1.1/generic/ObjectList.owl\">");
		pout.println("  <!ENTITY wsdl \"" + wsdlNamespace + "\">");
		pout.println("]>");
		pout.println("");
		pout.println("<rdf:RDF");
		pout.println("  xmlns:lang=\"http://amigo.gforge.inria.fr/owl/Amigo-S.owl#\"");
		pout.println("  xmlns:capabilities=\"http://amigo.gforge.inria.fr/owl/Capabilities.owl#\"");
		pout.println("  xmlns:process=\"http://www.daml.org/services/owl-s/1.1/Process.owl#\"");
		pout.println("  xmlns:service=\"http://www.daml.org/services/owl-s/1.1/Service.owl#\"");
		pout.println("  xmlns:profile=\"http://www.daml.org/services/owl-s/1.1/Profile.owl#\"");
		pout.println("  xmlns:grounding=\"http://www.daml.org/services/owl-s/1.1/Grounding.owl#\"");
		pout.println("  xmlns:objList=\"http://www.daml.org/services/owl-s/1.1/generic/ObjectList.owl#\"");
		pout.println("  xmlns:xsd=\"http://www.w3.org/2001/XMLSchema#\"");
		pout.println("  xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\"");
		pout.println("  xmlns:rdfs=\"http://www.w3.org/2000/01/rdf-schema#\"");
		pout.println("  xmlns:owl=\"http://www.w3.org/2002/07/owl#\"");
		pout.println("  xmlns:daml=\"http://www.daml.org/2001/03/daml+oil#\"");
		pout.println("  xmlns:dc=\"http://purl.org/dc/elements/1.1/\"");
		pout.println("  xmlns:generated=\"http://www.inria.fr/myGeneratedOntology.owl#\"");
		pout.println("  xmlns=\"" + owlFileURI + "#\"");
		pout.println("  xml:base=\"" + owlFileURI + "\"");
		pout.println("  xmlns:wsdl=\"" + wsdlNamespace + "\">");
		pout.println("");
		pout.println("  <owl:Ontology rdf:about=\"\">");
		pout.println("    <owl:imports rdf:resource=\"http://amigo.gforge.inria.fr/owl/Amigo-S.owl\"/>");
		pout.println("    <owl:imports rdf:resource=\"http://amigo.gforge.inria.fr/owl/Capabilities.owl\"/>");
		pout.println("    <owl:imports rdf:resource=\"http://www.inria.fr/myGeneratedOntology.owl\"/>");
		pout.println("  </owl:Ontology>");
		pout.println("");
		pout.println("  <service:Service rdf:ID=\"" + semanticServiceName
				+ "\">");
		pout.println("");
		pout.println("    <lang:ServiceType rdf:datatype=\"http://www.w3.org/2001/XMLSchema#string\">");
		pout.println("    	" + serviceTypeURI);
		pout.println("    </lang:ServiceType>");
		pout.println("");
		pout.println("    <service:presents>");
		pout.println("      <capabilities:ServiceProfile rdf:ID=\""
				+ semanticServiceProfileName + "\">");
		pout.println("");
		pout.println("        <!-- Provided capabilities -->");

		// Loop through all provided capabilities

		for (Operation operation : wsdlOperations) {
			// get the methodID
			String methodName = operation.getName();
			int methodID = Integer.parseInt(methodName.split("method_")[1].trim());

			/*******************************************************************
			 * Change here to have change rule of service generation
			 ******************************************************************/
			String opSemantic = "GeneratedCapabilitySem_" + serviceID + "_"
					+ methodID;
			// String opSemantic = "GeneratedCapabilitySem_0_"+methodID;

			String capabilityName = portType.getQName().getLocalPart()
					+ capitalise(operation.getName()) + "Capability";
			String conversationName = portType.getQName().getLocalPart()
					+ capitalise(operation.getName()) + "Conversation";

			pout.println("        <lang:hasProvidedCapability>");
			pout.println("          <generated:" + opSemantic + " rdf:ID=\""
					+ capabilityName + "\">");
			pout.println("            <lang:hasConversation rdf:resource=\"#"
					+ conversationName + "\"/>");

			Input in = operation.getInput();
			Message inMsg = in.getMessage();
			Map<QName, Part> inParts = inMsg.getParts();
			for (Map.Entry<QName, Part> entry : inParts.entrySet()) {
				Part inPart = entry.getValue();
				String inputName = portType.getQName().getLocalPart()
						+ capitalise(operation.getName()) + "Input"
						+ capitalise(inPart.getName());
				pout.println("            <lang:hasInput rdf:resource=\"#"
						+ inputName + "\"/>");
			}

			Output out = operation.getOutput();
			Message outMsg = out.getMessage();
			Map<QName, Part> outParts = outMsg.getParts();
			if (!outParts.isEmpty()) {
				for (Map.Entry<QName, Part> entry : outParts.entrySet()) {
					Part outPart = entry.getValue();
					String outputName = portType.getQName().getLocalPart()
							+ capitalise(operation.getName()) + "Output"
							+ capitalise(outPart.getName());
					pout.println("            <lang:hasOutput rdf:resource=\"#"
							+ outputName + "\"/>");
				}
			} else {
				String outputName = portType.getQName().getLocalPart()
						+ capitalise(operation.getName()) + "Output";
				pout.println("            <lang:hasOutput rdf:resource=\"#"
						+ outputName + "\"/>");
			}

			pout.println("          </generated:" + opSemantic + ">");
			pout.println("        </lang:hasProvidedCapability>");
			pout.println("");
		}

		pout.println("        <service:presentedBy rdf:resource=\"#"
				+ semanticServiceName + "\"/>");
		pout.println("");
		pout.println("        <profile:textDescription rdf:datatype=\"http://www.w3.org/2001/XMLSchema#string\">");
		pout.println("          " + textDescription);
		pout.println("        </profile:textDescription>");
		pout.println("");
		pout.println("        <profile:serviceName rdf:datatype=\"http://www.w3.org/2001/XMLSchema#string\">");
		pout.println("          " + textName);
		pout.println("        </profile:serviceName>");
		pout.println("");
		pout.println("      </capabilities:ServiceProfile>");
		pout.println("");
		pout.println("    </service:presents>");
		pout.println("");
		pout.println("    <service:supports rdf:resource=\"#"
				+ semanticServiceGrounding + "\"/>");
		pout.println("");
		pout.println("  </service:Service>");
		pout.println("");

		pout.println("  <!-- Conversations -->");
		pout.println("");

		// Loop through conversation for each operation

		for (Operation operation : wsdlOperations) {
			// get the methodID
			String methodName = operation.getName();
			int methodID = Integer.parseInt(methodName.split("method_")[1].trim());

			String conversationName = portType.getQName().getLocalPart()
					+ capitalise(operation.getName()) + "Conversation";

			pout.println("  <process:AtomicProcess rdf:ID=\""
					+ conversationName + "\">");
			pout.println("");

			Input in = operation.getInput();
			Message inMsg = in.getMessage();
			Map<QName, Part> inParts = inMsg.getParts();

			for (Map.Entry<QName, Part> entry : inParts.entrySet()) {

				Part inPart = entry.getValue();

				// get the inputID
				String inName = inPart.getName();
				int inputID = Integer.parseInt(inName.split("args")[1].trim());

				String inputName = portType.getQName().getLocalPart()
						+ capitalise(operation.getName()) + "Input"
						+ capitalise(inPart.getName());

				/***************************************************************
				 * Change here to have change rule of service generation
				 **************************************************************/
				String inputSemanticURI = "http://www.inria.fr/myGeneratedOntology.owl#GeneratedInputSem_"
						+ serviceID + "_" + methodID + "_" + inputID;
				// String inputSemanticURI =
				// "http://www.inria.fr/myGeneratedOntology.owl#GeneratedInputSem_0_"+methodID+"_"+inputID;

				pout.println("    <process:hasInput>");
				pout.println("      <process:Input rdf:ID=\"" + inputName
						+ "\">");
				pout.println("        <process:parameterType rdf:datatype=\"http://www.w3.org/2001/XMLSchema#anyURI\">");
				pout.println("          " + inputSemanticURI);
				pout.println("        </process:parameterType>");
				pout.println("      </process:Input>");
				pout.println("    </process:hasInput>");
				pout.println("");
			}

			Output out = operation.getOutput();
			Message outMsg = out.getMessage();
			Map<QName, Part> outParts = outMsg.getParts();
			if (!outParts.isEmpty()) {
				for (Map.Entry<QName, Part> entry : outParts.entrySet()) {
					Part outPart = entry.getValue();
					String outputName = portType.getQName().getLocalPart()
							+ capitalise(operation.getName()) + "Output"
							+ capitalise(outPart.getName());

					/***********************************************************
					 * Change here to have change rule of service generation
					 **********************************************************/
					String outputSemanticURI = "http://www.inria.fr/myGeneratedOntology.owl#GeneratedOutputSem_"
							+ serviceID + "_" + methodID;
					// String outputSemanticURI =
					// "http://www.inria.fr/myGeneratedOntology.owl#GeneratedOutputSem_0_"+methodID;

					pout.println("    <process:hasOutput>");
					pout.println("      <process:Output rdf:ID=\"" + outputName
							+ "\">");
					pout.println("        <process:parameterType rdf:datatype=\"http://www.w3.org/2001/XMLSchema#anyURI\">");
					pout.println("          " + outputSemanticURI);
					pout.println("        </process:parameterType>");
					pout.println("      </process:Output>");
					pout.println("    </process:hasOutput>");
					pout.println("");
				}
			} else {
				String outputName = portType.getQName().getLocalPart()
						+ capitalise(operation.getName()) + "Output";
				String outputSemanticURI = "@@ " + outputName
						+ " OUTPUT SEMANTIC URI @@";

				pout.println("    <process:hasOutput>");
				pout.println("      <process:Output rdf:ID=\"" + outputName
						+ "\">");
				pout.println("        <process:parameterType rdf:datatype=\"http://www.w3.org/2001/XMLSchema#anyURI\">");
				pout.println("          " + outputSemanticURI);
				pout.println("        </process:parameterType>");
				pout.println("      </process:Output>");
				pout.println("    </process:hasOutput>");
				pout.println("");

			}

			pout.println("  </process:AtomicProcess>");
			pout.println("");
			methodID++;
		}

		pout.println("  <!-- Grounding -->");
		pout.println("");
		pout.println("  <grounding:WsdlGrounding rdf:ID=\""
				+ semanticServiceGrounding + "\">");

		// Loop through for each op

		for (Operation operation : wsdlOperations) {
			String groundingName = portType.getQName().getLocalPart()
					+ capitalise(operation.getName()) + "Grounding";

			pout.println("    <grounding:hasAtomicProcessGrounding rdf:resource=\"#"
					+ groundingName + "\"/>");
		}

		pout.println("  </grounding:WsdlGrounding>");
		pout.println("");

		// Loop through for each op
		for (Operation operation : wsdlOperations) {
			String groundingName = portType.getQName().getLocalPart()
					+ capitalise(operation.getName()) + "Grounding";
			String conversationName = portType.getQName().getLocalPart()
					+ capitalise(operation.getName()) + "Conversation";

			pout.println("  <grounding:WsdlAtomicProcessGrounding rdf:ID=\""
					+ groundingName + "\">");
			pout.println("");
			pout.println("    <grounding:wsdlDocument rdf:datatype=\"&xsd;#anyURI\">");
			pout.println("      " + wsdlFileURI);
			pout.println("    </grounding:wsdlDocument>");
			pout.println("");
			pout.println("    <grounding:owlsProcess rdf:resource=\"#"
					+ conversationName + "\"/>");
			pout.println("");
			pout.println("    <grounding:wsdlOperation>");
			pout.println("      <grounding:WsdlOperationRef>");
			pout.println("        <grounding:portType rdf:datatype=\"&xsd;#anyURI\">");
			pout.println("          " + wsdlNamespace);
			pout.println("        </grounding:portType>");
			pout.println("        <grounding:operation rdf:datatype=\"&xsd;#anyURI\">");
			pout.println("          " + wsdlNamespace + "#"
					+ operation.getName());
			pout.println("        </grounding:operation>");
			pout.println("      </grounding:WsdlOperationRef>");
			pout.println("    </grounding:wsdlOperation>");
			pout.println("");

			Input in = operation.getInput();
			Message inMsg = in.getMessage();

			pout.println("    <grounding:wsdlInputMessage rdf:datatype=\"&xsd;#anyURI\">");
			pout.println("      " + wsdlNamespace + "#"
					+ inMsg.getQName().getLocalPart());
			pout.println("    </grounding:wsdlInputMessage>");
			pout.println("");

			Map<QName, Part> inParts = inMsg.getParts();
			for (Map.Entry<QName, Part> entry : inParts.entrySet()) {
				Part inPart = entry.getValue();
				String inputName = portType.getQName().getLocalPart()
						+ capitalise(operation.getName()) + "Input"
						+ capitalise(inPart.getName());

				pout.println("    <grounding:wsdlInput>");
				pout.println("      <grounding:WsdlInputMessageMap>");
				pout.println("        <grounding:owlsParameter rdf:resource=\"#"
						+ inputName + "\"/>");
				pout.println("        <grounding:wsdlMessagePart rdf:datatype=\"&xsd;#anyURI\">");
				pout.println("          " + wsdlNamespace + "#"
						+ inPart.getName());
				pout.println("        </grounding:wsdlMessagePart>");
				pout.println("      </grounding:WsdlInputMessageMap>");
				pout.println("    </grounding:wsdlInput>");
				pout.println("");
			}

			Output out = operation.getOutput();
			Message outMsg = out.getMessage();

			pout.println("    <grounding:wsdlOutputMessage rdf:datatype=\"&xsd;#anyURI\">");
			pout.println("      " + wsdlNamespace + "#"
					+ outMsg.getQName().getLocalPart());
			pout.println("    </grounding:wsdlOutputMessage>");
			pout.println("");

			Map<QName, Part> outParts = outMsg.getParts();
			for (Map.Entry<QName, Part> entry : outParts.entrySet()) {
				Part outPart = entry.getValue();
				String outputName = portType.getQName().getLocalPart()
						+ capitalise(operation.getName()) + "Output"
						+ capitalise(outPart.getName());

				pout.println("    <grounding:wsdlOutput>");
				pout.println("      <grounding:WsdlOutputMessageMap>");
				pout.println("        <grounding:owlsParameter rdf:resource=\"#"
						+ outputName + "\"/>");
				pout.println("        <grounding:wsdlMessagePart rdf:datatype=\"&xsd;#anyURI\">");
				pout.println("          " + wsdlNamespace + "#"
						+ outPart.getName());
				pout.println("        </grounding:wsdlMessagePart>");
				pout.println("      </grounding:WsdlOutputMessageMap>");
				pout.println("    </grounding:wsdlOutput>");
				pout.println("");
			}

			pout.println("  </grounding:WsdlAtomicProcessGrounding>");
			pout.println("");
		}

		pout.println("</rdf:RDF>");

		pout.flush();
		pout.close();

		System.out.println("Done.");

		return args[1];
	}

	@SuppressWarnings("unchecked")
	public static void main(String[] args) throws FileNotFoundException,
			WSDLException {
		String[] otherargs = new String[] { "@@ WSDL FILE URI @@",
				"@@ SEMANTIC SERVICE NAME @@", "@@ OWL FILE URI @@",
				"@@ SERVICE TYPE SEMANTIC URI @@", "@@ TEXT DESCRIPTION @@",
				"@@ TEXT NAME @@" };

		new AmigoSGenerator().generateAmigoS(args, otherargs, 1);

	}
}
