/**
 ##############################################################################
 # Copyright (C) 2006-2010 INRIA
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 2 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ##############################################################################
 * 
 */
package fr.inria.icocoa.test.semreptest.junittests.servicediscovery;

import static org.junit.Assert.assertEquals;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;

import fr.inria.icocoa.common.DebugPrinter;
import fr.inria.icocoa.common.Tools;
import fr.inria.icocoa.exception.ParsingException;
import fr.inria.icocoa.semrep.SemanticRepository;
import fr.inria.icocoa.semrep.SemanticRepositoryFactory;
import fr.inria.icocoa.semrep.SemanticRepositoryForCompositionEngine;
import fr.inria.icocoa.servicediscovery.SDResult;
import fr.inria.icocoa.servicediscovery.ServiceDiscovery;
import fr.inria.icocoa.servicemodel.Service;
import fr.inria.icocoa.servicemodel.UserTask;
import fr.inria.icocoa.servicemodel.matching.CapabilityMatch;
import fr.inria.icocoa.servicemodel.matching.MatchingType;
import fr.inria.icocoa.servicemodel.parsers.ServiceParserFactory;
import fr.inria.icocoa.servicemodel.parsers.jenaparser.SDL2ServiceParser;

/**
 * ServiceDiscoveryTest.java
 * 
 * @author <a href="mailto:sebastien.bianco@inria.fr">sebastien bianco</a> - <a
 *         href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
 * 
 * Creation Date : Oct 2, 2007 2:26:32 PM
 */
public class ServiceDiscoveryTest {

	private static String userDir = System.getProperty("user.dir");

	private static String slash = System.getProperty("file.separator");

	private static SemanticRepositoryForCompositionEngine repository = null;

	/**
	 * Registration of required services for tests.
	 * 
	 * @throws java.lang.Exception
	 */
	@BeforeClass
	public static void setUpBeforeClass() throws Exception {

		repository = SemanticRepositoryFactory.getInstanceCE();

		// String userDir = System.getProperty("user.dir");
		// String slash = System.getProperty("file.separator");

		String[] semDescs = { "amigo-s" + slash + "ServiceA.owl",
				"amigo-s" + slash + "ServiceB.owl",
				"amigo-s" + slash + "ServiceC.owl",
				"amigo-s" + slash + "ServiceD.owl",
				"amigo-s" + slash + "SimpleService.owl",
				"amigo-s" + slash + "CompositeService.owl",
				"amigo-s" + slash + "GetPiPart1Service.owl",
				"amigo-s" + slash + "GetPiPart2Service.owl",
				"amigo-s" + slash + "PiLowService.owl",
				"amigo-s" + slash + "MultimediaBrowserWithoutConversation.owl",
				"amigo-s" + slash + "MultimediaBrowser.owl",
				"amigo-s" + slash + "VerySimpleService.owl" };
		String[] wsdls = { "wsdl" + slash + "ServiceA.wsdl",
				"wsdl" + slash + "ServiceB.wsdl",
				"wsdl" + slash + "ServiceC.wsdl",
				"wsdl" + slash + "ServiceD.wsdl",
				"wsdl" + slash + "SimpleService.wsdl",
				"wsdl" + slash + "CompositeService.wsdl",
				"wsdl" + slash + "GetPiPart1Service.wsdl",
				"wsdl" + slash + "GetPiPart2Service.wsdl",
				"wsdl" + slash + "PiLowService.wsdl",
				"wsdl" + slash + "MultimediaBrowser.wsdl",
				"wsdl" + slash + "MultimediaBrowser.wsdl",
				"wsdl" + slash + "VerySimpleService.wsdl" };
		String[] uuids = new String[semDescs.length];

		// SemanticRepository repository =
		// SemanticRepositoryFactory.getInstance();

		repository.clearRepository();

		DebugPrinter.print("Registration of all services", DebugPrinter.INFO);
		for (int i = 0; i < semDescs.length; ++i) {
			DebugPrinter.print("> Registering " + semDescs[i],
				DebugPrinter.INFO);
			try {
				uuids[i] = repository.addService(semDescs[i], wsdls[i]);
			} catch (Exception e) {
				// System.err.println("Couldn't add service: " + e);
				// e.printStackTrace();
				DebugPrinter.print(e.getStackTrace(), e.getMessage(),
					DebugPrinter.WARN);
			}
		}
		DebugPrinter.print("End of registration.", DebugPrinter.INFO);
		// repository.displayAllServices();

	}

	/**
	 * Clean repository after test
	 * 
	 * @throws java.lang.Exception
	 */
	@AfterClass
	public static void tearDownAfterClass() throws Exception {
		SemanticRepository semrep = SemanticRepositoryFactory.getInstance();
		semrep.clearRepository();
	}

	/**
	 * Test method for
	 * {@link fr.inria.icocoa.servicediscovery.ServiceDiscovery#discoverServices(fr.inria.icocoa.servicemodel.UserTask)}.
	 * 
	 * @throws IOException
	 * @throws ParsingException
	 */
	@Test
	public final void testDiscoverServices_GetEasyServices()
			throws ParsingException, IOException {
		String SDLFilenameA = userDir + slash + "amigo-s" + slash + "TaskA.owl";

		/* get service from parsing */
		Long time = System.currentTimeMillis();
		UserTask ut = new UserTask(ServiceParserFactory.getInstance().getParser().parseProcessFromString(
			Tools.readFromFile(SDLFilenameA), false));
		time = System.currentTimeMillis() - time;
		DebugPrinter.print("\nparsing (without grounding) " + SDLFilenameA
				+ " -> time = " + time + " ms", DebugPrinter.INFO);

		/* do service discovery */
		ServiceDiscovery sdv = new ServiceDiscovery();

		time = System.currentTimeMillis();
		SDResult sdr = sdv.discoverServices(ut);
		time = System.currentTimeMillis() - time;
		DebugPrinter.print("\nService Discovery -> time = " + time + " ms",
			DebugPrinter.INFO);

		/* check result */

		// DebugPrinter.print("SDResult = " + sdr, DebugPrinter.INFO);
		// get Services object
		time = System.currentTimeMillis();
		Service sa = repository.getServiceObjectWithLocalName("http://amigo.gforge.inria.fr/owl/ServiceA.owl#ServiceA");
		Service sc = repository.getServiceObjectWithLocalName("http://amigo.gforge.inria.fr/owl/ServiceC.owl#ServiceC");
		Service sb = repository.getServiceObjectWithLocalName("http://amigo.gforge.inria.fr/owl/ServiceB.owl#ServiceB");
		Service sd = repository.getServiceObjectWithLocalName("http://amigo.gforge.inria.fr/owl/ServiceD.owl#ServiceD");
		Service svs = repository.getServiceObjectWithLocalName("http://amigo.gforge.inria.fr/owl/VerySimpleService.owl#VerySimpleService");
		time = System.currentTimeMillis() - time;
		DebugPrinter.print("\nGet Services (*5) in Repository -> time = "
				+ time + " ms", DebugPrinter.INFO);

		// Create List of CapabilityMatch
		List<CapabilityMatch> lcm = new ArrayList<CapabilityMatch>();
		lcm.add(new CapabilityMatch(
				ut.getCapabilityFromTypeName("Op1Semantic").get(0),
				sa.getCapabilityFromTypeName("Op1Semantic").get(0),
				MatchingType.EXACT));
		lcm.add(new CapabilityMatch(
				ut.getCapabilityFromTypeName("Op1Semantic").get(0),
				sb.getCapabilityFromTypeName("Op1Semantic").get(0),
				MatchingType.EXACT));
		lcm.add(new CapabilityMatch(
				ut.getCapabilityFromTypeName("Op1Semantic").get(0),
				sc.getCapabilityFromTypeName("Op1Semantic").get(0),
				MatchingType.EXACT));
		lcm.add(new CapabilityMatch(
				ut.getCapabilityFromTypeName("Op1Semantic").get(0),
				sd.getCapabilityFromTypeName("Op1Semantic").get(0),
				MatchingType.EXACT));
		lcm.add(new CapabilityMatch(
				ut.getCapabilityFromTypeName("Op1Semantic").get(0),
				svs.getCapabilityFromTypeName("Op1Semantic").get(0),
				MatchingType.EXACT));

		lcm.add(new CapabilityMatch(
				ut.getCapabilityFromTypeName("Op2Semantic").get(0),
				sa.getCapabilityFromTypeName("Op2Semantic").get(0),
				MatchingType.EXACT));
		lcm.add(new CapabilityMatch(
				ut.getCapabilityFromTypeName("Op2Semantic").get(0),
				sc.getCapabilityFromTypeName("Op2Semantic").get(0),
				MatchingType.EXACT));
		lcm.add(new CapabilityMatch(
				ut.getCapabilityFromTypeName("Op2Semantic").get(0),
				sd.getCapabilityFromTypeName("Op2Semantic").get(0),
				MatchingType.EXACT));

		lcm.add(new CapabilityMatch(
				ut.getCapabilityFromTypeName("Op3Semantic").get(0),
				sa.getCapabilityFromTypeName("Op3Semantic").get(0),
				MatchingType.EXACT));
		lcm.add(new CapabilityMatch(
				ut.getCapabilityFromTypeName("Op3Semantic").get(0),
				sc.getCapabilityFromTypeName("Op3Semantic").get(0),
				MatchingType.EXACT));
		lcm.add(new CapabilityMatch(
				ut.getCapabilityFromTypeName("Op3Semantic").get(0),
				sd.getCapabilityFromTypeName("Op3Semantic").get(0),
				MatchingType.EXACT));

		lcm.add(new CapabilityMatch(
				ut.getCapabilityFromTypeName("Op4Semantic").get(0),
				sa.getCapabilityFromTypeName("Op4Semantic").get(0),
				MatchingType.EXACT));
		lcm.add(new CapabilityMatch(
				ut.getCapabilityFromTypeName("Op4Semantic").get(0),
				sd.getCapabilityFromTypeName("Op4Semantic").get(0),
				MatchingType.EXACT));

		// create expected result object
		SDResult expected = new SDResult(lcm);

		// check result
		assertEquals("test ServiceDiscovery ok", expected, sdr);

	}

	/**
	 * Test method for
	 * {@link fr.inria.icocoa.servicediscovery.ServiceDiscovery#discoverServices(fr.inria.icocoa.servicemodel.UserTask)}.
	 * 
	 * @throws IOException
	 * @throws ParsingException
	 */
	@Test
	public final void testDiscoverServices_GetMoreComplexServices()
			throws ParsingException, IOException {
		String SDLFilenameA = userDir + slash + "amigo-s" + slash
				+ "MultimediaBrowserTask.owl";

		/* get service from parsing */
		Long time = System.currentTimeMillis();
		UserTask ut = new UserTask(ServiceParserFactory.getInstance().getParser().parseProcessFromString(
			Tools.readFromFile(SDLFilenameA), false));
		time = System.currentTimeMillis() - time;
		DebugPrinter.print("\nparsing (without grounding) " + SDLFilenameA
				+ " -> time = " + time + " ms", DebugPrinter.INFO);

		/* do service discovery */
		ServiceDiscovery sdv = new ServiceDiscovery();
		
		time = System.currentTimeMillis();
		SDResult sdr = sdv.discoverServices(ut);
		time = System.currentTimeMillis() - time;
		DebugPrinter.print("\nService Discovery -> time = " + time + " ms",
			DebugPrinter.INFO);
		int [] stats = sdv.getMatcher().getStat();
		DebugPrinter.print("\nService Discovery Result = " + sdr+"\n stats : SM="+stats[0]+", CM="+stats[1]+", IOM="+stats[2]+" matcher="+sdv.getMatcher().getClass().getName(),
				DebugPrinter.INFO);
		
		

		/* check result */

		// DebugPrinter.print("SDResult = " + sdr, DebugPrinter.INFO);
		// get Services object
		time = System.currentTimeMillis();
		Service smb = repository.getServiceObjectWithLocalName("http://www.dyonisos.org/MultimediaBrowser.owl#MultimediaBrowserService");
		Service smb2 = repository.getServiceObjectWithLocalName("http://www.dyonisos.org/MultimediaBrowserWithoutConversation.owl#MultimediaBrowserWithoutConversationService");
		DebugPrinter.print("\nGet Services  in Repository -> time = "
				+ (time - System.currentTimeMillis()) + " ms",
			DebugPrinter.INFO);

		//DebugPrinter.print("Displey all services",DebugPrinter.INFO);
		//repository.displayAllServices();
		
		// Create List of CapabilityMatch
		List<CapabilityMatch> lcm = new ArrayList<CapabilityMatch>();
		
		String matcherClassname = System.getProperty("matcher.classname");
		
		if(matcherClassname!=null && !matcherClassname.equals("fr.inria.icocoa.servicemodel.matching.GenericEncodedOntologyMatcher")){
		

		lcm.add(new CapabilityMatch(
				ut.getCapabilityFromTypeName("MediaServer").get(0),
				smb.getCapabilityFromTypeName("MediaServer").get(0),
				MatchingType.EXACT));
		lcm.add(new CapabilityMatch(ut.getCapabilityFromTypeName(
			"BrowseContent").get(0), smb.getCapabilityFromTypeName(
			"BrowseContent").get(0), MatchingType.EXACT));
		lcm.add(new CapabilityMatch(
				ut.getCapabilityFromTypeName("SetOn").get(0),
				smb.getCapabilityFromTypeName("SetOn").get(0),
				MatchingType.EXACT));
		lcm.add(new CapabilityMatch(ut.getCapabilityFromTypeName(
			"BrowseContent").get(0), smb2.getCapabilityFromTypeName(
			"BrowseContent").get(0), MatchingType.EXACT));
		lcm.add(new CapabilityMatch(
				ut.getCapabilityFromTypeName("SetOn").get(0),
				smb2.getCapabilityFromTypeName("SetOn").get(0),
				MatchingType.EXACT));
		
		}else{
			lcm.add(new CapabilityMatch(
					ut.getCapabilityFromTypeName("MediaServer").get(0),
					smb.getCapabilityFromTypeName("MediaServer").get(0),
					MatchingType.EXACT));
			lcm.add(new CapabilityMatch(ut.getCapabilityFromTypeName(
				"BrowseContent").get(0), smb.getCapabilityFromTypeName(
				"BrowseContent").get(0), MatchingType.EXACT));
			lcm.add(new CapabilityMatch(
					ut.getCapabilityFromTypeName("SetOn").get(0),
					smb.getCapabilityFromTypeName("SetOn").get(0),
					MatchingType.EXACT));
			lcm.add(new CapabilityMatch(ut.getCapabilityFromTypeName(
				"BrowseContent").get(0), smb2.getCapabilityFromTypeName(
				"BrowseContent").get(0), MatchingType.EXACT));
			lcm.add(new CapabilityMatch(
					ut.getCapabilityFromTypeName("SetOn").get(0),
					smb2.getCapabilityFromTypeName("SetOn").get(0),
					MatchingType.EXACT));
			lcm.add(new CapabilityMatch(
					ut.getCapabilityFromTypeName("MediaServer").get(0),
					smb.getCapabilityFromTypeName("BrowseContent").get(0),
					MatchingType.PLUGIN));
			lcm.add(new CapabilityMatch(
					ut.getCapabilityFromTypeName("MediaServer").get(0),
					smb2.getCapabilityFromTypeName("BrowseContent").get(0),
					MatchingType.PLUGIN));
		
			
			
			
		}

		// create expected result object
		SDResult expected = new SDResult(lcm);
		
		DebugPrinter.print("expected = "+expected,DebugPrinter.INFO);

		// check result
		assertEquals("test ServiceDiscovery ok", expected, sdr);

	}

	/**
	 * Test method for
	 * {@link fr.inria.icocoa.servicediscovery.ServiceDiscovery#discoverServices(fr.inria.icocoa.servicemodel.UserTask)}.
	 * 
	 * @throws IOException
	 * @throws ParsingException
	 */
	@Test
	public final void testDiscoverServices_GetMoreComplexServices2()
			throws ParsingException, IOException {
		String SDLFilenameA = userDir + slash + "amigo-s" + slash
				+ "SimpleTask.owl";

		/* get service from parsing */
		Long time = System.currentTimeMillis();
		UserTask ut = new UserTask(ServiceParserFactory.getInstance().getParser().parseProcessFromString(
			Tools.readFromFile(SDLFilenameA), false));
		time = System.currentTimeMillis() - time;
		DebugPrinter.print("\nparsing (without grounding) " + SDLFilenameA
				+ " -> time = " + time + " ms", DebugPrinter.INFO);

		/* do service discovery */
		ServiceDiscovery sdv = new ServiceDiscovery();

		time = System.currentTimeMillis();
		SDResult sdr = sdv.discoverServices(ut);
		time = System.currentTimeMillis() - time;
		DebugPrinter.print("\nService Discovery -> time = " + time + " ms",
			DebugPrinter.INFO);

		/* check result */

		// DebugPrinter.print("SDResult = " + sdr, DebugPrinter.INFO);
		// get Services object
		time = System.currentTimeMillis();
		Service ss = repository.getServiceObjectWithLocalName("http://mytesturi.inria.fr/SimpleService.owl#SimpleService");
		Service cs = repository.getServiceObjectWithLocalName("http://mytesturi.inria.fr/CompositeService.owl#CompositeService");
		DebugPrinter.print("\nGet Services  in Repository -> time = "
				+ (time - System.currentTimeMillis()) + " ms",
			DebugPrinter.INFO);

		// Create List of CapabilityMatch
		List<CapabilityMatch> lcm = new ArrayList<CapabilityMatch>();

		lcm.add(new CapabilityMatch(
				ut.getCapabilityFromTypeName("GetPiCapa").get(0),
				ss.getCapabilityFromTypeName("GetPiCapa").get(0),
				MatchingType.EXACT));
		lcm.add(new CapabilityMatch(
				ut.getCapabilityFromTypeName("GetPiCapa").get(0),
				cs.getCapabilityFromTypeName("GetPiCapa").get(0),
				MatchingType.EXACT));

		// create expected result object
		SDResult expected = new SDResult(lcm);

		// check result
		assertEquals("test ServiceDiscovery ok", expected, sdr);

	}

	/**
	 * Test method for
	 * {@link fr.inria.icocoa.servicediscovery.ServiceDiscovery#discoverServices(fr.inria.icocoa.servicemodel.UserTask)}.
	 * 
	 * @throws IOException
	 * @throws ParsingException
	 */
	@Test
	public final void testDiscoverServices_GetMoreComplexServices3()
			throws ParsingException, IOException {
		String SDLFilenameA = userDir + slash + "amigo-s" + slash
				+ "CompositeTask.owl";

		/* get service from parsing */
		Long time = System.currentTimeMillis();
		UserTask ut = new UserTask(ServiceParserFactory.getInstance().getParser().parseProcessFromString(
			Tools.readFromFile(SDLFilenameA), false));
		time = System.currentTimeMillis() - time;
		DebugPrinter.print("\nparsing (without grounding) " + SDLFilenameA
				+ " -> time = " + time + " ms", DebugPrinter.INFO);

		/* do service discovery */
		ServiceDiscovery sdv = new ServiceDiscovery();

		time = System.currentTimeMillis();
		SDResult sdr = sdv.discoverServices(ut);
		time = System.currentTimeMillis() - time;
		DebugPrinter.print("\nService Discovery -> time = " + time + " ms",
			DebugPrinter.INFO);

		/* check result */

		// DebugPrinter.print("SDResult = " + sdr, DebugPrinter.INFO);
		// get Services object
		time = System.currentTimeMillis();
		Service cs = repository.getServiceObjectWithLocalName("http://mytesturi.inria.fr/CompositeService.owl#CompositeService");
		Service gpip1s = repository.getServiceObjectWithLocalName("http://mytesturi.inria.fr/GetPiPart1Service.owl#GetPiPart1Service");
		Service gpip2s = repository.getServiceObjectWithLocalName("http://mytesturi.inria.fr/GetPiPart2Service.owl#GetPiPart2Service");
		time = System.currentTimeMillis() - time;
		DebugPrinter.print("\nGet Services  in Repository -> time = " + time
				+ " ms", DebugPrinter.INFO);

		// Create List of CapabilityMatch
		List<CapabilityMatch> lcm = new ArrayList<CapabilityMatch>();

		lcm.add(new CapabilityMatch(
				ut.getCapabilityFromTypeName("GetPiCapa").get(0),
				cs.getCapabilityFromTypeName("GetPiCapa").get(0),
				MatchingType.EXACT));

		lcm.add(new CapabilityMatch(ut.getCapabilityFromTypeName(
			"GetPiWithDecimalCapa").get(0), cs.getCapabilityFromTypeName(
			"GetPiWithDecimalCapa").get(0), MatchingType.EXACT));
		lcm.add(new CapabilityMatch(ut.getCapabilityFromTypeName(
			"GetPiWithDecimalCapa").get(0), gpip2s.getCapabilityFromTypeName(
			"GetPiWithDecimalCapa").get(0), MatchingType.EXACT));

		lcm.add(new CapabilityMatch(ut.getCapabilityFromTypeName(
			"SetPiDecimalCapa").get(0), cs.getCapabilityFromTypeName(
			"SetPiDecimalCapa").get(0), MatchingType.EXACT));
		lcm.add(new CapabilityMatch(ut.getCapabilityFromTypeName(
			"SetPiDecimalCapa").get(0), gpip1s.getCapabilityFromTypeName(
			"SetPiDecimalCapa").get(0), MatchingType.EXACT));

		// create expected result object
		SDResult expected = new SDResult(lcm);

		// check result
		assertEquals("test ServiceDiscovery ok", expected, sdr);

		// DebugPrinter.print(sdv.getServicesMap(),"sdv.getServiceMap",
		// DebugPrinter.INFO);
		// DebugPrinter.print(sdv.getIOMapping(),"sdv.getIOMapping",
		// DebugPrinter.INFO);
		// DebugPrinter.print(sdv.getMapCapability_ServiceFile(),"sdv.getMapCapability_ServiceFile",
		// DebugPrinter.INFO);
	}

}
