package vtt.vp.context.source;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;
import java.util.Vector;

import com.hp.hpl.jena.ontology.OntModel;
import com.hp.hpl.jena.query.Query;
import com.hp.hpl.jena.query.QueryException;
import com.hp.hpl.jena.query.QueryExecution;
import com.hp.hpl.jena.query.QueryExecutionFactory;
import com.hp.hpl.jena.query.QueryFactory;
import com.hp.hpl.jena.query.QuerySolution;
import com.hp.hpl.jena.query.ResultSet;
import com.hp.hpl.jena.query.ResultSetFormatter;
import com.hp.hpl.jena.rdf.model.ModelFactory;
import com.hp.hpl.jena.rdf.model.SimpleSelector;
import com.hp.hpl.jena.rdf.model.Statement;

import vtt.vp.context.ContextEventSubscription;
import vtt.vp.context.VPContextEventListener;

/**
 * Wraps a RDF file as a context source.
 * @author tptoni
 *
 */
public class StaticContextSource implements VPContextSource {

	/**
	 * The file that contains the data for this context source
	 */
	private File rdfFile;
	
	/**
	 * The in-memory model of the RDF data.
	 */
	private OntModel model;
	
	/**
	 * Class constructor that reads the given RDF file to a static context source.
	 * @param f	RDF file
	 * @throws FileNotFoundException
	 */
	public StaticContextSource(File f) throws FileNotFoundException{
		
		rdfFile = f;		
		model = ModelFactory.createOntologyModel();
		model.getDocumentManager().setProcessImports(false);
		
		try{             	
        	model.read(new FileInputStream(rdfFile), null, "N3");        	
        }catch(Exception e){        	        		
        	model.read(new FileInputStream(rdfFile), null, "RDF/XML");         	
        }
		
	}
	
	/**
	 * Documented in VPContextSource interface
	 */
	public boolean doAskQuery(String qStr) throws QueryException{
		
		Boolean answer = null;		
				
		Query query = QueryFactory.create(qStr);		
		if(!query.isAskType())
			throw new QueryException("Query not an ASK query");		
		QueryExecution queryExecution = QueryExecutionFactory.create(query, model);
		answer = queryExecution.execAsk();
		queryExecution.close();		
		
		return answer;		
		
	}

	/**
	 * Documented in VPContextSource interface
	 */
	public QuerySolution[] doSelectQuery(String qStr) throws QueryException{
		
		Vector <QuerySolution> solutionsVector = new Vector<QuerySolution>();		
			
		Query query = QueryFactory.create(qStr);	
		if(!query.isSelectType())
			throw new QueryException("Query not a SELECT query");		
		QueryExecution queryExecution = QueryExecutionFactory.create(query, model);			
		ResultSet results = queryExecution.execSelect();
		while(results.hasNext())
			solutionsVector.add(results.nextSolution());
		
		QuerySolution solutions[] = new QuerySolution[solutionsVector.size()];
		solutions = solutionsVector.toArray(solutions);				
		queryExecution.close();			
	      	 
  	  	return solutions;
  	  	
	}

	/**
	 * Documented in VPContextSource interface
	 */
	public String doSelectQueryXML(String qStr) throws QueryException{
		
		String resultXMLString;
			
		Query query = QueryFactory.create(qStr);	
		if(!query.isSelectType())
			throw new QueryException("Query not a SELECT query");		
		QueryExecution queryExecution = QueryExecutionFactory.create(query, model);			
		ResultSet results = queryExecution.execSelect();						
		resultXMLString = ResultSetFormatter.asXMLString(results);
		queryExecution.close();			
	      	 
  	  	return resultXMLString;
  	  	
	}

	/**
	 * Documented in VPContextSource interface
	 */
	public String getURI() {		
		return "Mist URI";
	}

	/**
	 * Documented in VPContextSource interface
	 */
	public void modify(Statement[] remStmts, Statement[] addStmts) {
		
		if(remStmts != null)
			model.remove(remStmts);
		
		if(addStmts != null)
			model.add(addStmts);
		
		//Fire model change event
		

	}

	/**
	 * Documented in VPContextSource interface
	 */
	public ContextEventSubscription subscribeConditionalContextEvent(
			String qStr, VPContextEventListener cel) {
		// TODO Auto-generated method stub
		return null;
	}

	/**
	 * Documented in VPContextSource interface
	 */
	public ContextEventSubscription subscribeContextEvent(
			VPContextEventListener cel) {
		// TODO Auto-generated method stub
		return null;
	}

	/**
	 * Documented in VPContextSource interface
	 */
	public ContextEventSubscription subscribeContextEvent(SimpleSelector s,
			VPContextEventListener cel) {
		// TODO Auto-generated method stub
		return null;
	}

	/**
	 * Documented in VPContextSource interface
	 */
	public ContextEventSubscription subscribeContextEvent(String qStr,
			VPContextEventListener cel) {
		// TODO Auto-generated method stub
		return null;
	}

	/**
	 * Documented in VPContextSource interface
	 */
	public void unsubscribe(ContextEventSubscription ces) {
		// TODO Auto-generated method stub

	}

	/**
	 * Documented in VPContextSource interface
	 */
	public String getName() {		
		return rdfFile.getName();
	}

	/**
	 * Documented in VPContextSource interface
	 */
	public int getType() {		
		return VPContextSource.STATIC_CS_TYPE;
	}

	/**
	 * Documented in VPContextSource interface
	 */
	public String getValueString() {		
		return "RDF";
	}
	
	public InputStream getRDFDataStream() {	
		
		ByteArrayOutputStream baos = new ByteArrayOutputStream();		
		model.write(baos);
		return new ByteArrayInputStream(baos.toByteArray());
		
	}
	
}
