package vtt.vp.library;

import java.util.Enumeration;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;

import javax.imageio.ImageIO;

import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;

import com.hp.hpl.jena.ontology.Individual;
import com.hp.hpl.jena.rdf.model.ModelFactory;
import com.hp.hpl.jena.ontology.*;
import com.hp.hpl.jena.rdf.model.Statement;

import java.util.Iterator;

/**
 * Class IconLibrary constructs icon sets from the data contained in the 
 * given icon library OWL file and puts the icon sets to a
 * concurrent hashmap with proper keys. IconLibrary offers a singleton
 * object statically to all VantagePoint components from which to get images from.
 * 
 * @author inilkka
 * @author tptoni
 */
public class IconLibrary{
	
	static final long serialVersionUID = 414;
	
	/**
	 * The singleton IconLibrary object
	 */
	private static IconLibrary iconLibrary;
	
	/**
	 * Key is the name of the icon set and the value is a Sting
	 * array containing all the image file URLs of one icon set.
	 */
	private ConcurrentHashMap<String, String[]> iconFileURLs;
		
	//area textures
	public static final String AT_DEFAULT = "DEFAULT";
	public static final String AT_BIRCH_PARQUET = "BIRCH_PARQUET";
	public static final String AT_OAK_PARQUET = "OAK_PARQUET";
	public static final String AT_CERAMIC = "CERAMIC";
	public static final String AT_CEMENT = "CEMENT";
	public static final String AT_GRASS = "GRASS";
	private BufferedImage defaultTexture[];
	private BufferedImage birchParquetTexture[];
	private BufferedImage oakParquetTexture[];
	private BufferedImage ceramicTexture[];
	private BufferedImage cementTexture[];
	private BufferedImage grassTexture[];
	
	/**
	 * Class constructor that must be private to ensure singleton pattern functionality.
	 */
	private IconLibrary(){
		iconFileURLs = new ConcurrentHashMap<String, String[]>();
	}
	
	/**
	 * Gives a reference to the singleton IconLibrary object. This method is built according 
	 * to singleton pattern. It checks whether there already is a IconLibrary instance 
	 * created;  if no, a new instance is created and returned, if yes, 
	 * the already constructed instance is returned. 
	 * 
	 * @return IconLibrary reference to the singleton IconLibrary object. 
	 */
	public static IconLibrary getIconLibraryObject(){				
		if (iconLibrary == null)			
			iconLibrary = new IconLibrary();		
		return iconLibrary;
	}
	
	/**
	 * Loads the icon library ontology from file. Constructs icon sets from the image data 
	 * contained in the icon library, and puts the icon sets with their identifier keys 
	 * to this library hashmap. Takes VantagePoint installation directory path as
	 * parameter and loads IconLibrary.owl from /ontologies/ and area textures
	 * from /graphic/textures/
	 * 
	 * @param vpPath	VantagePoint installation directory path
	 */	
	public void loadLibrary(String vpPath){
			
		//First load IconLibrary.owl
		
		OntModelSpec s = new OntModelSpec(OntModelSpec.OWL_MEM_MICRO_RULE_INF);
        OntDocumentManager mgr = new OntDocumentManager(); 		
		mgr.setMetadataSearchPath("file:" + vpPath + "config"+File.separator+"ont-policy.rdf",true);				
		s.setDocumentManager(mgr);		
		OntModel iconLibraryModel = ModelFactory.createOntologyModel(s);		
	 		 
		try{
			iconLibraryModel.read("file:" + vpPath + "ontologies"+File.separator+"IconLibrary.owl");
		}catch(Exception e){
	      	System.err.println("ERROR loading icon library model: " + e);
	      	return;
	    }	    	    
	    
		//Second load icon path URLs from icon library
	    
	    Individual ind;   				
		Statement nStm;
		Statement eStm;
		Statement sStm;
		Statement wStm;
		Statement widthStm;
		Statement heightStm;			
		String iconName;
		String iconPath = vpPath + "graphic"+File.separator+"icons"+File.separator;
		OntClass iconSet = iconLibraryModel.getOntClass("http://vantagepoint.fi/owl/IconSet.owl#IconSet");
	    Iterator iconSetIterator = iconSet.listInstances();	
	    while(iconSetIterator.hasNext()){
	    	
   			ind =  (Individual)iconSetIterator.next();   				
   			nStm = ind.getProperty(iconLibraryModel.getProperty("http://vantagepoint.fi/owl/IconSet.owl#imageFileNameNorth"));
   			eStm = ind.getProperty(iconLibraryModel.getProperty("http://vantagepoint.fi/owl/IconSet.owl#imageFileNameEast"));
   			sStm = ind.getProperty(iconLibraryModel.getProperty("http://vantagepoint.fi/owl/IconSet.owl#imageFileNameSouth"));
   			wStm = ind.getProperty(iconLibraryModel.getProperty("http://vantagepoint.fi/owl/IconSet.owl#imageFileNameWest"));
   			widthStm = ind.getProperty(iconLibraryModel.getProperty("http://vantagepoint.fi/owl/IconSet.owl#iconGridWidth"));
   			heightStm = ind.getProperty(iconLibraryModel.getProperty("http://vantagepoint.fi/owl/IconSet.owl#iconGridHeight"));
   			
   			if(nStm != null 
   					&& eStm != null 
   					&& sStm != null 
   					&& wStm != null 
   					&& widthStm != null 
   					&& heightStm != null){   					
   				
   				iconName = ind.getLocalName();
   				if(iconName.startsWith("_"))
   					iconName = iconName.substring(1);   				
   				
   				iconFileURLs.put(iconName, new String[]{
   						iconPath + nStm.getString(),
   						iconPath + eStm.getString(),
   						iconPath + sStm.getString(),
   						iconPath + wStm.getString(),
   						widthStm.getString(),
   						heightStm.getString()});
   				
			}else{
				System.err.println("IconSet defective: " + ind.getURI());
			}
	    }	    
	     
	    //Third load area textures
	    
	    String texturePath = vpPath + "graphic"+File.separator+"textures"+File.separator;
	    
	    try{
	    
		    defaultTexture = new BufferedImage[16];
		    defaultTexture[0] = ImageIO.read(new File(texturePath + "default.png"));
		    defaultTexture[1] = ImageIO.read(new File(texturePath + "defaultDEdge.png"));
		    defaultTexture[2] = ImageIO.read(new File(texturePath + "defaultLEdge.png"));
		    defaultTexture[3] = ImageIO.read(new File(texturePath + "defaultUEdge.png"));
		    defaultTexture[4] = ImageIO.read(new File(texturePath + "defaultREdge.png"));
		    defaultTexture[5] = ImageIO.read(new File(texturePath + "defaultDLCorner.png"));
		    defaultTexture[6] = ImageIO.read(new File(texturePath + "defaultULCorner.png"));
		    defaultTexture[7] = ImageIO.read(new File(texturePath + "defaultURCorner.png"));
		    defaultTexture[8] = ImageIO.read(new File(texturePath + "defaultDRCorner.png"));
		    defaultTexture[9] = ImageIO.read(new File(texturePath + "defaultUEdgeOpen.png"));
		    defaultTexture[10] = ImageIO.read(new File(texturePath + "defaultREdgeOpen.png"));
		    defaultTexture[11] = ImageIO.read(new File(texturePath + "defaultDEdgeOpen.png"));
		    defaultTexture[12] = ImageIO.read(new File(texturePath + "defaultLEdgeOpen.png"));
		    defaultTexture[13] = ImageIO.read(new File(texturePath + "defaultDUEdge.png"));
		    defaultTexture[14] = ImageIO.read(new File(texturePath + "defaultLREdge.png"));
		    defaultTexture[15] = ImageIO.read(new File(texturePath + "defaultClosed.png"));
		    
		    birchParquetTexture = new BufferedImage[16];
		    birchParquetTexture[0] = ImageIO.read(new File(texturePath + "birchParquet.png"));
		    birchParquetTexture[1] = ImageIO.read(new File(texturePath + "birchParquetDEdge.png"));
		    birchParquetTexture[2] = ImageIO.read(new File(texturePath + "birchParquetLEdge.png"));
		    birchParquetTexture[3] = ImageIO.read(new File(texturePath + "birchParquetUEdge.png"));
		    birchParquetTexture[4] = ImageIO.read(new File(texturePath + "birchParquetREdge.png"));
		    birchParquetTexture[5] = ImageIO.read(new File(texturePath + "birchParquetDLCorner.png"));
		    birchParquetTexture[6] = ImageIO.read(new File(texturePath + "birchParquetULCorner.png"));
		    birchParquetTexture[7] = ImageIO.read(new File(texturePath + "birchParquetURCorner.png"));
		    birchParquetTexture[8] = ImageIO.read(new File(texturePath + "birchParquetDRCorner.png"));
		    birchParquetTexture[9] = ImageIO.read(new File(texturePath + "birchParquetUEdgeOpen.png"));
		    birchParquetTexture[10] = ImageIO.read(new File(texturePath + "birchParquetREdgeOpen.png"));
		    birchParquetTexture[11] = ImageIO.read(new File(texturePath + "birchParquetDEdgeOpen.png"));
		    birchParquetTexture[12] = ImageIO.read(new File(texturePath + "birchParquetLEdgeOpen.png"));
		    birchParquetTexture[13] = ImageIO.read(new File(texturePath + "birchParquetDUEdge.png"));
		    birchParquetTexture[14] = ImageIO.read(new File(texturePath + "birchParquetLREdge.png"));
		    birchParquetTexture[15] = ImageIO.read(new File(texturePath + "birchParquetClosed.png"));
		    
		    oakParquetTexture = new BufferedImage[16];
		    oakParquetTexture[0] = ImageIO.read(new File(texturePath + "oakParquet.png"));
		    oakParquetTexture[1] = ImageIO.read(new File(texturePath + "oakParquetDEdge.png"));
		    oakParquetTexture[2] = ImageIO.read(new File(texturePath + "oakParquetLEdge.png"));
		    oakParquetTexture[3] = ImageIO.read(new File(texturePath + "oakParquetUEdge.png"));
		    oakParquetTexture[4] = ImageIO.read(new File(texturePath + "oakParquetREdge.png"));
		    oakParquetTexture[5] = ImageIO.read(new File(texturePath + "oakParquetDLCorner.png"));
		    oakParquetTexture[6] = ImageIO.read(new File(texturePath + "oakParquetULCorner.png"));
		    oakParquetTexture[7] = ImageIO.read(new File(texturePath + "oakParquetURCorner.png"));
		    oakParquetTexture[8] = ImageIO.read(new File(texturePath + "oakParquetDRCorner.png"));
		    oakParquetTexture[9] = ImageIO.read(new File(texturePath + "oakParquetUEdgeOpen.png"));
		    oakParquetTexture[10] = ImageIO.read(new File(texturePath + "oakParquetREdgeOpen.png"));
		    oakParquetTexture[11] = ImageIO.read(new File(texturePath + "oakParquetDEdgeOpen.png"));
		    oakParquetTexture[12] = ImageIO.read(new File(texturePath + "oakParquetLEdgeOpen.png"));
		    oakParquetTexture[13] = ImageIO.read(new File(texturePath + "oakParquetDUEdge.png"));
		    oakParquetTexture[14] = ImageIO.read(new File(texturePath + "oakParquetLREdge.png"));
		    oakParquetTexture[15] = ImageIO.read(new File(texturePath + "oakParquetClosed.png"));
		    
		    cementTexture = new BufferedImage[16];
		    cementTexture[0] = ImageIO.read(new File(texturePath + "cement.png"));
		    cementTexture[1] = ImageIO.read(new File(texturePath + "cementDEdge.png"));
		    cementTexture[2] = ImageIO.read(new File(texturePath + "cementLEdge.png"));
		    cementTexture[3] = ImageIO.read(new File(texturePath + "cementUEdge.png"));
		    cementTexture[4] = ImageIO.read(new File(texturePath + "cementREdge.png"));
		    cementTexture[5] = ImageIO.read(new File(texturePath + "cementDLCorner.png"));
		    cementTexture[6] = ImageIO.read(new File(texturePath + "cementULCorner.png"));
		    cementTexture[7] = ImageIO.read(new File(texturePath + "cementURCorner.png"));
		    cementTexture[8] = ImageIO.read(new File(texturePath + "cementDRCorner.png"));
		    cementTexture[9] = ImageIO.read(new File(texturePath + "cementUEdgeOpen.png"));
		    cementTexture[10] = ImageIO.read(new File(texturePath + "cementREdgeOpen.png"));
		    cementTexture[11] = ImageIO.read(new File(texturePath + "cementDEdgeOpen.png"));
		    cementTexture[12] = ImageIO.read(new File(texturePath + "cementLEdgeOpen.png"));
		    cementTexture[13] = ImageIO.read(new File(texturePath + "cementDUEdge.png"));
		    cementTexture[14] = ImageIO.read(new File(texturePath + "cementLREdge.png"));
		    cementTexture[15] = ImageIO.read(new File(texturePath + "cementClosed.png"));
		    
		    ceramicTexture = new BufferedImage[16];
		    ceramicTexture[0] = ImageIO.read(new File(texturePath + "ceramic.png"));
		    ceramicTexture[1] = ImageIO.read(new File(texturePath + "ceramicDEdge.png"));
		    ceramicTexture[2] = ImageIO.read(new File(texturePath + "ceramicLEdge.png"));
		    ceramicTexture[3] = ImageIO.read(new File(texturePath + "ceramicUEdge.png"));
		    ceramicTexture[4] = ImageIO.read(new File(texturePath + "ceramicREdge.png"));
		    ceramicTexture[5] = ImageIO.read(new File(texturePath + "ceramicDLCorner.png"));
		    ceramicTexture[6] = ImageIO.read(new File(texturePath + "ceramicULCorner.png"));
		    ceramicTexture[7] = ImageIO.read(new File(texturePath + "ceramicURCorner.png"));
		    ceramicTexture[8] = ImageIO.read(new File(texturePath + "ceramicDRCorner.png"));
		    ceramicTexture[9] = ImageIO.read(new File(texturePath + "ceramicUEdgeOpen.png"));
		    ceramicTexture[10] = ImageIO.read(new File(texturePath + "ceramicREdgeOpen.png"));
		    ceramicTexture[11] = ImageIO.read(new File(texturePath + "ceramicDEdgeOpen.png"));
		    ceramicTexture[12] = ImageIO.read(new File(texturePath + "ceramicLEdgeOpen.png"));
		    ceramicTexture[13] = ImageIO.read(new File(texturePath + "ceramicDUEdge.png"));
		    ceramicTexture[14] = ImageIO.read(new File(texturePath + "ceramicLREdge.png"));
		    ceramicTexture[15] = ImageIO.read(new File(texturePath + "ceramicClosed.png"));	
		    
		    grassTexture = new BufferedImage[16];
		    grassTexture[0] = ImageIO.read(new File(texturePath + "grass.png"));
		    grassTexture[1] = ImageIO.read(new File(texturePath + "grassDEdge.png"));
		    grassTexture[2] = ImageIO.read(new File(texturePath + "grassLEdge.png"));
		    grassTexture[3] = ImageIO.read(new File(texturePath + "grassUEdge.png"));
		    grassTexture[4] = ImageIO.read(new File(texturePath + "grassREdge.png"));
		    grassTexture[5] = ImageIO.read(new File(texturePath + "grassDLCorner.png"));
		    grassTexture[6] = ImageIO.read(new File(texturePath + "grassULCorner.png"));
		    grassTexture[7] = ImageIO.read(new File(texturePath + "grassURCorner.png"));
		    grassTexture[8] = ImageIO.read(new File(texturePath + "grassDRCorner.png"));
		    grassTexture[9] = ImageIO.read(new File(texturePath + "grassUEdgeOpen.png"));
		    grassTexture[10] = ImageIO.read(new File(texturePath + "grassREdgeOpen.png"));
		    grassTexture[11] = ImageIO.read(new File(texturePath + "grassDEdgeOpen.png"));
		    grassTexture[12] = ImageIO.read(new File(texturePath + "grassLEdgeOpen.png"));
		    grassTexture[13] = ImageIO.read(new File(texturePath + "grassDUEdge.png"));
		    grassTexture[14] = ImageIO.read(new File(texturePath + "grassLREdge.png"));
		    grassTexture[15] = ImageIO.read(new File(texturePath + "grassClosed.png"));
	    
	    }catch(IOException ioe){
	    	System.err.println("Could not load area textures: " + ioe);
	    }
		    
	}

	/**
	 * Retrieves the iconset corresponding to the given key value. If
	 * key does not exist in library return the default icon. 
	 * 
	 * @param iconSetKey	a string containing the identifier key of the wanted icon set.
	 * @return the Iconset correspoding to the given key value.  
	 */	
	public IconSet getIconSet(String iconSetKey){
		
		String values[] = iconFileURLs.get(iconSetKey);
		if(values == null){
			values = iconFileURLs.get("defaultIcon");
			iconSetKey = "defaultIcon";
		}
		
		IconSet set;
		try{
			set = new IconSet(iconSetKey,
					ImageIO.read(new File(values[0])), 
					ImageIO.read(new File(values[1])), 
					ImageIO.read(new File(values[2])), 
					ImageIO.read(new File(values[3])), 
					Integer.parseInt(values[4]), 
					Integer.parseInt(values[5])); 
		}catch(IOException ioe){
			System.err.println("Could not load images for: " + iconSetKey);
			//set = new IconSet(iconSetKey,null,null,null,null,0,0);
			set = null;
		}		
					
		return set;
		
	}
	
	/**
	 * Returns a set of all icon names in the library.
	 * @return
	 */
	public Set<String> getIconNames(){
		return iconFileURLs.keySet();
	}
	
	/**
	 * Gives the specific area texture images in a ImageIcon array.
	 * 
	 * @param textureName 	the name of the texture (IconLibrary.AT_DEFAULT etc.)
	 * @return
	 */
	public BufferedImage[] getAreaTexture(String textureName){		
		if(textureName.equals(AT_DEFAULT)) return defaultTexture;
		if(textureName.equals(AT_BIRCH_PARQUET)) return birchParquetTexture;
		if(textureName.equals(AT_OAK_PARQUET)) return oakParquetTexture;
		if(textureName.equals(AT_CEMENT)) return cementTexture;
		if(textureName.equals(AT_CERAMIC)) return ceramicTexture;
		if(textureName.equals(AT_GRASS)) return grassTexture;
		else return null;		
	}		
	
	/**
	 * Prints the iconsets with ther identifier keys for testing purposes. 
	 * 
	 * @param set		ConcurrenHashMap containing a key and a corresponding icon set. 
	 */	
	public void printIconLibrary(){		
		Enumeration keys = iconFileURLs.keys();
		String key;
		String values[];
		while (keys.hasMoreElements()){
			key = (String)keys.nextElement();
			System.out.println("Key/IconName: " + key);
			values = iconFileURLs.get(key);			
			System.out.println("North: " + values[0]);
			System.out.println("East: " + values[0]);
			System.out.println("South: " + values[0]);
			System.out.println("West: " + values[0]);
			System.out.println("Width: " + values[0]);
			System.out.println("Height: " + values[0]);			
			System.out.println("\n*******************************\n");
		}		
	}	
	
	
}
