package vtt.vp.utils;

import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.geom.AffineTransform;
import java.awt.image.AffineTransformOp;
import java.awt.image.BufferedImage;
import java.util.StringTokenizer;
import java.util.Vector;

import javax.swing.ImageIcon;

import com.hp.hpl.jena.ontology.Individual;
import com.hp.hpl.jena.ontology.OntClass;
import com.hp.hpl.jena.ontology.OntModel;
import com.hp.hpl.jena.rdf.model.Property;
import com.hp.hpl.jena.rdf.model.Resource;
import com.hp.hpl.jena.rdf.model.ResourceFactory;
import com.hp.hpl.jena.rdf.model.Statement;
import com.hp.hpl.jena.rdf.model.StmtIterator;
import com.hp.hpl.jena.util.iterator.ExtendedIterator;

import vtt.vp.view.VPItem;
import vtt.vp.view.VisualComponent;

public class VPUtils {

	//VantagePoint namespace and all properties related to VantagePoint
	public static final String VANTAGEPOINT_NS = "http://vantagepoint.fi/owl/VantagePoint.owl#";
	public static final String ICONSET_NS = "http://vantagepoint.fi/owl/IconSet.owl#";
	public static final String ICONLIBRARY_NS = "http://vantagepoint.fi/owl/IconLibrary.owl#";
	public static final Resource ITEM_CLASS = ResourceFactory.createResource(VANTAGEPOINT_NS + "Item");
	public static final Property GRIDX_PROP = ResourceFactory.createProperty(VANTAGEPOINT_NS + "gridX");
	public static final Property GRIDY_PROP = ResourceFactory.createProperty(VANTAGEPOINT_NS + "gridY");
	public static final Property DIRECTION_PROP = ResourceFactory.createProperty(VANTAGEPOINT_NS + "direction");
	public static final Property ISLOCATEDIN_PROP = ResourceFactory.createProperty(VANTAGEPOINT_NS + "isLocatedIn");
	public static final Property CONTAINS_PROP = ResourceFactory.createProperty(VANTAGEPOINT_NS + "contains");
	public static final Property AREAMATRIX_PROP = ResourceFactory.createProperty(VANTAGEPOINT_NS + "areaMatrix");
	public static final Property TEXTURE_PROP = ResourceFactory.createProperty(VANTAGEPOINT_NS + "texture");
	public static final Property ICONSET_PROP = ResourceFactory.createProperty(VANTAGEPOINT_NS + "iconSet");
	public static final Property ICONGRIDWIDTH_PROP = ResourceFactory.createProperty(ICONSET_NS + "iconGridWidth");
	public static final Property ICONGRIDHEIGHT_PROP = ResourceFactory.createProperty(ICONSET_NS + "iconGridHeight");	
	public static final Property PROVIDESSERVICE_PROP = ResourceFactory.createProperty(VANTAGEPOINT_NS + "providesService");	
	public static final Property SERVICEDESCRIPTIONURI_PROP = ResourceFactory.createProperty(VANTAGEPOINT_NS + "serviceDescriptionURI");	
	public static final Property SERVICEGROUNDINGURI_PROP = ResourceFactory.createProperty(VANTAGEPOINT_NS + "serviceGroundingURI");		
	
	/**
	 * Utility method for acquiring a particular VisualComponent from a vector 
	 * containing VisualComponent. The VisualComponent that matches the given URI
	 * is returned or null if there is no matching URI.
	 * @param components	the Vector of VisualComponents
	 * @param URI			the URI of the VisualComponent searched
	 * @return				
	 */
	public static VisualComponent getVisualComponentFromVector(Vector components, String URI){
		
		VisualComponent vc = null;
		
		for(int i=0; i<components.size(); i++){
			try{
				vc = (VisualComponent)components.elementAt(i);
			}catch(ClassCastException cce){
				System.err.println("Vector does not contain VisualComponents: "+cce);
			}
   			if(vc.getURI().equals(URI)) return vc;
   		}		
   		return null;
		
	}
	
	/**
	 * Parses the given area matrix string to boolean table that defines
	 * the size and shape of the area. 
	 * @param areaMatrixString
	 * @return
	 */
	public static boolean[][] parseAreaString(String areaMatrixString){
		
		StringTokenizer amString = new StringTokenizer(areaMatrixString, ";");
		String row, rowBunch;
		Vector<String> rows = new Vector<String>();
		int i,j, multiplier;
				
		//Parse and check row bunches
		while(amString.hasMoreTokens()){
			
			rowBunch = amString.nextToken();
			multiplier = Integer.parseInt(rowBunch.substring(0,1));			
			row = parseMatrixRow(rowBunch.substring(1));
			
			if(row == null){
				System.err.println("Area matrix string parse error: row malformed");
				return null;
			}		
			
			for(i=0; i<multiplier; i++)
				rows.add(row);			
		}
			
		//Check row lengths
		int rowLength = rows.firstElement().length();
		for(i=1; i<rows.size(); i++){
			if(rowLength != rows.elementAt(i).length()){
				System.err.println("Area matrix string parse error: inconsistent row lengths");
				return null;
			}				
		}
		
		//Create area map boolean table
		boolean areaMap[][] = new boolean[rows.size()][rowLength];
		for(j=0; j<rows.size(); j++){			
			for(i=0; i<rows.elementAt(j).length(); i++){
				if(rows.elementAt(j).charAt(i) == '0')
					areaMap[j][i] = false;
				else if(rows.elementAt(j).charAt(i) == '1')
					areaMap[j][i] = true;
			}
		}			
		
		return areaMap;		
		
	}
	
	/**
	 * Parses an area matrix row and returns the parsed String.
	 * If errors in parsing occur null is returned.
	 * @param row	one row from the area matrix
	 * @return parsed String or null if parse errors occur
	 */
	private static String parseMatrixRow(String row){
		
		String parsedString = "";
		char c;
		int i,j;
		int multiplier = 1;
		boolean multiplierSet = false;
		
		for(i=0; i<row.length(); i++){
			
			c = row.charAt(i);
			
			if(!Character.isDigit(c))return null;
			
			if(c == '0'){
				for(j=0;j<multiplier;j++)
					parsedString = parsedString + "0";
				multiplier = 1;
				multiplierSet = false;
			}else if(c == '1'){
				for(j=0;j<multiplier;j++)
					parsedString = parsedString + "1";
				multiplier = 1;
				multiplierSet = false;
			}else if(multiplierSet){
				return null;
			}else{
				multiplier = Character.digit(c,10);
				multiplierSet = true;
			}
			
		}
		
		return parsedString;
		
	}
	
	/**
	 * Converts the given boolean area matrix to string representation.
	 * 
	 * @param matrix	a boolean matrix that defines an area
	 * @return	a string that represents the given matrix
	 */
	public static String parseAreaMatrix(boolean matrix[][]){
		
		String rawRow;
		String parsedRow;
		String areaString = "";		
		String temp = "";
		int multiplier = 0;
				
		if(matrix.length == 1){			
			rawRow = "";
			for(int i=0; i<matrix[0].length; i++){
				if(matrix[0][i]) rawRow = rawRow + "1";
				else rawRow = rawRow + "0";
			}			
			return "1" + compressMatrixRow(rawRow) + ";";			
		}
		
		for(int j=0; j<matrix.length; j++){
					
			//Parse one row
			rawRow = "";
			for(int i=0; i<matrix[j].length; i++){
				if(matrix[j][i]) rawRow = rawRow + "1";
				else rawRow = rawRow + "0";
			}					
			parsedRow = compressMatrixRow(rawRow);			
			
			//Increase multiplier
			if(parsedRow.equals(temp) || temp.equals(""))
				multiplier++;			
						
			if(j == matrix.length-1){				
				if(parsedRow.equals(temp))					
					areaString = areaString + multiplier + temp + ";";					
				else{
					if(temp.equals(""))
						areaString = areaString + "1" + parsedRow + ";";
					else
						areaString = areaString + multiplier + temp + ";1" + parsedRow + ";";
				}
				break;				
			}
			
			//Write parsed row to area String
			if((!parsedRow.equals(temp) && !temp.equals("")) || multiplier == 9){
				areaString = areaString + multiplier + temp + ";";				
				if(multiplier == 9){
					multiplier = 0;
					temp = "";
				}else{
					multiplier = 1;
					temp = parsedRow;
				}			
			}else{				
				temp = parsedRow;					
			}	
			
		}
		
		return areaString;
		
	}
	
	/**
	 * Transforms the given area matrix row string to compressed form.
	 * 
	 * @param uncompressedRow the row as 1s and 0s
	 * @return the row with multipliers and 1s and 0s
	 */
	private static String compressMatrixRow(String uncompressedRow){
		
		char digit;				
		String compressedRow = "";
		String buffer = "";
		
		if(uncompressedRow.length() < 3)
			return uncompressedRow;
		
		for(int i=0; i<uncompressedRow.length(); i++){
			
			//Read one character from uncompressed row
			digit = uncompressedRow.charAt(i);	
			
			//Test that character is a digit
			if(!Character.isDigit(digit)) return null;	
			
			//Append digit to buffer
			if(buffer.length() == 0 || digit == buffer.charAt(buffer.length()-1))
				buffer = buffer + digit;				
			
			//If last character, write buffer and the remainings
			if(i == uncompressedRow.length()-1){				
				if(digit == buffer.charAt(buffer.length()-1)){
					if(buffer.length() == 1 || buffer.length() == 2) 
						compressedRow = compressedRow + buffer;					
					if(buffer.length() > 2)
						compressedRow = compressedRow + buffer.length() + buffer.charAt(0);					
				}else{
					if(buffer.length() == 1 || buffer.length() == 2) 
						compressedRow = compressedRow + buffer + digit;					
					if(buffer.length() > 2)
						compressedRow = compressedRow + buffer.length() + buffer.charAt(0) + digit;					
				}				
				break;				
			}			
			
			//Write multiplier and character to compressed row
			if(digit != buffer.charAt(buffer.length()-1) || buffer.length() == 9){								
				if(buffer.length() == 1 || buffer.length() == 2) 
					compressedRow = compressedRow + buffer;				
				if(buffer.length() > 2)
					compressedRow = compressedRow + buffer.length() + buffer.charAt(0);				
				if(buffer.length() == 9) buffer = "";
				else buffer = Character.toString(digit);				
			}		
			
		}
		
		return compressedRow;
		
	}
	
	/**
	 * Returns area string that represents a rectangular area.
	 * @param width
	 * @param height
	 * @return
	 */
	public static String getRectangularAreaString(int width, int height){
		
		int fullRowBunches = height / 9;
		int remainingRows = height % 9;
		int fullCharacterBunches = width / 9;
		int remainingCharacters = width % 9;
		
		String row = "";
		for(int i=0; i<fullCharacterBunches; i++)
			row = row + "91";
		
		if(remainingCharacters == 1)
			row = row + "1";
		if(remainingCharacters == 2)
			row = row + "11";
		if(remainingCharacters > 2)
			row = row + remainingCharacters + "1";
		
		String areaString = "";
		for(int i=0; i<fullRowBunches; i++)
			areaString = areaString + "9" + row + ";";
		
		if(remainingRows > 0)
			areaString = areaString + remainingRows + row + ";";
		
		return areaString;
		
	}
	
	/**
	 * Creates a tile code map according an area map. Tile code map has a special tile
	 * code for each area square that describes the style of the tile.
	 * @param areaMap
	 * @return
	 */
	public static int[][] convertToTileCodeMap(boolean areaMap[][]){
		
		int tileMap[][] = new int[areaMap.length][areaMap[0].length];
		
		boolean leftOccupied;
		boolean rightOccupied;
		boolean upOccupied;
		boolean downOccupied;
		
		for(int j=0 ; j<areaMap.length ; j++){
			for(int i=0 ; i<areaMap[0].length ; i++){
				
				leftOccupied = false;
				rightOccupied = false;
				upOccupied = false;
				downOccupied = false;				
				
				if(areaMap[j][i]){						
					
					try{if(areaMap[j][i-1]) leftOccupied = true;}catch(Exception e){}							
					try{if(areaMap[j][i+1]) rightOccupied = true;}catch(Exception e){}						
					try{if(areaMap[j-1][i]) upOccupied = true;}catch(Exception e){}						
					try{if(areaMap[j+1][i]) downOccupied = true;}catch(Exception e){}											
					
					if(leftOccupied && rightOccupied && upOccupied && downOccupied) tileMap[j][i] = 0;							
					else if(leftOccupied && rightOccupied && upOccupied && !downOccupied)tileMap[j][i] = 1;
					else if(!leftOccupied && rightOccupied && upOccupied && downOccupied)tileMap[j][i] = 2;
					else if(leftOccupied && rightOccupied && !upOccupied && downOccupied)tileMap[j][i] = 3;						
					else if(leftOccupied && !rightOccupied && upOccupied && downOccupied)tileMap[j][i] = 4;					
					else if(!leftOccupied && rightOccupied && upOccupied && !downOccupied)tileMap[j][i] = 5;						
					else if(!leftOccupied && rightOccupied && !upOccupied && downOccupied)tileMap[j][i] = 6;						
					else if(leftOccupied && !rightOccupied && !upOccupied && downOccupied)tileMap[j][i] = 7;						
					else if(leftOccupied && !rightOccupied && upOccupied && !downOccupied)tileMap[j][i] = 8;					
					else if(!leftOccupied && !rightOccupied && upOccupied && !downOccupied)tileMap[j][i] = 9;						
					else if(!leftOccupied && rightOccupied && !upOccupied && !downOccupied)tileMap[j][i] = 10;						
					else if(!leftOccupied && !rightOccupied && !upOccupied && downOccupied)tileMap[j][i] = 11;						
					else if(leftOccupied && !rightOccupied && !upOccupied && !downOccupied)tileMap[j][i] = 12;						
					else if(leftOccupied && rightOccupied && !upOccupied && !downOccupied)tileMap[j][i] = 13;						
					else if(!leftOccupied && !rightOccupied && upOccupied && downOccupied)tileMap[j][i] = 14;				
					else tileMap[j][i] = 15;	
					
				}else{
					
					tileMap[j][i] = -1;
					
				}
			}
		}
		
		return tileMap;
		
	}
	
	/**
	 * Finds the biggest integer from the given
	 * integer array. 
	 * @return the biggest int in the given array
	 */	
	 public static int maximum(int [] numbers){		
		 int biggest = Integer.MIN_VALUE;		
		 for(int i=0; i<numbers.length; i++){			
			 if(numbers[i]>biggest)
				 biggest = numbers[i];			
		 }		
		 return(biggest);		
	 }
	 
	 /**
	 * Finds the smallest integer from the given
	 * integer array.
	 * @return the smallest int in the given array
	 */	
	 public static int minimum(int [] numbers){		
		 int smallest = Integer.MAX_VALUE;		
		 for(int i=0; i<numbers.length; i++){			
			 if(numbers[i]<smallest)
				 smallest = numbers[i];			
		 }		
		 return(smallest);		
	 }	 
	 
	 /**
	  * Converts the direction int value to String representation
	  * @param direction
	  * @return
	  */
	 public static String DirectionIntToString(int direction){
		 if(direction == VPItem.ITEM_SOUTH) return "SOUTH";
		 if(direction == VPItem.ITEM_WEST) return "WEST";
		 if(direction == VPItem.ITEM_NORTH) return "NORTH";
		 if(direction == VPItem.ITEM_EAST) return "EAST";
		 return null;
	 }
	 
	 /**
	  * Converts the direction String to int value. If String is illegal
	  * -1 is returned.
	  * @param direction
	  * @return
	  */
	 public static int DirectionStringToInt(String direction){
		 if(direction.equalsIgnoreCase("SOUTH")) return VPItem.ITEM_SOUTH;
		 if(direction.equalsIgnoreCase("WEST")) return VPItem.ITEM_WEST;
		 if(direction.equalsIgnoreCase("NORTH")) return VPItem.ITEM_NORTH;
		 if(direction.equalsIgnoreCase("EAST")) return VPItem.ITEM_EAST;
		 return -1;
	 }
	 
	 /**
     * Converts isometric point in absolute pixel coordinates to grid coordinates. 
     * User must give a point in normal 2D plane and then the location of
     * the origo where the isometric coordinates begin in that plane.	 
     * 
     * @param x 	the x coordinate in absolute pixel coordinates
     * @param y 	the y coordinate in absolute pixel coordinates
     * @param origo	the origo of the isometric coordinates
     * @return a point that is the corresponding grid coordinate
     */	       
	 public static Point isometricPointToGridPoint(int x, int y, Point origo){
	   	 		 
		 boolean aboveLine1; //from left upper corner to right lower corner
		 boolean aboveLine2; //from left lower corner to right upper corner
	   	 		 
		 //Transform given coordinates to origo coordinate space.
		 int x0 = x-origo.x; 
		 int y0 = y-origo.y;
		 
		 //Divide coordinates with isometric tile dimensions and if
		 //coordinate is negative it must be reduced by one because
		 //division in negative numbers rounds the answer closer to
		 //zero which is not the direction we want.
		 if(x0 < 0) x0 = (x0/32)-1;
		 else x0 = x0/32;		 
		 if(y0 < 0) y0 = (y0/16)-1;
		 else y0 = y0/16;
		 
		 //Multiply division results and transform to original coordinate space.
		 //Now we have the location of 32*16 test rectangle in original coordinates
		 //inside which the given point is. The test rectangle is positioned so that
		 //it is divided into 4 sections by isometric grid lines. Next thing to do is to
		 //find out in what section the given point is, in order to know the isometric
		 //grid location.
		 x0 = (x0*32)+origo.x;
		 y0 = (y0*16)+origo.y;
		 		 	   
		 //The given point transformed into test rectangle coordinate space
		 int x1 = x-x0;
		 int y1 = y-y0;
	   	   
		 //Line 1 goes from test rectangle origo to (32,16)
		 if(x1<2*y1)aboveLine1 = false;
		 else aboveLine1 = true;
	   
		 //Line 2 goes from (0,16) to (32,0)
		 if(x1<((-2)*y1)+32)aboveLine2 = true;
		 else aboveLine2 = false;
	  
		 //The distance of the test rectangle from the origo in isometric tile halves.
		 //From these distances we can calculate the isometric grid coordinate of the
		 //test rectangle.
		 int distX = (x0-origo.x)/16;
		 int distY = (y0-(origo.y))/8;	 
		 
		 //This is the isometric grid point of the tile starting from the same point
		 //where the test rectangle was.
		 int gridX = (distX+distY)/2; 
		 int gridY = (distY-distX)/2;
	   
		 //Now it is only a matter of logic to test in which section of the test
		 //rectangle the given point was. The default answer is the same tile where
		 //the test rectangle located.
		 if(aboveLine1 && aboveLine2) return new Point(gridX ,gridY-1);
		 else if(!aboveLine1 && !aboveLine2) return new Point(gridX+1 ,gridY);
		 else if(aboveLine1 && !aboveLine2) return new Point(gridX+1 ,gridY-1);
		 else return new Point(gridX ,gridY);
		 
	   
	 }
	 
	 /**
	  * Translates given grid point to isometric coordinates with the given
	  * origo. The return value is the upper left corner of the isometric tile
	  * related to the given isometric coordinate space origo.
	  * @param x	grid x coordinate
	  * @param y	grid y coordinate
	  * @param origo	the isometric origo from which the point is calculated
	  * @return	location of the isometric tile
	  */
	 public static Point gridPointToIsometricPoint(int x, int y, Point origo){
		 return new Point((x-y)*16+origo.x,(x+y)*8+origo.y);
	 }
	 
	 /**
	  * Calculates the absolute grid location of given VisualComponent in the
	  * given model. All locations in model are relative so a search through
	  * parent areas is necessary to know the absolute location.
	  * @param vc		VisualComponent whose absolute location is returned 
	  * @return a Point instance that has the absolute grid coordinates
	  */
	 public static Point getAbsoluteGridLocation(Individual ind){
		 		 		 
		 Statement parentStm = ind.getProperty(ISLOCATEDIN_PROP);
		 Statement statement;
		 Individual parentInd;
		 int gridX = ind.getProperty(GRIDX_PROP).getInt();		 
		 int gridY = ind.getProperty(GRIDY_PROP).getInt();
		 OntModel model = ind.getOntModel();		 		 
		 		 
		 //This condition should only concern root areas
		 if(parentStm == null)			 
			 return new Point(gridX,gridY);			 
		 		 
		 //Add the coordinates of all parent areas
		 StmtIterator parentIter = ind.listProperties(ISLOCATEDIN_PROP);		 
		 while(parentIter.hasNext()){		 
			 parentStm = parentIter.nextStatement();
			 parentInd = model.getIndividual(parentStm.getObject().toString());			 
			 statement = parentInd.getProperty(GRIDX_PROP);
			 gridX = gridX + statement.getInt();
			 statement = parentInd.getProperty(GRIDY_PROP);
			 gridY = gridY + statement.getInt();			 
		 }	 
		 
		 return new Point(gridX,gridY);
		 
	 }
	 
	 /**
	  * Sums up all the parent areas this individual has. Parent area as a
	  * property is transitive so an individual can have multiple parent
	  * areas. The ground is counted as a parent area.
	  * 
	  * @param ind	Individual whose parent areas are counted
	  * @return	the amount of parent areas the Individual has 
	  */
	 public static int getParentCount(Individual ind){
		
		 int parentCount = 0;
		 StmtIterator parentIter = ind.listProperties(ISLOCATEDIN_PROP);				
		 while(parentIter.hasNext()){
			 parentCount++;
			 parentIter.nextStatement();
		 }
		 return parentCount;
		
	 }
	 
	 /**
	  * Returns the URI of the immediate parent of this individual.
	  * @param ind	individual whose immediate parent is resolved
	  * @return	immediate parent URI and null if no parent found
	  */
	 public static String getImmediateParentURI(Individual ind){
		 		 
		 Vector<String> parentURIs = new Vector<String>();
		 StmtIterator parentIter = ind.listProperties(ISLOCATEDIN_PROP);
		 
		 while(parentIter.hasNext())			 
			 parentURIs.add(parentIter.nextStatement().getObject().toString());
		 
		 if(parentURIs.size() == 0) return null;
		 
		 if(parentURIs.size() == 1) return parentURIs.firstElement();
		 
		 OntModel model = ind.getOntModel();		 
		 int biggestIndex = 0;
		 int biggest = 0;
		 int parentCount;
		 for(int i=0; i<parentURIs.size(); i++){			 
			 parentCount = getParentCount(model.getIndividual(parentURIs.elementAt(i)));
			 if(parentCount > biggest){
				 biggest = parentCount;
				 biggestIndex = i;
			 }				 
		 }
		 
		 return parentURIs.elementAt(biggestIndex);
		 
	 }
	 
	 /**
	  * Returns a vector that contains all the URIs of the immediate child components.
	  * If the given individual is null, it is considered as the visualisation base
	  * and all the root areas are returned.
	  * 
	  * @param ind	the parent whose immediate children URIs are returned
	  * @return
	  */
	 public static Vector<Individual> listImmediateChildren(Individual ind, OntModel m){
		 
		OntModel model = m;
		Vector<Individual> childComponents = new Vector<Individual>();
				
		if(ind == null){
		//Gather all root areas
			
			OntClass areaOntClass = model.getOntClass(VPUtils.VANTAGEPOINT_NS + "Area");		
			ExtendedIterator areasIterator = areaOntClass.listInstances();		
			Object areaArray[] = areasIterator.toSet().toArray();
			Individual areaInd;	
			
			for(int i=0; i<areaArray.length; i++){
				areaInd = (Individual)areaArray[i];
				if(areaInd.getProperty(ISLOCATEDIN_PROP) == null)
					childComponents.add(areaInd);				
			}	
			
		}else{
		//Gather all immediate child visual components
			
			StmtIterator childIter = ind.listProperties(CONTAINS_PROP);
			Individual childInd;
			int pCount = VPUtils.getParentCount(ind);
			
			while(childIter.hasNext()){				
				childInd = model.getIndividual(childIter.nextStatement().getObject().toString());				
				if(pCount == VPUtils.getParentCount(childInd)-1)
					childComponents.add(childInd);
			}
			
		}		
		 
		 return childComponents;
		 
	 }
	 
	 /**
	  * Scales the given ImageIcon to fit specified width and height. If ImageIcon
	  * is smaller than the given dimension constraints scaling will not be done,
	  * and the icon will be centered.
	  * 
	  * @param orgIcon		the original ImageIcon
	  * @param maxWidth		maximum width of thumbnail
	  * @param maxHeight	maximum height of thumbnail
	  * @return the reduced size ImageIcon
	  */
	 public static ImageIcon getIconThumbnail(ImageIcon orgIcon, int maxWidth, int maxHeight){
	 		
		 //Draw ImageIcon as BufferedImage
		 BufferedImage image = new BufferedImage(
				 orgIcon.getIconWidth(),
				 orgIcon.getIconHeight(),
				 BufferedImage.TYPE_4BYTE_ABGR);		 
		 image.getGraphics().drawImage(orgIcon.getImage(), 0, 0, null);			  
			 
		 double scale;	
		 int scaledW = image.getWidth(null);
		 int scaledH = image.getHeight(null);
		 int x = 0;
		 int y = 0;
		 
		 if(image.getWidth(null) > image.getHeight(null))
			 scale = (double)maxWidth/(double)image.getWidth(null);
		 else
			 scale = (double)maxHeight/(double)image.getHeight(null);
				 
		 //Set the scale.	 	
		 //If the image is smaller than 
		 //the desired image size,
		 //don't bother scaling.
		 AffineTransform tx = new AffineTransform();		 
		 if(scale<1.0d){			 
			 tx.scale(scale, scale);
			 scaledW = (int)(image.getWidth(null)*scale);			 
			 scaledH = (int)(image.getHeight(null)*scale);	
		 }
		
		 AffineTransformOp ato = new AffineTransformOp(tx,null);
		 
		 if(scaledW <= maxWidth)
			 x = (maxWidth - scaledW)/2;
		 if(scaledH <= maxHeight)
			 y = (maxHeight - scaledH)/2;		 
		 
		 BufferedImage bim = new BufferedImage(maxWidth, maxHeight, BufferedImage.TYPE_4BYTE_ABGR);
		 Graphics2D g2d = (Graphics2D)(bim.createGraphics());
		 g2d.drawImage(image, ato, x, y);		 
		 return(new ImageIcon(bim));		  
		 
	 }
	 
}
