package vtt.vp.view;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Container;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.MouseInfo;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.io.File;
import java.io.InputStream;
import java.util.Vector;

import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JLayeredPane;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JViewport;
import javax.swing.SwingUtilities;

import vtt.vp.context.source.VPContextSource;
import vtt.vp.gui.VantagePoint;
import vtt.vp.library.IconLibrary;
import vtt.vp.library.IconSet;
import vtt.vp.utils.IsometricGrid;
import vtt.vp.utils.VPUtils;

import com.hp.hpl.jena.ontology.Individual;
import com.hp.hpl.jena.ontology.OntClass;
import com.hp.hpl.jena.ontology.OntDocumentManager;
import com.hp.hpl.jena.ontology.OntModel;
import com.hp.hpl.jena.ontology.OntModelSpec;
import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.rdf.model.ModelFactory;
import com.hp.hpl.jena.rdf.model.Statement;
import com.hp.hpl.jena.rdf.model.StmtIterator;
import com.hp.hpl.jena.util.iterator.ExtendedIterator;

public class IsometricView extends VPView 
implements ActionListener,MouseListener, MouseMotionListener {
	
	//Removes the serializable warning
	private static final long serialVersionUID = 221;
	
	/**
	 * The model to be visualized.
	 */
	private OntModel model;	
		
	/**
	 * The currently selected VisualComponent in VPEditView
	 */
	private Component selected;
	
	/**
	 * The currently highlighted component. 
	 */
	private Component highlighted;
	
	/**
	 * The popup-menu that displays info and operations available on
	 * visual components
	 */
	private JPopupMenu popupMenu;
	private JMenu rotateMenu;
	private JMenuItem sendToBackMenuItem;
	private JMenuItem moveMenuItem;
		
	private boolean moveEnabled = false;
	
	private Point origo;
	private Point grabPoint;
	private Point viewportStartLocation;
	private int gridWidth;
	private int gridHeight;
	private IconLibrary iconLib;
	
	private Model iconLibModel;
	private Model vantagePointModel;
	private VPContextSource contextSource;
	private JLayeredPane layeredPane = new JLayeredPane();	
	
	private Vector<String[]> defectiveInstances;
		
	/**
	 * Class constructor. 
	 * @param m	the model to be visualized
	 */
	public IsometricView(VPContextSource vpcs){
		
		super();
		contextSource = vpcs;
		
		//Initialize OntModel
		OntDocumentManager mgr = new OntDocumentManager("file:"+VantagePoint.vpPath+"config"+File.separator+"ont-policy.rdf");		
        OntModelSpec s = new OntModelSpec(OntModelSpec.OWL_MEM_MICRO_RULE_INF);        			
		s.setDocumentManager(mgr);
		model = ModelFactory.createOntologyModel(s);		
		
		//Load IconLibrary.owl
		iconLibModel = ModelFactory.createDefaultModel();		
		iconLibModel.read("file:" + VantagePoint.vpPath + "ontologies" + File.separator + "IconLibrary.owl");
				
		//Load VantagePoint.owl
		vantagePointModel = ModelFactory.createDefaultModel();		
		vantagePointModel.read("file:" + VantagePoint.vpPath + "ontologies" + File.separator + "VantagePoint.owl");
				
		defectiveInstances = new Vector<String[]>();
		layeredPane.addMouseListener(this);
		layeredPane.addMouseMotionListener(this);		
				
		iconLib = IconLibrary.getIconLibraryObject();		
		origo = new Point(0,0);
				
		initComponents();
		refresh();		
			
		
	}

	/**
	 * Scans the model for visualizable components and refreshes the
	 * view. First create all areas and items to vectors. Then locate
	 * components in recursive fashion starting from the root areas.
	 *
	 */
	public void refresh(){			
				
		Statement statement;
		Individual individual;
		int gridX;
		int gridY;	
		String areaMatrixString;
		String textureString;		
		Vector<IsometricArea> areas = new Vector<IsometricArea>();
		Vector<IsometricItem> items = new Vector<IsometricItem>();
		
		//Empty model from statements
		model.removeAll();
		
		//Read the context source
		InputStream rdfStream = contextSource.getRDFDataStream();
		model.read(rdfStream, "");
		
		//Add imports
		model.add(iconLibModel);
		model.add(vantagePointModel);
		
		//First clear all components
		layeredPane.removeAll();
		this.defectiveInstances = new Vector<String[]>();
						
		//set the size and marginal on top of origo
		gridWidth = 50;
		gridHeight = 50;
   		int upperMarginal = 160;
   		layeredPane.setPreferredSize(new Dimension((gridWidth+gridHeight)*16, (gridWidth+gridHeight)*8+upperMarginal));
   		layeredPane.setSize((gridWidth+gridHeight)*16, (gridWidth+gridHeight)*8+upperMarginal);
   		
   		//Set isometric origo location
		origo.setLocation(gridHeight*16,upperMarginal);
		
		//Get all areas in model
		OntClass areaOntClass = model.getOntClass(VPUtils.VANTAGEPOINT_NS + "Area");		
		ExtendedIterator areasIterator = areaOntClass.listInstances();		
		Object [] areaArray = areasIterator.toSet().toArray();		
		
		//Create IsometricAreas
		//
		//-create and set properties
		//-add to vectors		
		//		
		String parentURI;
		long start = System.currentTimeMillis();
		for(int i=0; i<areaArray.length; i++){
			
			//Create IsometricArea instance, area matrix (obligatory)
			individual = (Individual)areaArray[i];
			statement = individual.getProperty(VPUtils.AREAMATRIX_PROP);
			if(statement == null){
				defectiveInstances.add(new String[]{
						individual.getURI(),
						"Area matrix property not found."});
				continue;				
			}
			areaMatrixString = statement.getString();
			statement = individual.getProperty(VPUtils.TEXTURE_PROP);
			if(statement == null)
				textureString = IconLibrary.AT_DEFAULT;					
			else
				textureString = statement.getString();
			IsometricArea isoArea = new IsometricArea(
					individual.getURI(), 
					areaMatrixString, 
					textureString);
			
			//Set location (obligatory)
			statement = individual.getProperty(VPUtils.GRIDX_PROP);
			if(statement == null){
				defectiveInstances.add(new String[]{
						individual.getURI(),
						"Grid x property not found."});
				continue;				
			}
			gridX = statement.getInt();
			statement = individual.getProperty(VPUtils.GRIDY_PROP);
			if(statement == null){
				defectiveInstances.add(new String[]{
						individual.getURI(),
						"Grid y property not found."});
				continue;				
			}
			gridY = statement.getInt();
			isoArea.setGridX(gridX);
			isoArea.setGridY(gridY);
			
			//Set child components (optional)
			StmtIterator childIter = individual.listProperties(VPUtils.CONTAINS_PROP);
			String childURI;
			Individual childIndividual;
			while(childIter.hasNext()){
				
				childURI = childIter.nextStatement().getObject().toString();
				childIndividual = model.getIndividual(childURI);
				
				if(childIndividual.hasRDFType(areaOntClass))
					isoArea.addChildAreaURI(childURI);
				else
					isoArea.addChildItemURI(childURI);
			}
			
			//Set parent area
			parentURI = VPUtils.getImmediateParentURI(individual);			
			isoArea.setParentAreaURI(parentURI);
						
			areas.add(isoArea);
						
		}
		
		//Get all items in model
		OntClass itemOntClass = model.getOntClass(VPUtils.VANTAGEPOINT_NS + "Item");		
		ExtendedIterator itemsIterator = itemOntClass.listInstances();		
		Object [] itemArray = itemsIterator.toSet().toArray();		
		
		//Create IsometricItems
		//
		//-create and set properties
		//-add to vector		
		//
		String iconSetURI;
		String iconSetName;		
		IconSet iconSet;		
		for(int i=0; i<itemArray.length; i++){
			
			//Create IsometricItem instance
			individual = (Individual)itemArray[i];
			statement = individual.getProperty(VPUtils.ICONSET_PROP);
			if(statement == null){
				defectiveInstances.add(new String[]{
						individual.getURI(),
						"Icon set property not found."});
				continue;				
			}
			iconSetURI = statement.getObject().toString();			
			iconSetName = iconSetURI.substring(iconSetURI.indexOf("#")+1,iconSetURI.length());
			if(iconSetName.startsWith("_")) iconSetName = iconSetName.substring(1, iconSetName.length());
			iconSet = iconLib.getIconSet(iconSetName);			
			if(iconSet == null){
				defectiveInstances.add(new String[]{
						individual.getURI(),
						"Icon set does not exist."});
				continue;
			}
			IsometricItem isoItem = new IsometricItem(individual.getURI(),iconSet);
						
			//Set location (obligatory)
			statement = individual.getProperty(VPUtils.GRIDX_PROP);
			if(statement == null){
				defectiveInstances.add(new String[]{
						individual.getURI(),
						"Grid x property not found."});
				continue;
			}
			gridX = statement.getInt();
			statement = individual.getProperty(VPUtils.GRIDY_PROP);
			if(statement == null){
				defectiveInstances.add(new String[]{
						individual.getURI(),
						"Grid y property not found."});
				continue;
			}
			gridY = statement.getInt();
			isoItem.setGridX(gridX);
			isoItem.setGridY(gridY);
			
			//Set direction (optional)
			statement = individual.getProperty(VPUtils.DIRECTION_PROP);
			if(statement == null)
				isoItem.setDirection(VPItem.ITEM_SOUTH);
			else
				isoItem.setDirection(VPUtils.DirectionStringToInt(statement.getString()));
						
			//Set parent area (obligatory)
			parentURI = VPUtils.getImmediateParentURI(individual);
			if(parentURI == null){
				defectiveInstances.add(new String[]{
						individual.getURI(), 
						"Parent area property not found."});
				continue;
			}
			isoItem.setParentAreaURI(parentURI);
			
			//Add to vector			
			items.add(isoItem);
						
		}		
		System.out.println("Item and Area creation time: " + (System.currentTimeMillis() - start));		
		
		JLabel origoPoint = new JLabel();
		//origoPoint.setIcon(new ImageIcon(VPLauncher.vpPath+"graphic\\images\\origo.png"));
		origoPoint.setBounds(origo.x,origo.y,1,1);
		layeredPane.add(origoPoint,new Integer(0));
		
		//Attach and locate VisualComponents		
		attachAndLocateComponents(areas, items);
		
		//Check orders 		
    	checkItemOrder();
    	checkAreaOrder();
    			
		//Add grid
		IsometricGrid isoGrid = new IsometricGrid(gridWidth,gridHeight);
		isoGrid.setLocation(0, origo.y);
		layeredPane.add(isoGrid, new Integer(0));
		
		layeredPane.repaint();
		
	}	

	public VPContextSource getContextSource(){
		return contextSource;
	}
	
	/**
	 * Empties the view from components and clears the defective instances vector.
	 *
	 */
	public void clear(){
		layeredPane.removeAll();
		defectiveInstances = new Vector<String[]>();
	}
	
	/**
	 * Get the list of components that were disqualified from the visualization.
	 * The vector contains String arrays in which the first element is the URI
	 * of the defective component and second element is the reason in few words.
	 * 
	 * @return	defective components vector
	 */
	public Vector<String[]> getDefectiveInstances(){
		return defectiveInstances;
	}
	
	/**
	 * Sets the highlighted selection off from layeredPane view.
	 */
	public void selectionOff(){
		if(selected != null){
			((VisualComponent)selected).setSelected(false);
		}
	}
	
	/**
	 * Constructs a swing component tree recursively. 
	 * @param rootArea
	 * @param areas
	 */
	/*private void buildComponentTree(IsometricArea area, Vector<IsometricArea> areas, Vector<IsometricItem> items){
		
		//Attach area to IsometricView		
		if(area.getParentAreaURI() == null){
			area.setLocation(
   	       			origo.x+((area.getGridX()-area.getGridY())*16)-area.getGridHeight()*16,
   	   				origo.y+(area.getGridX()+area.getGridY())*8);
			layeredPane.add(area);	
			layeredPane.setLayer(area, 1, 0);
		}else{
			IsometricArea parentArea = (IsometricArea)VPUtils.getVisualComponentFromVector(areas, area.getParentAreaURI());
			Point parentOrigo = new Point(parentArea.getLocation().x+parentArea.getGridHeight()*16, parentArea.getLocation().y);
			area.setLocation(
   					parentOrigo.x+((area.getGridX()-area.getGridY())*16)-area.getGridHeight()*16,
   					parentOrigo.y+(area.getGridX()+area.getGridY())*8);
			layeredPane.add(area);	
			layeredPane.setLayer(area, 1, 0);			
		}
		
		//Attach child items
		IsometricItem childItem;
		Vector<String> childItemURIs = area.getChildItemURIs();
		for(int i=0; i<childItemURIs.size(); i++){			
			childItem = (IsometricItem)VPUtils.getVisualComponentFromVector(items, childItemURIs.elementAt(i));
			Point parentOrigo = new Point(area.getLocation().x+area.getGridHeight()*16, area.getLocation().y);
			childItem.setLocation(
   					parentOrigo.x+((childItem.getGridX()-childItem.getGridY())*16)-childItem.getXOffset(),
   					parentOrigo.y+((childItem.getGridX()+childItem.getGridY())*8)-childItem.getYOffset());
			layeredPane.add(childItem);	
			layeredPane.setLayer(childItem, 2);
		}
		
		//Build IMMEDIATE child area component trees
		IsometricArea childArea;
		Vector<String> childAreaURIs = area.getChildAreaURIs();
		int parentAreaCount = VPUtils.getParentAreaCount(model.getIndividual(area.getURI()));
		for(int i=0; i<childAreaURIs.size(); i++){			
			childArea = (IsometricArea)VPUtils.getVisualComponentFromVector(areas, childAreaURIs.elementAt(i));
			if(parentAreaCount == VPUtils.getParentAreaCount(model.getIndividual(childArea.getURI()))-1)
				buildComponentTree(childArea,areas,items);
		}		
		
	}*/
	
	private void attachAndLocateComponents(Vector<IsometricArea> areas, Vector<IsometricItem> items){
		
		for(int i=0; i<areas.size(); i++){
			
			IsometricArea area = areas.elementAt(i);
			Individual ind = model.getIndividual(area.getURI());
			Point absLocation = VPUtils.getAbsoluteGridLocation(ind);
			area.setLocation(
   	       			origo.x+((absLocation.x-absLocation.y)*16)-area.getGridHeight()*16,
   	   				origo.y+(absLocation.x+absLocation.y)*8);
			layeredPane.add(area);	
			layeredPane.setLayer(area, 1, 0);
			
		}
		
		for(int i=0; i<items.size(); i++){
			
			IsometricItem item = items.elementAt(i);
			Individual ind = model.getIndividual(item.getURI());
			Point absLocation = VPUtils.getAbsoluteGridLocation(ind);
			item.setLocation(
   					origo.x+((absLocation.x-absLocation.y)*16)-item.getXOffset(),
   					origo.y+((absLocation.x+absLocation.y)*8)-item.getYOffset());
			layeredPane.add(item);	
			layeredPane.setLayer(item, 2);
			
		}			
		
	}
	
	/**
	 * Method performs a recursive operation through all items and draws them in 
	 * correct order. When items overlap the isometric illusion fails. See
	 * drawItem() method for details.
	 *
	 */
	private void checkItemOrder() {
		
		//Gather initial array
		Component items[] = layeredPane.getComponentsInLayer(2);		
		Object itemArray [][] = new Object[items.length][];		
		Object entry[];
		IsometricItem item;
		Point headPoint, tailPoint;
		int width, height;
		
		for(int i=0; i<items.length; i++){			
			item = (IsometricItem)items[i];
			entry = new Object[4];			
			entry[0] = item;						
			tailPoint = VPUtils.getAbsoluteGridLocation(model.getIndividual(item.getURI()));
			entry[2] = tailPoint;
			if(item.getDirection() == VPItem.ITEM_EAST || item.getDirection() == VPItem.ITEM_WEST){
				width = item.getGridHeight();
				height = item.getGridWidth();
			}else{
				width = item.getGridWidth();
				height = item.getGridHeight();
			}
			headPoint = new Point(tailPoint.x+width,tailPoint.y+height);
			entry[1] = headPoint;
			entry[3] = false;
			itemArray[i] = entry;
		}
		
		/*for(int i=0; i<itemArray.length; i++){
			System.out.print(itemArray[i][0] + " ; ");
			System.out.print(itemArray[i][1] + " ; ");
			System.out.print(itemArray[i][2] + " ; ");
			System.out.print(itemArray[i][3] + " ; ");
			System.out.println();
		}*/
		
		for(int i=0; i<itemArray.length; i++){
			if(!(Boolean)itemArray[i][3])				
				drawItem(i,itemArray);
		}
		
	}
	
	/**
	 * Recursive method that places items in the top position in a layered pane layer
	 * so that they appear in right order in isometric view. ItemArray is a specific
	 * kind of twodimensional Object array. One row has the item, head point, tail point
	 * and boolean variable to mark item as drawn.
	 * 
	 * @param itemIndex		index of the currently positioned item in itemArray
	 * @param itemArray		itemArray is a collection of items
	 */
	private void drawItem(int itemIndex, Object itemArray[][]){
		
		//System.out.println("Starting to Draw: " + itemArray[itemIndex][0]);		
		itemArray[itemIndex][3] = true;
		Object itemEntry[] = itemArray[itemIndex];		
		
		Point itemHeadPoint = (Point)itemEntry[1];
		Point testTailPoint, testHeadPoint;
		
		//Draw all items that are not yet drawn and behind the given item.
		for(int i=0; i<itemArray.length; i++){			
			if(!(Boolean)itemArray[i][3]){
				
				//System.out.println("\tComparing to: " + itemArray[i][0]);
				
				testTailPoint = (Point)itemArray[i][2];
				testHeadPoint = (Point)itemArray[i][1];
				
				//System.out.println("\t\tItem head x: " + itemHeadPoint.x + " ; test tail x: " + testTailPoint.x);
				//System.out.println("\t\tItem head y: " + itemHeadPoint.y + " ; test tail y: " + testTailPoint.y);
				//System.out.println("\t\tItem head y: " + itemHeadPoint.y + " ; test head y: " + testHeadPoint.y);
				
				if(itemHeadPoint.x > testTailPoint.x
						&& itemHeadPoint.y > testTailPoint.y
						&& (itemHeadPoint.x >= testHeadPoint.x || itemHeadPoint.y >= testHeadPoint.y))
					drawItem(i, itemArray);
				
			}			
		}
		
		//Draw given item
		layeredPane.setPosition((IsometricItem)itemArray[itemIndex][0], 0);
		//System.out.println("Drawing: " + itemArray[itemIndex][0]);
		
	}
	/**
	 * Draws all areas (components in layer 1) so that the ones with lowest
	 * parent count are drawn at the bottom and the ones with highest parent
	 * count are drawn at the top of layer 1. First parent counts must be 
	 * calculated and then they need to be sorted.
	 *
	 */
	private void checkAreaOrder(){
		
		Component comps[] = layeredPane.getComponentsInLayer(1);
		Object entries[][] = new Object[comps.length][];
		Vector<Object[]> areasInOrder = new Vector<Object[]>();
		VisualComponent vc;
		int parentCount;
		int vectorSize;
		
		//Gather all components and their parent counts in entries
		for(int i=0; i<comps.length; i++){
			vc = (VisualComponent)comps[i];
			parentCount = VPUtils.getParentCount(model.getIndividual(vc.getURI()));
			entries[i] = new Object[]{comps[i], parentCount};
		}
				
		//Go through all entries and sort them to vector
		for(int i=0; i<entries.length; i++){
			
			if(areasInOrder.size() == 0){
				areasInOrder.add(entries[i]);
				continue;
			}
			
			//Place entries to the right place in vector
			vectorSize = areasInOrder.size();
			for(int j=0; j<vectorSize; j++){				
				parentCount = (Integer)entries[i][1];								
				if(parentCount < (Integer)areasInOrder.elementAt(j)[1])
					areasInOrder.add(j, entries[i]);
				else if(j == vectorSize-1)
					areasInOrder.add(vectorSize, entries[i]);				
			}
			
		}
		
		//Draw areas in order
		for(int i=0; i<areasInOrder.size(); i++)			
			layeredPane.setPosition((Component)areasInOrder.elementAt(i)[0], 0);
		
		
	}
	
	private void initComponents(){
		
		JScrollPane scrollPane = new JScrollPane(layeredPane);		
		this.setLayout(new BorderLayout());
		this.add(scrollPane, BorderLayout.CENTER);
		
		popupMenu = new JPopupMenu();		 
	 	
		//Instance Information
		JMenuItem informationMenuItem = new JMenuItem("Instance Information...");
		informationMenuItem.setActionCommand("INSTANCE_INFORMATION");
		informationMenuItem.addActionListener(this);
	    popupMenu.add(informationMenuItem);
	    
	    //Move
	    moveMenuItem = new JMenuItem("Move");
	    moveMenuItem.setActionCommand("MOVE");
	    moveMenuItem.addActionListener(this);
	    popupMenu.add(moveMenuItem);
			    
	    //Delete
	    JMenuItem deleteMenuItem = new JMenuItem("Delete");
	    deleteMenuItem.setActionCommand("DELETE");
	    deleteMenuItem.addActionListener(this);
	    popupMenu.add(deleteMenuItem);
	    
	    //Rotate
	    rotateMenu = new JMenu("Rotate");
	    popupMenu.add(rotateMenu);    
	    JMenuItem rotateNorth = new JMenuItem("North", new ImageIcon("graphic/arrow_north.png"));
	    rotateNorth.setActionCommand("ROTATE_NORTH");
	    rotateNorth.addActionListener(this);
	    rotateMenu.add(rotateNorth);   
	    JMenuItem rotateEast = new JMenuItem("East", new ImageIcon("graphic/arrow_east.png"));
	    rotateEast.setActionCommand("ROTATE_EAST");
	    rotateEast.addActionListener(this);
	    rotateMenu.add( rotateEast);	    
	    JMenuItem rotateSouth = new JMenuItem("South", new ImageIcon("graphic/arrow_south.png"));
		rotateSouth.setActionCommand("ROTATE_SOUTH");
		rotateSouth.addActionListener(this);
		rotateMenu.add(rotateSouth);	
		JMenuItem rotateWest = new JMenuItem("West", new ImageIcon("graphic/arrow_west.png"));
		rotateWest.setActionCommand("ROTATE_WEST");
		rotateWest.addActionListener(this);
		rotateMenu.add(rotateWest);
		
		//Send to back
		sendToBackMenuItem = new JMenuItem("Send to back");
		sendToBackMenuItem.setActionCommand("SEND_TO_BACK");
		sendToBackMenuItem.addActionListener(this);
	    popupMenu.add(sendToBackMenuItem);
		
	}
	
	/**
	 * Gathers properties relating to the instance given as argument and
	 * sends them to VPGui to be displayed.
	 * @param vc	the VisualComponent whose properties are displayed
	 */
	private void displayInstanceProperties(VisualComponent vc){
		
		Statement stm;
		Individual ind = model.getIndividual(vc.getURI());
		StmtIterator propIter = ind.listProperties();				
		Vector<String[]> propValuePairs = new Vector<String[]>();
		
		propValuePairs.add(new String[]{"URI",vc.getURI()});
		
		while(propIter.hasNext()){			
			
			stm = propIter.nextStatement();			
			propValuePairs.add(new String[]{
					stm.getPredicate().toString(),
					stm.getObject().toString()});
			
			if(stm.getPredicate().toString().equals(VPUtils.ICONSET_PROP.getURI())){				
				Individual iconSetInd = model.getIndividual(stm.getObject().toString());
				stm = iconSetInd.getProperty(VPUtils.ICONGRIDWIDTH_PROP);
				propValuePairs.add(new String[]{
						stm.getPredicate().toString(),
						stm.getObject().toString()});
				stm = iconSetInd.getProperty(VPUtils.ICONGRIDHEIGHT_PROP);
				propValuePairs.add(new String[]{
						stm.getPredicate().toString(),
						stm.getObject().toString()});				
			}
			
		}
			
		Object data[][] = new Object[propValuePairs.size()][2];
		for(int i=0; i<propValuePairs.size(); i++){			
			data[i][0] = propValuePairs.elementAt(i)[0];
			data[i][1] = propValuePairs.elementAt(i)[1];			
		}
		
		//vpGui.displayInstanceInfo(data);
		
	}

	public void actionPerformed(ActionEvent ae) {
		
		//vpGui.printMessage(ae.getActionCommand(), null);
		
		/*if(ae.getActionCommand().equals("INSTANCE_INFORMATION")){
			displayInstanceProperties((VisualComponent)selected);
		}else if(ae.getActionCommand().equals("DELETE")){
			String URI = ((VisualComponent)selected).getURI();
			if(!vpController.deleteVisualComponent(URI))
				vpGui.printMessage("Could not delete individual: "+URI, null);			
		}else if(ae.getActionCommand().equals("MOVE")){
			moveEnabled = true;
			layeredPane.setCursor(new Cursor(Cursor.MOVE_CURSOR));			
			layeredPane.setLayer(selected, 3);			
		}else if(ae.getActionCommand().equals("SEND_TO_BACK")){			
			layeredPane.setPosition(selected, -1);
			selected.repaint();
		}else if(ae.getActionCommand().equals("ROTATE_SOUTH")){
			VPItem focusItem = (VPItem)selected;
			if(!vpController.setItemDirection(focusItem.getURI(), 0))
				vpGui.printMessage("Could not rotate item: "+focusItem.getURI(), null);							
		}else if(ae.getActionCommand().equals("ROTATE_WEST")){
			VPItem focusItem = (VPItem)selected;
			if(!vpController.setItemDirection(focusItem.getURI(), 1))
				vpGui.printMessage("Could not rotate item: "+focusItem.getURI(), null);			
		}else if(ae.getActionCommand().equals("ROTATE_NORTH")){
			VPItem focusItem = (VPItem)selected;
			if(!vpController.setItemDirection(focusItem.getURI(), 2))
				vpGui.printMessage("Could not rotate item: "+focusItem.getURI(), null);			
		}else if(ae.getActionCommand().equals("ROTATE_EAST")){
			VPItem focusItem = (VPItem)selected;
			if(!vpController.setItemDirection(focusItem.getURI(), 3))
				vpGui.printMessage("Could not rotate item: "+focusItem.getURI(), null);			
		}*/
		
		
	}
	
	/**
	 * Selects new focus component.
	 */
	public void mouseClicked(MouseEvent me) {
		
		if(moveEnabled)return;
		
		VisualComponent vc;
				
		//previous selection off
		if(selected != null){
			vc = (VisualComponent)selected;			
			vc.setSelected(false);			
		}
		
		//acquire new selected	
		//Point p = VPUtils.isometricPointToGridPoint(me.getX(), me.getY(), origo);		
		selected = SwingUtilities.getDeepestComponentAt(layeredPane, me.getX(), me.getY());
				
		//new selection
		if(selected instanceof VisualComponent){
			vc = (VisualComponent)selected;
			vc.setSelected(true);			
		}else
			selected = null;		
		
		//Signal VPGui about selection
		//vpGui.setSelectedInstance((VisualComponent)selected, layeredPane);
		
		//Show popup menu
		if(selected != null && me.getButton() == MouseEvent.BUTTON3){			
			if(selected instanceof VPArea){
				rotateMenu.setEnabled(false);
				sendToBackMenuItem.setEnabled(false);
				moveMenuItem.setEnabled(false);
			}
			else{
				rotateMenu.setEnabled(true);	
				sendToBackMenuItem.setEnabled(true);
				moveMenuItem.setEnabled(true);
			}
			popupMenu.show(me.getComponent(),me.getX(), me.getY());				
		}
		
	}

	public void mouseEntered(MouseEvent me) {
		// TODO Auto-generated method stub
		
	}

	public void mouseExited(MouseEvent me) {
		//vpGui.updateCoordinateArea(0,0);		
	}

	public void mousePressed(MouseEvent me) {		
		
		Container parent = layeredPane.getParent();
		
		if(parent instanceof JViewport){
	 		grabPoint = MouseInfo.getPointerInfo().getLocation();
	 		viewportStartLocation = ((JViewport)parent).getViewPosition();
		}
		
	}

	public void mouseReleased(MouseEvent me) {
		
		if(moveEnabled){
			
			String URI = ((VisualComponent)selected).getURI();		
			String parentURI = null;	
			Point newLocation;
			
			moveEnabled = false;
			layeredPane.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));	
			
			//Focus is removed from the IsometricView to use the
			//SwingUtilities.getDeepestComponentAt() correctly			
			layeredPane.remove(selected);		
			Component topmost = SwingUtilities.getDeepestComponentAt(layeredPane, me.getX(), me.getY());
						
			//Get the URI of the topmost space
			if(topmost instanceof VisualComponent){				
				VisualComponent vc = (VisualComponent)topmost;				
				if(topmost instanceof VPItem){
					parentURI = vc.getParentAreaURI();					
				}else
					parentURI = vc.getURI();				
			}			
			
			Point parentAbsoluteGridLocation;			
			Individual ind = model.getIndividual(parentURI);
			parentAbsoluteGridLocation = VPUtils.getAbsoluteGridLocation(ind);			
						
			Point dropLocation = VPUtils.isometricPointToGridPoint(me.getX(), me.getY(), origo);			
			newLocation = new Point(dropLocation.x-parentAbsoluteGridLocation.x,
					dropLocation.y-parentAbsoluteGridLocation.y);
						
			/*if(!vpController.moveVisualComponent(URI, parentURI, newLocation.x, newLocation.y)){
				vpGui.printMessage("Illegal move: "+URI, null);
				layeredPane.refresh();
			}		*/	
						
		}
		
	}

	public void mouseDragged(MouseEvent me) {
		
		Point p = VPUtils.isometricPointToGridPoint(me.getX(), me.getY(), origo);
		//vpGui.updateCoordinateArea(p.x, p.y);
		
		if(moveEnabled){
			
			IsometricItem isoItem = (IsometricItem)selected;
			Point gridPoint = VPUtils.isometricPointToGridPoint(me.getX(), me.getY(), origo);
						
			Point isoPoint = VPUtils.gridPointToIsometricPoint(gridPoint.x, gridPoint.y, origo);
			isoItem.setLocation(isoPoint.x-isoItem.getXOffset(),isoPoint.y-isoItem.getYOffset());
			
		}				
		
		//View port move block
		if(!moveEnabled && layeredPane.getParent() instanceof JViewport){
						
			JViewport parentViewport = (JViewport)layeredPane.getParent();			
			
			Point cursorLocation = MouseInfo.getPointerInfo().getLocation();
			Point cursorShift = new Point(cursorLocation.x - grabPoint.x,
					cursorLocation.y - grabPoint.y);
						 
			Point newViewportLocation = new Point(
					viewportStartLocation.x - cursorShift.x,
					viewportStartLocation.y - cursorShift.y);
			
			if(newViewportLocation.x < 0) newViewportLocation.x = 0;
			if(newViewportLocation.x > layeredPane.getWidth() - parentViewport.getWidth()) 
				newViewportLocation.x = layeredPane.getWidth() - parentViewport.getWidth();
			if(newViewportLocation.y < 0) newViewportLocation.y = 0;
			if(newViewportLocation.y > layeredPane.getHeight() - parentViewport.getHeight()) 
				newViewportLocation.y = layeredPane.getHeight() - parentViewport.getHeight();
			
			parentViewport.setViewPosition(newViewportLocation);			 
			
		}
		
	}

	/**
	 * Updates highligh.
	 */
	public void mouseMoved(MouseEvent me) {
		
		Point p = VPUtils.isometricPointToGridPoint(me.getX(), me.getY(), origo);
		//vpGui.updateCoordinateArea(p.x, p.y);		
		
		if(moveEnabled)return;				
		
		//acquire new highlight
		Component newHighlight = SwingUtilities.getDeepestComponentAt(layeredPane, me.getX(), me.getY());
				
		//previous highlight off
		if(!newHighlight.equals(highlighted) && highlighted instanceof VisualComponent)
			((VisualComponent)highlighted).setHighlight(false);
				
		//set current highlight on
		if(newHighlight instanceof VisualComponent)
			((VisualComponent)newHighlight).setHighlight(true);
		
		highlighted = newHighlight;				
		
	}	
	
}

